
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: GetExpansionLevel, GetMaxLevelForExpansionLevel, LibStub, next, select, string

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(Mixin(_G, {
	[(...)] = Mixin(select(2, ...),{
		[1] = LibStub("AceAddon-3.0"):NewAddon(..., "AceEvent-3.0"),
		[2] = LibStub("SixBorder-1.0"):CreateFactory({}),
		[3] = {},
		[4] = LibStub("AceEvent-3.0"):Embed({}),
	}),
})[(...)])

-------------------------------------------------------------------------------
-- AddOn
-------------------------------------------------------------------------------

function A:OnInitialize()
	D:CreateDatabase()
	self:UpdateModuleConfigs()
	self:UpdateResources()
	self:UpdateBorder()
end

function A:OnEnable()
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("PLAYER_LEVEL_CHANGED")
	self:RegisterEvent("UPDATE_EXPANSION_LEVEL")
	self:RegisterMessage("SIXUI_CONFIG_CHANGED")
	self:RegisterMessage("SIXUI_PROFILE_CHANGED")
	self:UpdateModuleStates()
end

-- Generate a SixBorder config from the database.
function A:GetBorderConfig()
	return {
		accentBackdropInfo = {
			edgeFile = [[Interface\Buttons\WHITE8x8]],
			tile = true,
			tileSize = 8,
			tileEdge = true,
			edgeSize = D.profile.border.accent.width,
		},
		accentAnchorOffset = D.profile.border.accent.anchorOffset,
		accentBackdropBorderColor = {C.BORDER_ACCENT_COLOR:GetRGBA()},
		backgroundAnchorOffset = D.profile.border.background.inset,
		backgroundBackdropColor = {C.BORDER_BACKGROUND_COLOR:GetRGBA()},
		strokeBackdropInfo = {
			edgeFile = [[Interface\Buttons\WHITE8x8]],
			tile = true,
			tileSize = 8,
			tileEdge = true,
			edgeSize = D.profile.border.stroke.width,
		},
		strokeAnchorOffset = D.profile.border.stroke.anchorOffset,
		strokeBackdropBorderColor = {C.BORDER_STROKE_COLOR:GetRGBA()},
	}
end

-- Update SixUI's border to reflect the database.
function A:UpdateBorder()
	B:SetConfig(self:GetBorderConfig())
end

-- Update SixUI's resources to reflect the database.
function A:UpdateResources()
	for key in next, C.CONFIG_KEY_TO_RESOURCE do
		local c, g, k = string.split(".", key)
		local v = D.profile[c][g][k]

		if v then
			if (c == "border" and k == "color") or c == "color" then
				self:UpdateResource(key, v.r, v.g, v.b, v.a)
			elseif c == "font" then
				self:UpdateResource(key, v)
			end
		end
	end
end

function A:UpdateResource(key, ...)
	local c, g, k = string.split(".", key)

	if (c == "border" and k == "color") or c == "color" then
		if C.CONFIG_KEY_TO_RESOURCE[key] then
			C.CONFIG_KEY_TO_RESOURCE[key]:SetRGBA(...)
		end
	end

	if c == "border" then
		self:UpdateBorder()
	end

	if c == "font" and (g == "glyphFontSize" or g == "textFontSize") then
		local fontObject = C.CONFIG_KEY_TO_RESOURCE[key]

		if fontObject then
			local fontFile, _, fontFlags = fontObject:GetFont()

			fontObject:SetFont(fontFile, select(1, ...), fontFlags)
		end
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function A:ADDON_LOADED(event, addOnName)
	for _, moduleName in self:IterateModulesWithDependencies(addOnName) do
		self:UpdateModuleState(moduleName)
	end
end

function A:PLAYER_LEVEL_CHANGED(event, oldLevel, newLevel, real)
	if real then
		C.PLAYER_LEVEL = newLevel
	end
end

function A:SIXUI_CONFIG_CHANGED(event, key, ...)
	self:UpdateResource(key, ...)
end

function A:SIXUI_PROFILE_CHANGED()
	self:UpdateResources()
	self:UpdateBorder()
end

function A:UPDATE_EXPANSION_LEVEL()
	C.PLAYER_MAX_LEVEL = GetMaxLevelForExpansionLevel(GetExpansionLevel())
end
