
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: C_AddOns, NORMAL_FONT_COLOR, rawset, select, type

local IsAltKeyDown = IsAltKeyDown
local IsControlKeyDown = IsControlKeyDown
local IsShiftKeyDown = IsShiftKeyDown

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local Social = LibStub:GetLibrary("SixSocial-1.0", true)

-------------------------------------------------------------------------------
-- SixUI
-------------------------------------------------------------------------------

C.VERSION = C_AddOns.GetAddOnMetadata(..., "Version")
C.COMMAND = "sixui"

C.IS_RETAIL = (WOW_PROJECT_ID == WOW_PROJECT_MAINLINE)

-------------------------------------------------------------------------------
-- Layout
-------------------------------------------------------------------------------

C.BORDER_PADDING = 2

C.CHATFRAME_INSET = 10

C.MINIMAP_BUTTON_INSET = 15

C.TOOLTIP_CELL_MARGIN_H = 6
C.TOOLTIP_CELL_MARGIN_V = 3
C.TOOLTIP_SEPARATOR_HEIGHT = 1

-------------------------------------------------------------------------------
-- Resources
-------------------------------------------------------------------------------

C.BORDER_ACCENT_COLOR = CreateColor(1, 1, 1, 0.2)
C.BORDER_BACKGROUND_COLOR = CreateColor(0.078, 0.070, 0.082, 0.9)
C.BORDER_STROKE_COLOR = CreateColor(0, 0, 0, 1)

C.CURSE_DEBUFF_COLOR = CreateColor(0.6, 0, 1, 1)
C.DISEASE_DEBUFF_COLOR = CreateColor(0.6, 4, 0, 1)
C.MAGIC_DEBUFF_COLOR = CreateColor(0.2, 6, 1, 1)
C.OTHER_DEBUFF_COLOR = CreateColor(0.8, 0, 0, 1)
C.POISON_DEBUFF_COLOR = CreateColor(0, 6, 0, 1)

C.ALLIANCE_FACTION_COLOR = CreateColor(0, 0.5, 0.8)
C.HORDE_FACTION_COLOR = CreateColor(0.8, 0.15, 0)

C.GOLD_RATING_COLOR = CreateColor(0.78, 0.69, 0.21)
C.SILVER_RATING_COLOR = CreateColor(0.84, 0.84, 0.84)
C.BRONZE_RATING_COLOR = CreateColor(0.68, 0.54, 0.34)
C.UNTIMED_RATING_COLOR = CreateColor(0.8, 0, 0)

C.HOSTILE_REACTION_COLOR = CreateColor(0.9, 0, 0)
C.NEUTRAL_REACTION_COLOR = CreateColor(0.93, 0.93, 0)
C.FRIENDLY_REACTION_COLOR = CreateColor(0.2, 0.9, 0.2)

C.EXCEPTIONALLY_HOSTILE_REPUTATION_COLOR = CreateColor(0.8, 0.3, 0.22)
C.VERY_HOSTILE_REPUTATION_COLOR = CreateColor(0.8, 0.3, 0.22)
C.HOSTILE_REPUTATION_COLOR = CreateColor(0.75, 0.27, 0)
C.NEUTRAL_REPUTATION_COLOR = CreateColor(0.9, 0.7, 0)
C.FRIENDLY_REPUTATION_COLOR = CreateColor(0, 0.6, 0.1)
C.VERY_FRIENDLY_REPUTATION_COLOR = CreateColor(0, 0.6, 0.1)
C.EXCEPTIONALLY_FRIENDLY_REPUTATION_COLOR = CreateColor(0, 0.6, 0.1)
C.EXALTED_REPUTATION_COLOR = CreateColor(0, 0.6, 0.1)

C.HONOR_RESOURCE_COLOR = CreateColor(1, 0.24, 0)
C.RENOWN_RESOURCE_COLOR = CreateColor(0, 0.75, 0.95)
C.RESTED_XP_RESOURCE_COLOR = CreateColor(0, 0.39, 0.88)
C.VALOR_RESOURCE_COLOR = CreateColor(0.68, 0.47, 1)
C.XP_RESOURCE_COLOR = CreateColor(0.58, 0, 0.55)

C.FALLBACK_UTILITY_COLOR = CreateColor(1, 1, 1)
C.GUILD_UTILITY_COLOR = CreateColor(0.23, 0.88, 0.25)
C.HIGHLIGHT_UTILITY_COLOR = CreateColor(0.659, 0.843, 1)

C.CLASS_TO_COLOR = {}

for _, list in next, {LOCALIZED_CLASS_NAMES_MALE, LOCALIZED_CLASS_NAMES_FEMALE} do
	for token, name in next, list do
		local value = C_ClassColor.GetClassColor(token)
		local color = CreateColor(value.r, value.g, value.b)

		C.CLASS_TO_COLOR[name] = color
		C.CLASS_TO_COLOR[token] = color
	end
end

-------------------------------------------------------------------------------
-- Player
-------------------------------------------------------------------------------

C.PLAYER_NAME = UnitName("player")
C.PLAYER_REALM_NAME = GetRealmName():gsub("[%s%-]", "")
C.PLAYER_REGION_ID = GetCurrentRegion()
C.PLAYER_CLASS = select(2, UnitClass("player"))
C.PLAYER_COLOR = C.PLAYER_CLASS == "PRIEST" and C.HIGHLIGHT_UTILITY_COLOR or C.CLASS_TO_COLOR[C.PLAYER_CLASS]
C.PLAYER_LEVEL = UnitLevel("player")
C.PLAYER_MAX_LEVEL = GetMaxLevelForExpansionLevel(GetExpansionLevel())
C.PLAYER_SEX = UnitSex("player")

-------------------------------------------------------------------------------
-- Fonts
-------------------------------------------------------------------------------

C.TEXT_FONT_NAME = "Titillium"
C.TEXT_FONT_PATH = [[Interface\AddOns\SixUI\Resources\Fonts\Titillium.ttf]]
C.GLYPH_FONT_PATH = [[Interface\AddOns\SixUI\Resources\Fonts\Font Awesome 5 Free-Solid-900.otf]]

C.GLYPH_LARGE = CreateFont("SixUI Glyph Large")
C.GLYPH_LARGE:SetFont(C.GLYPH_FONT_PATH, 16, "")
C.GLYPH_LARGE:SetTextColor(1, 1, 1)

C.GLYPH_REGULAR = CreateFont("SixUI Glyph Regular")
C.GLYPH_REGULAR:SetFont(C.GLYPH_FONT_PATH, 12, "")
C.GLYPH_REGULAR:SetTextColor(C.PLAYER_COLOR:GetRGB())
C.GLYPH_REGULAR:SetShadowOffset(0, 0)

C.GLYPH_SMALL = CreateFont("SixUI Glyph Small")
C.GLYPH_SMALL:SetFont(C.GLYPH_FONT_PATH, 7, "")
C.GLYPH_SMALL:SetTextColor(NORMAL_FONT_COLOR:GetRGB())
C.GLYPH_SMALL:SetShadowOffset(0, 0)

C.TEXT_LARGE = CreateFont("SixUI Text Large")
C.TEXT_LARGE:SetFont(C.TEXT_FONT_PATH, 14, "")
C.TEXT_LARGE:SetTextColor(1, 1, 1)

C.TEXT_MEDIUM = CreateFont("SixUI Text Medium")
C.TEXT_MEDIUM:SetFont(C.TEXT_FONT_PATH, 13, "")
C.TEXT_MEDIUM:SetTextColor(1, 1, 1)

C.TEXT_REGULAR = CreateFont("SixUI Text Regular")
C.TEXT_REGULAR:SetFont(C.TEXT_FONT_PATH, 12, "")
C.TEXT_REGULAR:SetTextColor(1, 1, 1)

-------------------------------------------------------------------------------
-- Enumerators
-------------------------------------------------------------------------------

C.Enum = {}

C.Enum.AutoCollapseBuffs = {
	Always = 1,
	WhileInAnInstance = 2,
	Never = 3,
}

C.Enum.BossStatus = {
	Alive = 1,
	AliveSaved = 2,
	Dead = 3,
	DeadUnsaved = 4,
}

-- A precomputed table mapping boss statuses based on conditions: isInScenario, isDefeated, and isLocked.
-- Access pattern: C.Enum.BossStatus[isInScenario][isDefeated][isLocked] to get the boss's status.
C.Enum.BossStatusLookup = {
	[true] = {
		[true] = {
			[true] = C.Enum.BossStatus.Dead,
			[false] = C.Enum.BossStatus.Dead,
		},
		[false] = {
			[true] = C.Enum.BossStatus.Alive,
			[false] = C.Enum.BossStatus.Alive,
		},
	},
	[false] = {
		[true] = {
			[true] = C.Enum.BossStatus.Dead,
			[false] = C.Enum.BossStatus.DeadUnsaved,
		},
		[false] = {
			[true] = C.Enum.BossStatus.AliveSaved,
			[false] = C.Enum.BossStatus.Alive,
		},
	},
}

C.Enum.BossStatusColor = {
	[C.Enum.BossStatus.Alive] = GREEN_FONT_COLOR,
	[C.Enum.BossStatus.AliveSaved] = YELLOW_FONT_COLOR,
	[C.Enum.BossStatus.Dead] = RED_FONT_COLOR,
	[C.Enum.BossStatus.DeadUnsaved] = LIGHTGRAY_FONT_COLOR,
}

C.Enum.FriendColumn = {
	status = STATUS,
	icon = EMBLEM_SYMBOL,
	friend = FRIEND,
	level = LEVEL,
	name = NAME,
	game = GAME,
	note = LABEL_NOTE,
}

C.Enum.Glyph = {
	Bars = "",
	Bolt = "",
	Bomb = "",
	Book = "",
	Bookmark = "",
	Bug = "",
	Bullhorn = "",
	Bullseye = "",
	CalendarAlt = "",
	Camera = "",
	ChartBar = "",
	Check = "",
	Circle = "",
	CircleExclamation = "",
	CircleQuestion = "",
	Clipboard = "",
	Clock = "",
	Cog = "",
	Coins = "",
	Comment = "",
	CommentSlash = "",
	Compass = "",
	Crosshairs = "",
	Crown = "",
	Dice = "",
	Droplet = "",
	Envelope = "",
	FaceSmile = "",
	Fire = "",
	Flag = "",
	Flask = "",
	Gem = "",
	Gift = "",
	Heart = "",
	Hourglass = "",
	Image = "",
	Info = "",
	InfoCircle = "",
	Key = "",
	Leaf = "",
	Lightbulb = "",
	List = "",
	LocationDot = "",
	Lock = "",
	Magnet = "",
	Map = "",
	Microphone = "",
	Moon = "",
	MugSaucer = "",
	Music = "",
	Paw = "",
	PuzzlePiece = "",
	Receipt = "",
	Search = "",
	ShieldHalved = "",
	Signal = "",
	Skull = "",
	Square = "",
	SquareCheck = "",
	Star = "",
	Stopwatch = "",
	Tree = "",
	Trophy = "",
	UserFriends = "",
	Users = "",
	UserSlash = "",
	WandMagic = "",
	Wifi = "",
}

C.Enum.GuildColumn = {
	status = STATUS,
	level = LEVEL,
	name = NAME,
	zone = ZONE,
	rank = RANK,
	note = LABEL_NOTE,
	mythicRating = DUNGEON_SCORE,
}

C.Enum.ModifierKey = {
	Alt = 0,
	AltShift = 1,
	Any = 2,
	Control = 3,
	ControlAlt = 4,
	ControlAltShift = 5,
	ControlShift = 6,
	None = 7,
	Shift = 8,
}

C.Enum.ModifierKeyFunction = setmetatable({
	[C.Enum.ModifierKey.Alt] = IsAltKeyDown,
	[C.Enum.ModifierKey.AltShift] = function () return IsAltKeyDown() and IsShiftKeyDown() end,
	[C.Enum.ModifierKey.Any] = IsModifierKeyDown,
	[C.Enum.ModifierKey.Control] = IsControlKeyDown,
	[C.Enum.ModifierKey.ControlAlt] = function () return IsControlKeyDown() and IsAltKeyDown() end,
	[C.Enum.ModifierKey.ControlAltShift] = function () return IsControlKeyDown() and IsAltKeyDown() and IsShiftKeyDown() end,
	[C.Enum.ModifierKey.ControlShift] = function () return IsControlKeyDown() and IsShiftKeyDown() end,
	[C.Enum.ModifierKey.Shift] = IsShiftKeyDown,
}, {
	__index = function (t, k) return function () return false end end,
})

C.Enum.ModifierKeyLabel = {
	[C.Enum.ModifierKey.Alt] = "Alt",
	[C.Enum.ModifierKey.AltShift] = "Alt + Shift",
	[C.Enum.ModifierKey.Any] = "Any",
	[C.Enum.ModifierKey.Control] = "Control",
	[C.Enum.ModifierKey.ControlAlt] = "Control + Alt",
	[C.Enum.ModifierKey.ControlAltShift] = "Control + Alt + Shift",
	[C.Enum.ModifierKey.ControlShift] = "Control + Shift",
	[C.Enum.ModifierKey.None] = "None",
	[C.Enum.ModifierKey.Shift] = "Shift",
}

C.Enum.ModifierKeyStateText = {
	[C.Enum.ModifierKey.Alt] = "mod:alt",
	[C.Enum.ModifierKey.AltShift] = "mod:alt,mod:shift",
	[C.Enum.ModifierKey.Any] = "mod",
	[C.Enum.ModifierKey.Control] = "mod:ctrl",
	[C.Enum.ModifierKey.ControlAlt] = "mod:ctrl,mod:alt",
	[C.Enum.ModifierKey.ControlAltShift] = "mod:ctrl,mod:alt,mod:shift",
	[C.Enum.ModifierKey.ControlShift] = "mod:ctrl,mod:shift",
	[C.Enum.ModifierKey.None] = "mod,nomod",
	[C.Enum.ModifierKey.Shift] = "mod:shift",
}

C.Enum.MouseButton = {
	Any = 1,
	Left = 2,
	Middle = 3,
	None = 4,
	Right = 5,
}

C.Enum.Skada = {}

C.Enum.SkadaColumnType = {
	Always = 1,
	NoClutter = 2,
	Clutter = 3,
	Never = 4,
}

C.Enum.SkadaColumnTypeLabel = {
	[C.Enum.SkadaColumnType.Always] = "Always Show",
	[C.Enum.SkadaColumnType.NoClutter] = "Show by Default",
	[C.Enum.SkadaColumnType.Clutter] = "Hide by Default",
	[C.Enum.SkadaColumnType.Never] = "Always Hide",
}

C.Enum.StatusColor = not Social and {} or {
	[Social.Enum.Status.Away] = YELLOW_FONT_COLOR,
	[Social.Enum.Status.Busy] = RED_FONT_COLOR,
	[Social.Enum.Status.Mobile] = YELLOW_FONT_COLOR,
	[Social.Enum.Status.Offline] = GRAY_FONT_COLOR,
	[Social.Enum.Status.Online] = GREEN_FONT_COLOR,
}

-------------------------------------------------------------------------------
-- Maps
-------------------------------------------------------------------------------

C.CHAT_TYPE_ID_TO_KEY = {}

for _, key in next, {
	"SYSTEM",
	"SAY",
	"PARTY",
	"RAID",
	"GUILD",
	"OFFICER",
	"YELL",
	"WHISPER",
	"SMART_WHISPER",
	"WHISPER_INFORM",
	"REPLY",
	"EMOTE",
	"TEXT_EMOTE",
	"MONSTER_SAY",
	"MONSTER_PARTY",
	"MONSTER_YELL",
	"MONSTER_WHISPER",
	"MONSTER_EMOTE",
	"CHANNEL",
	"CHANNEL_JOIN",
	"CHANNEL_LEAVE",
	"CHANNEL_LIST",
	"CHANNEL_NOTICE",
	"CHANNEL_NOTICE_USER",
	"TARGETICONS",
	"AFK",
	"DND",
	"IGNORED",
	"SKILL",
	"LOOT",
	"CURRENCY",
	"MONEY",
	"OPENING",
	"TRADESKILLS",
	"PET_INFO",
	"COMBAT_MISC_INFO",
	"COMBAT_XP_GAIN",
	"COMBAT_HONOR_GAIN",
	"COMBAT_FACTION_CHANGE",
	"BG_SYSTEM_NEUTRAL",
	"BG_SYSTEM_ALLIANCE",
	"BG_SYSTEM_HORDE",
	"RAID_LEADER",
	"RAID_WARNING",
	"RAID_BOSS_WHISPER",
	"RAID_BOSS_EMOTE",
	"QUEST_BOSS_EMOTE",
	"FILTERED",
	"INSTANCE_CHAT",
	"INSTANCE_CHAT_LEADER",
	"RESTRICTED",
	"CHANNEL1",
	"CHANNEL2",
	"CHANNEL3",
	"CHANNEL4",
	"CHANNEL5",
	"CHANNEL6",
	"CHANNEL7",
	"CHANNEL8",
	"CHANNEL9",
	"CHANNEL10",
	"CHANNEL11",
	"CHANNEL12",
	"CHANNEL13",
	"CHANNEL14",
	"CHANNEL15",
	"CHANNEL16",
	"CHANNEL17",
	"CHANNEL18",
	"CHANNEL19",
	"CHANNEL20",
	"ACHIEVEMENT",
	"GUILD_ACHIEVEMENT",
	"PARTY_LEADER",
	"BN_WHISPER",
	"BN_WHISPER_INFORM",
	"BN_ALERT",
	"BN_BROADCAST",
	"BN_BROADCAST_INFORM",
	"BN_INLINE_TOAST_ALERT",
	"BN_INLINE_TOAST_BROADCAST",
	"BN_INLINE_TOAST_BROADCAST_INFORM",
	"BN_WHISPER_PLAYER_OFFLINE",
	"PET_BATTLE_COMBAT_LOG",
	"PET_BATTLE_INFO",
	"GUILD_ITEM_LOOTED",
	"COMMUNITIES_CHANNEL",
	"VOICE_TEXT",
} do
	if ChatTypeInfo[key] and ChatTypeInfo[key].id then
		C.CHAT_TYPE_ID_TO_KEY[ChatTypeInfo[key].id] = key
	end
end

C.CONFIG_KEY_TO_RESOURCE = {
	["border.accent.color"] = C.BORDER_ACCENT_COLOR,
	["border.background.color"] = C.BORDER_BACKGROUND_COLOR,
	["border.stroke.color"] = C.BORDER_STROKE_COLOR,
	["color.debuff.curse"] = C.CURSE_DEBUFF_COLOR,
	["color.debuff.disease"] = C.DISEASE_DEBUFF_COLOR,
	["color.debuff.magic"] = C.MAGIC_DEBUFF_COLOR,
	["color.debuff.other"] = C.OTHER_DEBUFF_COLOR,
	["color.debuff.poison"] = C.POISON_DEBUFF_COLOR,
	["color.faction.alliance"] = C.ALLIANCE_FACTION_COLOR,
	["color.faction.horde"] = C.HORDE_FACTION_COLOR,
	["color.rating.bronze"] = C.BRONZE_RATING_COLOR,
	["color.rating.gold"] = C.GOLD_RATING_COLOR,
	["color.rating.silver"] = C.SILVER_RATING_COLOR,
	["color.rating.untimed"] = C.UNTIMED_RATING_COLOR,
	["color.reaction.friendly"] = C.FRIENDLY_REACTION_COLOR,
	["color.reaction.hostile"] = C.HOSTILE_REACTION_COLOR,
	["color.reaction.neutral"] = C.NEUTRAL_REACTION_COLOR,
	["color.reputation.exceptionallyHostile"] = C.EXCEPTIONALLY_HOSTILE_REPUTATION_COLOR,
	["color.reputation.veryHostile"] = C.VERY_HOSTILE_REPUTATION_COLOR,
	["color.reputation.hostile"] = C.HOSTILE_REPUTATION_COLOR,
	["color.reputation.neutral"] = C.NEUTRAL_REPUTATION_COLOR,
	["color.reputation.friendly"] = C.FRIENDLY_REPUTATION_COLOR,
	["color.reputation.veryFriendly"] = C.VERY_FRIENDLY_REPUTATION_COLOR,
	["color.reputation.exceptionallyFriendly"] = C.EXCEPTIONALLY_FRIENDLY_REPUTATION_COLOR,
	["color.reputation.exalted"] = C.EXALTED_REPUTATION_COLOR,
	["color.resource.honor"] = C.HONOR_RESOURCE_COLOR,
	["color.resource.renown"] = C.RENOWN_RESOURCE_COLOR,
	["color.resource.restedXP"] = C.RESTED_XP_RESOURCE_COLOR,
	["color.resource.valor"] = C.VALOR_RESOURCE_COLOR,
	["color.resource.xp"] = C.XP_RESOURCE_COLOR,
	["color.utility.fallback"] = C.FALLBACK_UTILITY_COLOR,
	["color.utility.guild"] = C.GUILD_UTILITY_COLOR,
	["color.utility.highlight"] = C.HIGHLIGHT_UTILITY_COLOR,
	["font.glyphFontSize.large"] = C.GLYPH_LARGE,
	["font.glyphFontSize.regular"] = C.GLYPH_REGULAR,
	["font.glyphFontSize.small"] = C.GLYPH_SMALL,
	["font.textFontSize.large"] = C.TEXT_LARGE,
	["font.textFontSize.medium"] = C.TEXT_MEDIUM,
	["font.textFontSize.regular"] = C.TEXT_REGULAR,
}

C.DEBUFF_TYPE_TO_COLOR = setmetatable({
	Magic = C.MAGIC_DEBUFF_COLOR,
	Curse = C.CURSE_DEBUFF_COLOR,
	Disease = C.DISEASE_DEBUFF_COLOR,
	Poison = C.POISON_DEBUFF_COLOR,
}, {
	__index = function (t, k) return C.OTHER_DEBUFF_COLOR end,
})

C.EVENT_TO_UNIT = {
	PLAYER_TARGET_CHANGED = "target",
	PLAYER_FOCUS_CHANGED = "focus",
}

C.FACTION_TO_COLOR = {
	Alliance = C.ALLIANCE_FACTION_COLOR,
	Horde = C.HORDE_FACTION_COLOR,
}

C.PVP_TYPE_TO_COLOR = setmetatable({
	sanctuary = {0.41, 0.8, 0.94},
	arena = {1.0, 0.1, 0.1},
	friendly = {1.0, 1.0, 1.0},
	hostile = {1.0, 0.1, 0.1},
	contested = {1.0, 0.7, 0.0},
}, {
	__index = function (t, k) return {NORMAL_FONT_COLOR:GetRGB()} end,
})

C.REACTION_TO_COLOR = {
	C.HOSTILE_REACTION_COLOR, -- Hated
	C.HOSTILE_REACTION_COLOR, -- Hostile
	C.NEUTRAL_REACTION_COLOR, -- Unfriendly
	C.NEUTRAL_REACTION_COLOR, -- Neutral
	C.FRIENDLY_REACTION_COLOR, -- Friendly
	C.FRIENDLY_REACTION_COLOR, -- Honored
	C.FRIENDLY_REACTION_COLOR, -- Revered
	C.FRIENDLY_REACTION_COLOR, -- Exalted
}

C.REPUTATION_TO_COLOR = {
	C.EXCEPTIONALLY_HOSTILE_REPUTATION_COLOR,
	C.VERY_HOSTILE_REPUTATION_COLOR,
	C.HOSTILE_REPUTATION_COLOR,
	C.NEUTRAL_REPUTATION_COLOR,
	C.FRIENDLY_REPUTATION_COLOR,
	C.VERY_FRIENDLY_REPUTATION_COLOR,
	C.EXCEPTIONALLY_FRIENDLY_REPUTATION_COLOR,
	C.EXALTED_REPUTATION_COLOR,
}

-------------------------------------------------------------------------------
-- Setup
-------------------------------------------------------------------------------

C.Enum.Layout = {
	Standard = "Standard",
	Minimal = "Minimal",
	Damage = "Damage",
}

C.Enum.Profile = {
	Standard = "SixUI",
	Minimal = "SixUI Minimal",
	Damage = "SixUI Damage",
}

C.Enum.LayoutToProfile = {
	[C.Enum.Layout.Standard] = C.Enum.Profile.Standard,
	[C.Enum.Layout.Minimal] = C.Enum.Profile.Minimal,
	[C.Enum.Layout.Damage] = C.Enum.Profile.Damage,
}

C.Enum.ProfileToLayout = {
	[C.Enum.Profile.Standard] = C.Enum.Layout.Standard,
	[C.Enum.Profile.Minimal] = C.Enum.Layout.Minimal,
	[C.Enum.Profile.Damage] = C.Enum.Layout.Damage,
}

C.SetupMixin = {
	BACKUP_PROFILE_NAME = "SixUI Backup",
	CreateProfileFromString = function (self, profileName, profileStrin, successFn) error("CreateProfileFromString is not implemented") end,
	DeleteProfile = function (self, profileName) error("DeleteProfile is not implemented") end,
	HasProfile = function (self, profileName) error("HasProfile is not implemented") end,
	GetActiveProfile = function (self, profileName) error("GetActiveProfile is not implemented") end,
	IsActiveProfile = function (self, profileName) return self:GetActiveProfile() == profileName end,
	IsControlledProfile = function (self) return A.table.any(C.Enum.Profile, function (p) return self:IsActiveProfile(p) end) end,
	SetActiveProfile = function (self, profileName) error("SetActiveProfile is not implemented") end,
}

C.AceSetupMixin = Mixin({}, C.SetupMixin, {
	DeleteProfile = function (self, profileName)
		if self:IsActiveProfile(profileName) then
			self:SetActiveProfile(self.BACKUP_PROFILE_NAME)
		end

		self:GetDatabase():DeleteProfile(profileName, true)
	end,
	GetActiveProfile = function (self) return self:GetDatabase():GetCurrentProfile() end,
	GetDatabase = function (self) error("GetDatabase is not implemented") end,
	HasProfile = function (self, profileName) return A.table.any(self:GetDatabase():GetProfiles(), function (p) return p == profileName end) end,
	SetActiveProfile = function (self, profileName) self:GetDatabase():SetProfile(profileName) end,
})

-------------------------------------------------------------------------------
-- Miscellaneous
-------------------------------------------------------------------------------

C.CLUTTER_KEY = C.Enum.ModifierKey.Control
C.CLUTTER_KEY_FUNCTION = C.Enum.ModifierKeyFunction[C.CLUTTER_KEY]
C.CLUTTER_KEY_STATE_TEXT = C.Enum.ModifierKeyStateText[C.CLUTTER_KEY]

C.COPPER_PER_THOUSAND_GOLD = COPPER_PER_GOLD * 1000
C.COPPER_PER_MILLION_GOLD = C.COPPER_PER_THOUSAND_GOLD * 1000

C.LIGHT_TEXTURE_NAME = "SixUI: Light"
C.LIGHT_TEXTURE_PATH = [[Interface\AddOns\SixUI\Resources\Textures\Light.tga]]

C.NOOP_HANDLER = function () end

C.TRANSPARENT_COLOR = CreateColor(0, 0, 0, 0)

C.XP_TEXT = "%s / %s (%s%%)"
C.RESTED_XP_TEXT = "%s (%s%%)"

C.ZOOMED_TEX_COORDS = setmetatable({}, {
	__index = function (t, k)
		if type(k) == "number" and k >= 0 and k <= 1 then
			local v = {k * 0.25, 1 - k * 0.25, k * 0.25, 1 - k * 0.25}
			rawset(t, k, v)
			return v
		else
			return {0, 1, 0, 1}
		end
	end,
})
