
-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database

-------------------------------------------------------------------------------
-- Database
-------------------------------------------------------------------------------

function D.GetColorTable(color)
	if not color then color = C.FALLBACK_UTILITY_COLOR end
	return {r = color.r, g = color.g, b = color.b, a = color.a or 1}
end

function D.GetFontSize(font)
	return select(2, (font or C.TEXT_REGULAR):GetFont())
end

D.BAR_CONFIG_DEFAULTS = {
	["**"] = {
		reverseRowsOnHorizontalBars = false,
		showBindingText = true,
		showMacroText = true,
		showTextWithClutterKey = false,
		skinActionButtons = true,
		stateText = "",
	},
	["3"] = {
		stateText = "[clutter] [cursor] show; hide",
	},
	["4"] = {
		stateText = "[clutter] [cursor] show; hide",
	},
	["5"] = {
		stateText = "[clutter] [cursor] show; hide",
	},
}

D.DISPLAY_CONFIG_DEFAULTS = {
	barR = 0, barG = 0, barB = 0,
	enabled = false,
	glyph = false,
	glyphOverride = "",
	glyphR = 0, glyphG = 0, glyphB = 0,
	group = "SixUI_DisplayGroup1",
	icon = false,
	iconSize = 16,
	iconZoom = 0,
	label = false,
	labelR = 1, labelG = 1, labelB = 1,
	order = 10,
	suffix = false,
	suffixR = 0, suffixG = 0, suffixB = 0,
	text = false,
	textR = 1, textG = 1, textB = 1,
	useCustomBarColor = false,
	value = false,
	valueR = 1, valueG = 1, valueB = 1,
	width = 0,
}

D.DATABASE_DEFAULTS = {
	char = {},
	global = {},
	profile = {
		border = {
			background = {
				color = D.GetColorTable(C.BORDER_BACKGROUND_COLOR),
				inset = 0,
			},
			accent = {
				anchorOffset = 1,
				color = D.GetColorTable(C.BORDER_ACCENT_COLOR),
				width = 1,
			},
			stroke = {
				anchorOffset = 0,
				color = D.GetColorTable(C.BORDER_STROKE_COLOR),
				width = 3,
			},
		},
		color = {
			debuff = {
				curse = D.GetColorTable(C.CURSE_DEBUFF_COLOR),
				disease = D.GetColorTable(C.DISEASE_DEBUFF_COLOR),
				magic = D.GetColorTable(C.MAGIC_DEBUFF_COLOR),
				other = D.GetColorTable(C.OTHER_DEBUFF_COLOR),
				poison = D.GetColorTable(C.POISON_DEBUFF_COLOR),
			},
			faction = {
				alliance = D.GetColorTable(C.ALLIANCE_FACTION_COLOR),
				horde = D.GetColorTable(C.HORDE_FACTION_COLOR),
			},
			rating = {
				bronze = D.GetColorTable(C.BRONZE_RATING_COLOR),
				gold = D.GetColorTable(C.GOLD_RATING_COLOR),
				silver = D.GetColorTable(C.SILVER_RATING_COLOR),
				untimed = D.GetColorTable(C.UNTIMED_RATING_COLOR),
			},
			reaction = {
				hostile = D.GetColorTable(C.HOSTILE_REACTION_COLOR),
				neutral = D.GetColorTable(C.NEUTRAL_REACTION_COLOR),
				friendly = D.GetColorTable(C.FRIENDLY_REACTION_COLOR),
			},
			reputation = {
				exceptionallyHostile = D.GetColorTable(C.EXCEPTIONALLY_HOSTILE_REPUTATION_COLOR),
				veryHostile = D.GetColorTable(C.VERY_HOSTILE_REPUTATION_COLOR),
				hostile = D.GetColorTable(C.HOSTILE_REPUTATION_COLOR),
				neutral = D.GetColorTable(C.NEUTRAL_REPUTATION_COLOR),
				friendly = D.GetColorTable(C.FRIENDLY_REPUTATION_COLOR),
				veryFriendly = D.GetColorTable(C.VERY_FRIENDLY_REPUTATION_COLOR),
				exceptionallyFriendly = D.GetColorTable(C.EXCEPTIONALLY_FRIENDLY_REPUTATION_COLOR),
				exalted = D.GetColorTable(C.EXALTED_REPUTATION_COLOR),
			},
			resource = {
				honor = D.GetColorTable(C.HONOR_RESOURCE_COLOR),
				renown = D.GetColorTable(C.RENOWN_RESOURCE_COLOR),
				restedXP = D.GetColorTable(C.RESTED_XP_RESOURCE_COLOR),
				valor = D.GetColorTable(C.VALOR_RESOURCE_COLOR),
				xp = D.GetColorTable(C.XP_RESOURCE_COLOR),
			},
			utility = {
				fallback = D.GetColorTable(C.FALLBACK_UTILITY_COLOR),
				guild = D.GetColorTable(C.GUILD_UTILITY_COLOR),
				highlight = D.GetColorTable(C.HIGHLIGHT_UTILITY_COLOR),
			},
		},
		font = {
			appearance = {
				name = C.TEXT_FONT_NAME,
				overrideGameFonts = true,
				overrideSharedMediaFonts = true,
			},
			glyphFontSize = {
				large = D.GetFontSize(C.GLYPH_LARGE),
				regular = D.GetFontSize(C.GLYPH_REGULAR),
				small = D.GetFontSize(C.GLYPH_SMALL),
			},
			textFontSize = {
				large = D.GetFontSize(C.TEXT_LARGE),
				medium = D.GetFontSize(C.TEXT_MEDIUM),
				regular = D.GetFontSize(C.TEXT_REGULAR),
			},
		},
		layout = {
			activeLayout = C.Enum.Layout.Standard,
		},
		module = {
			["**"] = {
				enabled = true,
			},
			["Automation Interact"] = {
				acceptQuests = true,
				acceptReadyChecks = true,
				chooseGossipOptions = true,
				completeQuests = true,
				modifierKey = C.Enum.ModifierKey.Shift,
			},
			["Automation Invite"] = {
				allowClubMembers = true,
				allowFriends = true,
				allowLastCharacterWhispered = true,
				blacklist = {},
				disableInInstance = true,
				disableInQueue = true,
			},
			["Automation Repair"] = {
				useCharacterGold = true,
				useGuildGold = true,
			},
			["Automation Sell"] = {
				blacklist = {},
			},
			["Blizzard ActionBar"] = {
				barConfigs = {
					Damage = CreateFromMixins(D.BAR_CONFIG_DEFAULTS, {
						["1"] = {
							showMacroText = false,
							showTextWithClutterKey = true,
						},
						["2"] = {
							showMacroText = false,
							showTextWithClutterKey = true,
						},
					}),
					Minimal = CreateFromMixins(D.BAR_CONFIG_DEFAULTS, {
						["1"] = {
							reverseRowsOnHorizontalBars = true,
							showMacroText = false,
							stateText = "[clutter] [cursor] [possessbar] show; hide",
						},
						["2"] = {
							reverseRowsOnHorizontalBars = true,
							showMacroText = false,
							stateText = "[clutter] [cursor] show; hide",
						},
					}),
					Standard = CreateFromMixins(D.BAR_CONFIG_DEFAULTS, {
						["1"] = {
							reverseRowsOnHorizontalBars = true,
							showBindingText = false,
							showMacroText = false,
							showTextWithClutterKey = true,
						},
						["2"] = {
							reverseRowsOnHorizontalBars = true,
							showBindingText = false,
							showMacroText = false,
							showTextWithClutterKey = true,
						},
					}),
				},
				disableAnimations = true,
			},
			["Blizzard ObjectiveTracker"] = {
				showArtwork = false,
				showButton = false,
				stateText = "[clutter] show; [@arena*,exists] [@boss*,exists] hide; show",
			},
			["Buff Bars"] = {
				enabled = false,
			},
			["Buff CollapseAndExpandButton"] = {
				autoCollapseBuffs = C.Enum.AutoCollapseBuffs.WhileInAnInstance,
				showCollapseAndExpandButton = false,
			},
			["Buff Frame"] = {
				skinBuffs = true,
				skinDebuffs = true,
			},
			["Core RaidFrameConfig"] = {
				raidFrameConfigs = {
					["*"] = {
						["**"] = {
							height = 76,
							offsetX = 0,
							offsetY = 338,
							point = "TOP",
							relativePoint = "BOTTOM",
							relativeTo = "UIParent",
							width = 100,
						},
						[25] = {
							width = 100,
							height = 60,
						},
						[30] = {
							width = 100,
							height = 50,
						},
						[35] = {
							width = 100,
							height = 36,
						},
						[40] = {
							width = 100,
							height = 36,
						},
					},
					Damage = {
						["**"] = {
							height = 60,
							offsetX = 20,
							offsetY = 0,
							point = "LEFT",
							relativePoint = "LEFT",
							relativeTo = "UIParent",
							width = 92,
						},
						[15] = {
							width = 92,
							height = 50,
						},
						[20] = {
							width = 92,
							height = 50,
						},
						[25] = {
							width = 92,
							height = 36,
						},
						[30] = {
							width = 92,
							height = 36,
						},
						[35] = {
							width = 92,
							height = 36,
						},
						[40] = {
							width = 92,
							height = 36,
						},
					},
				},
			},
			["Display Friends"] = {
				categorizeByNote = true,
				categorizeCharacterFriends = true,
				categorizeFavorites = true,
				columns = C.Enum.FriendColumn,
				showMobile = false,
				showOffline = false,
			},
			["Display Group"] = {
				displayConfigs = {
					["data source"] = {
						["**"] = CreateFromMixins(D.DISPLAY_CONFIG_DEFAULTS, {
							icon = true,
							text = true,
						}),
						["SixUI Currency"] = {
							enabled = true,
							icon = false,
							order = 4,
							suffix = true,
							text = false,
							value = true,
						},
						["SixUI Durability"] = {
							enabled = true,
							icon = false,
							order = 3,
							suffix = true,
							text = false,
							value = true,
						},
						["SixUI Framerate"] = {
							enabled = true,
							icon = false,
							order = 1,
							suffix = true,
							text = false,
							value = true,
						},
						["SixUI Friends"] = {
							enabled = true,
							glyph = true,
							group = "SixUI_DisplayGroup2",
							icon = false,
							order = 1,
							text = false,
							value = true,
						},
						["SixUI Guild"] = {
							enabled = true,
							glyph = true,
							group = "SixUI_DisplayGroup2",
							icon = false,
							order = 2,
							text = false,
							value = true,
						},
						["SixUI History"] = {
							enabled = true,
							glyph = true,
							group = "SixUI_DisplayGroup2",
							icon = false,
							order = 3,
							text = false,
							suffix = false,
						},
						["SixUI Latency"] = {
							enabled = true,
							icon = false,
							order = 2,
							suffix = true,
							text = false,
							value = true,
						},
					},
					["launcher"] = {
						["**"] = CreateFromMixins(D.DISPLAY_CONFIG_DEFAULTS, {
							icon = true,
							width = 25,
						}),
						["SixUI Calendar"] = {
							["enabled"] = true,
							["glyph"] = true,
							["glyphR"] = 1, ["glyphG"] = 1, ["glyphB"] = 1,
							["group"] = "SixUI_MinimapButtons",
							["icon"] = false,
							["order"] = 3,
							["width"] = 0,
						},
						["SixUI Garrison Report"] = {
							["enabled"] = true,
							["glyph"] = true,
							["glyphR"] = 1, ["glyphG"] = 1, ["glyphB"] = 1,
							["group"] = "SixUI_MinimapButtons",
							["icon"] = false,
							["order"] = 4,
							["width"] = 0,
						},
						["SixUI Instance Info"] = {
							["enabled"] = true,
							["glyph"] = true,
							["glyphR"] = 1, ["glyphG"] = 1, ["glyphB"] = 1,
							["group"] = "SixUI_MinimapButtons",
							["icon"] = false,
							["order"] = 1,
							["width"] = 0,
						},
						["SixUI Settings"] = {
							["enabled"] = true,
							["glyph"] = true,
							["glyphR"] = 1, ["glyphG"] = 1, ["glyphB"] = 1,
							["group"] = "SixUI_MinimapButtons",
							["icon"] = false,
							["order"] = 5,
							["width"] = 0,
						},
						["SixUI Tracking"] = {
							["enabled"] = true,
							["glyph"] = true,
							["glyphR"] = 1, ["glyphG"] = 1, ["glyphB"] = 1,
							["group"] = "SixUI_MinimapButtons",
							["icon"] = false,
							["order"] = 2,
							["width"] = 0,
						},
					},
					["progress bar"] = {
						["**"] = CreateFromMixins(D.DISPLAY_CONFIG_DEFAULTS, {
							group = "SixUI_ProgressGroup",
						}),
						["SixUI Honor"] = {
							enabled = true,
							order = 3,
						},
						["SixUI MythicPlus"] = {
							enabled = true,
							order = 5,
						},
						["SixUI Reputation"] = {
							enabled = true,
							order = 2,
						},
						["SixUI Valor"] = {
							enabled = true,
							order = 4,
						},
						["SixUI XP"] = {
							enabled = true,
							order = 1,
						},
					},
				},
			},
			["Display Guild"] = {
				columns = C.Enum.GuildColumn,
			},
			["Integration AdiBags"] = {
				skinBagsAndItems = true,
				zoomIcons = 0.3,
			},
			["Integration Bagnon"] = {
				skinItems = true,
				zoomIcons = 0.3,
			},
			["Integration Dominos"] = {
				showBindingText = true,
				showMacroText = true,
			},
			["Integration Gnosis"] = {
				skinBars = true,
				useClassColors = true,
				useReactionColors = true,
			},
			["Integration Grid2"] = {
				controlButtonSize = true,
				controlPanelPosition = true,
				overrideTextures = true,
				skinRaidFrames = true,
				useAlternativeRaidIcons = true,
				useClutterKey = true,
				useHighQualityIcons = true,
				useRaidFrameConfig = true,
			},
			["Integration ShadowedUnitFrames"] = {
				skinAuras = true,
				skinUnitFrames = true,
				useClutterKey = true,
				useInstantFading = true,
			},
			["Integration Skada"] = {
				optimizeBarHeight = true,
				optimizeBarSpacing = true,
				columnTotal = C.Enum.SkadaColumnType.Clutter,
				columnPercent = C.Enum.SkadaColumnType.Clutter,
				columnThroughput = C.Enum.SkadaColumnType.NoClutter,
				overrideTextures = true,
				skinWindows = true,
				useClutterKey = true,
			},
			["Integration VuhDo"] = {
				controlButtonSize = true,
				controlPanelPosition = true,
				overrideHolyPaladinRole = false,
				overrideHunterRole = false,
				overrideTextures = true,
				resizeDebuffs = true,
				skinRaidFrames = true,
				useRaidFrameConfig = true,
				zoomIcons = 0.3,
			},
			["Integration WeakAuras"] = {
				blacklist = {
					["DF Dungeons (wago.io/dfdungeons)"] = true,
					["Liquid - Amirdrassil"] = true,
					["Liquid Anchors (don't rename these)"] = true,
					["Nexus Sx Amirdrassil"] = true,
					["SM: Targeted Spells"] = true,
				},
				skinIcons = true,
				skinProgressBars = true,
			},
			["Minimap Frame"] = {
				skinMinimap = true,
			},
			["Miscellaneous Clutter"] = {
				modifierKey = C.Enum.ModifierKey.Control,
			},
		},
	},
}

function D:CreateDatabase()
	local db = LibStub("AceDB-3.0"):New("SixDB", self.DATABASE_DEFAULTS, true)

	setmetatable(self, {__index = db, __newindex = function (_, k, v) db[k] = v end})

	self:RegisterCallback("OnProfileChanged")
	self:RegisterCallback("OnProfileCopied", "OnProfileChanged")
	self:RegisterCallback("OnProfileReset", "OnProfileChanged")
end

function D:IsDatabase(db)
	return (getmetatable(self) or {}).__index == db
end

function D:OnProfileChanged()
	A:UpdateModuleConfigs()
	A:UpdateModuleStates()
	A:SendMessage("SIXUI_PROFILE_CHANGED")
end
