
-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database

-------------------------------------------------------------------------------
-- ModulePrototype
-------------------------------------------------------------------------------

A.ModulePrototype = {}

function A.ModulePrototype:CreateBorder(...)
	return B:CreateBorder(...)
end

function A.ModulePrototype:CreateHook(funcName, ...)
	if not self:IsHooked(...) then
		self[funcName](self, ...)
		return true
	else
		return false
	end
end

function A.ModulePrototype:IsDisabledByAddOns()
	return self.dependencies and A.table.findIndex(self.dependencies, function (l, a) return C_AddOns.IsAddOnLoaded(a) ~= l end) ~= nil
end

function A.ModulePrototype:IsEnabledByConfig()
	return D.profile.module[self:GetName()].enabled
end

function A.ModulePrototype:RemoveBorder(...)
	return B:RemoveBorder(...)
end

function A.ModulePrototype:WithConflict(...)
	if not self.dependencies then
		self.dependencies = {}
	end

	for _, addOnName in next, {...} do
		self.dependencies[addOnName] = false
	end

	return self
end

function A.ModulePrototype:WithDependency(...)
	if not self.dependencies then
		self.dependencies = {}
	end

	for _, addOnName in next, {...} do
		self.dependencies[addOnName] = true
	end

	return self
end

A:SetDefaultModuleLibraries("AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")
A:SetDefaultModulePrototype(A.ModulePrototype)
A:SetDefaultModuleState(false)

-------------------------------------------------------------------------------
-- AddOn
-------------------------------------------------------------------------------

function A:IterateModulesWithDependencies(filter)
	return coroutine.wrap(function ()
		for name, module in self:IterateModules() do
			if module.dependencies and (not filter or module.dependencies[filter]) then
				coroutine.yield(module, name, module.dependencies)
			end
		end
	end)
end

function A:EnableModule(name)
	self:SetModuleState(name, true)
end

function A:DisableModule(name)
	self:SetModuleState(name, false)
end

function A:IsModuleEnabledByConfig(name)
	return not not D.profile.module[name].enabled
end

function A:SetModuleState(name, state)
	local module, config = self:GetModule(name, true), self:GetModuleConfig(name)

	if not module then
		self.PrintF("The module '%s' wasn't recognized.", self.Highlight(name))
	elseif not module:IsDisabledByAddOns() and module:IsEnabled() ~= state then
		config.enabled = state
		self:UpdateModuleConfig(name, module)
		module[state and "Enable" or "Disable"](module)
		self:GetModule("Miscellaneous Clutter"):Broadcast()
	end
end

function A:GetModuleConfig(name)
	return D.profile.module[name]
end

function A:UpdateModuleConfigs()
	for name, module in self:IterateModules() do
		self:UpdateModuleConfig(name, module)
	end
end

function A:UpdateModuleConfig(name, module)
	module.config = self:GetModuleConfig(name)
end

function A:UpdateModuleStates()
	for name in self:IterateModules() do
		self:UpdateModuleState(name)
	end
end

function A:UpdateModuleState(name)
	self:SetModuleState(name, self:IsModuleEnabledByConfig(name))
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function A:OnModuleCreated(module)
	if D.profile then
		self:UpdateModuleConfig(module:GetName(), module)
	end
end
