
-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database

-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: C_EncounterJournal, C_RaidLocks, C_Scenario, C_SocialQueue, ChatEdit_ActivateChat, ChatEdit_ChooseBoxForSend, ChatEdit_GetActiveWindow, ChatFrame_GetMessageEventFilters, ChatFrame_OpenChat, ChatFrame_SendTell, CreateColor, DEFAULT_CHAT_FRAME, EJ_GetEncounterInfoByIndex, EJ_SelectInstance, GetInstanceInfo, GetRelativeDifficultyColor, GetUnitName, GRAY_FONT_COLOR, IsAltKeyDown, IsControlKeyDown, IsModifierKeyDown, IsShiftKeyDown, select, SettingsPanel, SocialQueueUtil_HasRelationshipWithLeader, SocialQueueUtil_SetTooltip, strsub, table, tonumber, tostring, type, UnitCanCooperate, UnitEffectiveLevel, UnitInParty, UnitInRaid, UnitIsConnected, UnitIsPlayer, UnitLevel

local C_Map = C_Map
local CHALLENGE_MODE_ITEM_POWER_LEVEL = CHALLENGE_MODE_ITEM_POWER_LEVEL
local COPPER_AMOUNT_SYMBOL = COPPER_AMOUNT_SYMBOL
local COPPER_PER_GOLD = COPPER_PER_GOLD
local COPPER_PER_SILVER = COPPER_PER_SILVER
local DUNGEON_NAME_WITH_DIFFICULTY = DUNGEON_NAME_WITH_DIFFICULTY
local EJ_GetInstanceForMap = EJ_GetInstanceForMap
local floor = floor
local format = format
local GetNormalizedRealmName = GetNormalizedRealmName
local GOLD_AMOUNT_SYMBOL = GOLD_AMOUNT_SYMBOL
local next = next
local SILVER_AMOUNT_SYMBOL = SILVER_AMOUNT_SYMBOL
local string = string
local UNKNOWN = UNKNOWN

-------------------------------------------------------------------------------
-- Quick Join
-------------------------------------------------------------------------------

-- Show a Quick Join tooltip.
function A:ShowGroupTooltip(tooltip, name, groupGUID)
	local canJoin, _, _, _, _, _, _, leaderGUID = C_SocialQueue.GetGroupInfo(groupGUID)
	if not canJoin then return end
	local queues = C_SocialQueue.GetGroupQueues(groupGUID)
	if not queues then return end
	local knowsLeader = SocialQueueUtil_HasRelationshipWithLeader(leaderGUID)
	SocialQueueUtil_SetTooltip(tooltip, name, queues, canJoin, knowsLeader)
end

-------------------------------------------------------------------------------
-- Click Handler
-------------------------------------------------------------------------------

function A:GetModifierKey()
	return IsAltKeyDown() and C.Enum.ModifierKey.Alt
		or IsControlKeyDown() and C.Enum.ModifierKey.Control
		or IsShiftKeyDown() and C.Enum.ModifierKey.Shift
		or C.Enum.ModifierKey.None
end

function A:GetMouseButton(button)
	return button == "LeftButton" and C.Enum.MouseButton.Left
		or button == "MiddleButton" and C.Enum.MouseButton.Middle
		or button == "RightButton" and C.Enum.MouseButton.Right
		or C.Enum.MouseButton.None
end

function A:HandleClick(entries, button, ...)
	local modifierKey = self:GetModifierKey()
	local mouseButton = self:GetMouseButton(button)

	for _, entry in next, entries do
		if (entry.mouseButton == nil or entry.mouseButton == mouseButton or entry.mouseButton == C.Enum.MouseButton.Any) and
			(entry.modifierKey == nil or entry.modifierKey == modifierKey or entry.modifierKey == C.Enum.ModifierKey.Any) and
			(entry.test == nil or entry.test(...) == true) then
			return entry.callback(...)
		end
	end
end

-------------------------------------------------------------------------------
-- Table
-------------------------------------------------------------------------------

A.table = setmetatable({}, {__index = _G.table})

-- Returns true if all values in `t` satisfy `predicateFn`.
function A.table.all(t, predicateFn)
	return A.table.findIndex(t, function(...) return not predicateFn(...) end) == nil
end

-- Returns true if any value in `t` satisfies `predicateFn`.
function A.table.any(t, predicateFn)
	return A.table.findIndex(t, predicateFn) ~= nil
end

-- Returns true if `t1` and `t2` are equal.
function A.table.compare(t1, t2)
	if type(t1) == "table" and type(t2) == "table" then
		for index, value in next, t1 do
			if type(value) == "table" then
				if not A.table.compare(value, t2[index]) then
					return false
				end
			else
				if value ~= t2[index] then
					return false
				end
			end
		end
	end

	return true
end

-- Returns a new table containing only the values of `t` that satisfy `predicateFn`.
function A.table.filter(t, predicateFn, sequential)
	local result = {}

	for k, v in next, t do
		if predicateFn(v, k) then
			if sequential then
				table.insert(result, v)
			else	
				result[k] = v
			end
		end
	end

	return result
end

-- Returns the index of the first value in `t` that satisfies `predicateFn`, or nil if none is found.
function A.table.findIndex(t, predicateFn)
	for i, v in next, t do
		if predicateFn(v, i) then
			return i
		end
	end
end

-- Returns a new table containing all keys from `t`.
function A.table.keys(t)
	return A.table.reduce(t, function(ks, _, k)
		table.insert(ks, k)
		return ks
	end, {})
end

-- Returns the number of elements in `t`.
function A.table.length(t)
	return #A.table.filter(t, function () return true end, true)
end

-- Returns a new table (or initial) by iteratively applying `reducerFn` to each key-value pair in `t`, starting with `initial` as the initial accumulator value.
function A.table.reduce(t, reducerFn, initial)
	local result = initial

	for k, v in pairs(t) do
		result = reducerFn(result, v, k)
	end

	return result
end

function A.table.upsert(t, v)
	local index = A.table.findIndex(t, function (entry) return entry == v end)

	if index then
		t[index] = v
	else
		table.insert(t, v)
	end
end

-- Returns a new table containing all values from `t`.
function A.table.values(t)
	return A.table.reduce(t, function(vs, v)
		table.insert(vs, v)
		return vs
	end, {})
end

-------------------------------------------------------------------------------
-- Utility
-------------------------------------------------------------------------------

function A.AtlasToFile(atlas)
	return (C_Texture.GetAtlasInfo(atlas) or {}).file
end

function A.AtlasToTexCoord(atlas)
	local info = C_Texture.GetAtlasInfo(atlas) or {}
	return info.leftTexCoord, info.rightTexCoord, info.topTexCoord, info.bottomTexCoord
end

function A.CloseEditBox()
	ChatEdit_ChooseBoxForSend():ClearFocus()
end

function A.CreateProxyTable(original, override, isSticky)
	return setmetatable({}, {
		__call = function (_, v)
			original = v
		end,
		__index = function (t, k)
			if k == "original" then
				return original
			end

			local overrideValue = override[k]
			local originalValue = original[k]

			if overrideValue ~= nil then
				local result = (type(overrideValue) == "function" and {overrideValue(t, k)} or {overrideValue})[1]

				if result ~= nil then
					return type(result) == "table" and not getmetatable(result) and type(originalValue) == "table" and A.CreateProxyTable(originalValue, result, isSticky) or result
				end
			end

			return type(originalValue) == "function" and function (_, ...) return originalValue(original, ...) end or originalValue
		end,
		__newindex = function (_, k, v)
			if not isSticky or override[k] == nil then
				original[k] = v
			end
		end,
	})
end

-- Returns a table of values filtered and transformed from an iterator.
function A.FilterIterator(iteratorFn, predicateFn, transformFn, ...)
	local result = {}

	if not transformFn then
		transformFn = function (arg1) return arg1 end
	end

	for arg1, arg2, arg3, arg4, arg5 in iteratorFn(...) do
		if predicateFn(arg1, arg2, arg3, arg4, arg5) then
			table.insert(result, transformFn(arg1, arg2, arg3, arg4, arg5))
		end
	end

	return result
end

function A.FlavourStateText(stateText)
	local clutter = C.CLUTTER_KEY_STATE_TEXT
	local noclutter = clutter:gsub("mod:", "nomod:")

	stateText = stateText:gsub("noclutter", noclutter)
	stateText = stateText:gsub("clutter", clutter)
	stateText = stateText:gsub("(%b[])", function (match)
		if match:find("arena%*") then
			local arena = {}

			for i = 1, 5 do
				table.insert(arena, "" .. match:gsub("arena%*", "arena" .. i))
			end

			return table.concat(arena, " ")
		elseif match:find("boss%*") then
			local boss = {}

			for i = 1, 8 do
				table.insert(boss, "" .. match:gsub("boss%*", "boss" .. i))
			end

			return table.concat(boss, " ")
		else
			return match
		end
	end)

	return stateText
end

function A.FlavourStateText(stateText)
	local clutter = C.CLUTTER_KEY_STATE_TEXT
	local noclutter = clutter:gsub("mod:", "nomod:")

	stateText = stateText:gsub("noclutter", noclutter)
	stateText = stateText:gsub("clutter", clutter)
	stateText = stateText:gsub("(%b[])", function (match)
		if match:find("arena%*") then
			local arena = {}

			for i = 1, 5 do
				table.insert(arena, "" .. match:gsub("arena%*", "arena" .. i))
			end

			return table.concat(arena, " ")
		elseif match:find("boss%*") then
			local boss = {}

			for i = 1, 8 do
				table.insert(boss, "" .. match:gsub("boss%*", "boss" .. i))
			end

			return table.concat(boss, " ")
		else
			return match
		end
	end)

	return stateText
end

-- Convert 1.234567 into 1.23.
function A.FormatDecimal(value, precision, simplify)
	local result = string.format("%." .. (precision or 2) .. "f", value)

	if simplify then
		result = result:gsub("(%.)?0+$", "")
	end

	return tonumber(result)
end

-- Returns a formatted string with the instance name, difficulty, and keystone level (if applicable).
function A.FormatInstanceName(name, difficultyName, keystoneLevel)
	return DUNGEON_NAME_WITH_DIFFICULTY:format(name, keystoneLevel and keystoneLevel ~= 0 and CHALLENGE_MODE_ITEM_POWER_LEVEL:format(keystoneLevel) or difficultyName or UNKNOWN)
end

-- Returns "Name" if the realm is the player's realm, or "Name-Realm".
function A.FormatUnit(name, realm, alwaysReturnRealm)
	if not name then return end
	name, realm = unpack(name:match("-") and {string.split("-", name)} or {name, realm or C.PLAYER_REALM_NAME})
	return Ambiguate(name .. "-" .. realm, alwaysReturnRealm and "mail" or "none")
end

function A.FormatMoney(money)
	local gold, symbol

	if money >= C.COPPER_PER_MILLION_GOLD then
		gold = A.FormatDecimal(money / C.COPPER_PER_MILLION_GOLD)
		symbol = "m"
	elseif money >= C.COPPER_PER_THOUSAND_GOLD then
		gold = floor(money / C.COPPER_PER_THOUSAND_GOLD)
		symbol = "k"
	elseif money >= COPPER_PER_GOLD then
		gold = floor(money / COPPER_PER_GOLD)
		symbol = GOLD_AMOUNT_SYMBOL
	elseif money >= COPPER_PER_SILVER then
		gold = floor(money / COPPER_PER_SILVER)
		symbol = SILVER_AMOUNT_SYMBOL
	else
		gold = money
		symbol = COPPER_AMOUNT_SYMBOL
	end

	local formatted = string.format("%s|cff%s%s|r", gold, symbol == "c" and "eda55f" or symbol == "s" and "c7c7cf" or "ffd700", symbol)

	return formatted, gold, symbol
end

function A.IterateJournalBosses(journalInstanceID)
	if not journalInstanceID then return C.NOOP_HANDLER end
	local index = 1
	EJ_SelectInstance(journalInstanceID)

	return coroutine.wrap(function ()
		while true do
			local info = {EJ_GetEncounterInfoByIndex(index, journalInstanceID)}
			if not info[1] then break end
			coroutine.yield(unpack(info))
			index = index + 1
		end
	end)
end

function A.GetJournalBosses(journalInstanceID)
	local bosses = {}
	local difficultyID = select(3, GetInstanceInfo())
	local isInScenario = C_Scenario.IsInScenario()

	for name, _, journalEncounterID, _, _, _, encounterID, mapID in A.IterateJournalBosses(journalInstanceID) do
		local isDefeated = C_EncounterJournal.IsEncounterComplete(journalEncounterID)
		local isLocked = not isInScenario and C_RaidLocks.IsEncounterComplete(mapID, encounterID, difficultyID)
		local status = C.Enum.BossStatusLookup[isInScenario][isDefeated][isLocked]

		table.insert(bosses, {name = name, status = status})
	end

	return bosses
end

function A.GetJournalInstanceID()
	local mapID = C_Map.GetBestMapForUnit("player")
	local journalInstanceID = mapID and EJ_GetInstanceForMap(mapID) or 0
	if (journalInstanceID ~= 0) then return journalInstanceID end
end

-- Convert a frame's UI size to pixel size.
function A.GetPixelSize(uiSize, frame)
	return PixelUtil.GetNearestPixelSize(uiSize, (frame or UIParent):GetEffectiveScale())
end

-- Get a frame's closest screen corner.
function A.GetScreenCorner(frame)
	local x, y = frame:GetCenter()
	local w, h = UIParent:GetSize()
	return (y > h / 2 and "TOP" or "BOTTOM") .. (x > w / 2 and "RIGHT" or "LEFT")
end

function A.GetSettingsCategoryID(name)
	for _, category in next, SettingsPanel:GetCategoryList():GetAllCategories() do
		if category.name == name then
			return category.ID
		end
	end
end

-- Get the relative difficulty color of a level.
function A.GetDifficultyColor(difficultyLevel, isScaling)
	if tonumber(difficultyLevel) then
		local playerLevel = isScaling and UnitEffectiveLevel("player") or UnitLevel("player")
		local color = GetRelativeDifficultyColor(playerLevel, difficultyLevel)

		return CreateColor(color.r, color.g, color.b)
	end

	return GRAY_FONT_COLOR
end

-- Wrap text in a color code.
function A.Highlight(text, color)
	return (color or C.HIGHLIGHT_UTILITY_COLOR):WrapTextInColorCode(text or "")
end

-- Check whether an event is being filtered.
function A.IsFiltered(event, ...)
	local filters = ChatFrame_GetMessageEventFilters(event)

	for _, filterFunc in next, filters do
		if filterFunc(DEFAULT_CHAT_FRAME, event, ...) then
			return true
		end
	end

	return false
end

-- Check whether a string starts with "/".
function A.IsSlashCommand(text)
	return strsub(tostring(text), 1, 1) == "/"
end

-- Open the edit box.
function A.OpenEditBox()
	ChatEdit_ActivateChat(ChatEdit_ChooseBoxForSend())
end

-- Print a message to the default chat frame.
function A.Print(message)
	DEFAULT_CHAT_FRAME:AddMessage(message, 1, 1, 1)
end

-- Print a formatted message to the default chat frame.
function A.PrintF(message, ...)
	A.Print(format(message, ...))
end

-- Set the edit box's text.
function A.SetEditBoxText(text)
	local editBox = ChatEdit_ChooseBoxForSend()

	if editBox ~= ChatEdit_GetActiveWindow() then
		ChatFrame_OpenChat(text)
	else
		editBox:SetFocus()
		editBox:SetText(text)
		editBox:HighlightText()
	end
end

-- Send a tell to the player's target.
function A.TellTarget()
	if UnitIsPlayer("target") and UnitIsConnected("target") and UnitCanCooperate("player", "target") then
		ChatFrame_SendTell(GetUnitName("target", true))
	end
end

-- Check whether a unit is in the player's raid or party.
function A.UnitInGroup(unit)
	return unit and (UnitInRaid(unit) or UnitInParty(unit))
end
