
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: coroutine, MainMenuBar, next, QuickKeybindFrame

local AnchorUtil = AnchorUtil
local EditModeManagerFrame = EditModeManagerFrame
local GridLayoutUtil = GridLayoutUtil
local InCombatLockdown = InCombatLockdown
local KeybindFrames_InQuickKeybindMode = KeybindFrames_InQuickKeybindMode
local math = math
local RegisterAttributeDriver = RegisterAttributeDriver
local Settings = Settings
local table = table
local type = type
local unpack = unpack

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Blizzard ActionBar"):WithConflict("Bartender4", "Dominos")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.ActionBar = {
	ActionBar1 = MainMenuBar,
	ActionBar2 = MultiBarBottomLeft,
	ActionBar3 = MultiBarBottomRight,
	ActionBar4 = MultiBarRight,
	ActionBar5 = MultiBarLeft,
	ActionBar6 = MultiBar5,
	ActionBar7 = MultiBar6,
	ActionBar8 = MultiBar7,
	ExtraActionBar = ExtraActionBarFrame,
	PetBar = PetActionBar,
	PossessBar = PossessActionBar,
	StanceBar = StanceBar,
}

M.ActionBarWithBindingText = {
	[M.ActionBar.ActionBar1] = true,
	[M.ActionBar.ActionBar2] = true,
	[M.ActionBar.ActionBar3] = true,
	[M.ActionBar.ActionBar4] = true,
	[M.ActionBar.ActionBar5] = true,
	[M.ActionBar.ActionBar6] = true,
	[M.ActionBar.ActionBar7] = true,
	[M.ActionBar.ActionBar8] = true,
	[M.ActionBar.ExtraActionBar] = true,
	[M.ActionBar.PetBar] = true,
}

M.ActionBarWithGrid = {
	[M.ActionBar.ActionBar1] = true,
	[M.ActionBar.ActionBar2] = true,
	[M.ActionBar.ActionBar3] = true,
	[M.ActionBar.ActionBar4] = true,
	[M.ActionBar.ActionBar5] = true,
	[M.ActionBar.ActionBar6] = true,
	[M.ActionBar.ActionBar7] = true,
	[M.ActionBar.ActionBar8] = true,
	[M.ActionBar.PetBar] = true,
	[M.ActionBar.PossessBar] = true,
	[M.ActionBar.StanceBar] = true,
}

M.ActionBarWithKeyBinding = {
	[M.ActionBar.ActionBar1] = true,
	[M.ActionBar.ActionBar2] = true,
	[M.ActionBar.ActionBar3] = true,
	[M.ActionBar.ActionBar4] = true,
	[M.ActionBar.ActionBar5] = true,
	[M.ActionBar.ActionBar6] = true,
	[M.ActionBar.ActionBar7] = true,
	[M.ActionBar.ActionBar8] = true,
	[M.ActionBar.ExtraActionBar] = true,
	[M.ActionBar.PetBar] = true,
	[M.ActionBar.StanceBar] = true,
}

M.ActionBarWithMacroText = {
	[M.ActionBar.ActionBar1] = true,
	[M.ActionBar.ActionBar2] = true,
	[M.ActionBar.ActionBar3] = true,
	[M.ActionBar.ActionBar4] = true,
	[M.ActionBar.ActionBar5] = true,
	[M.ActionBar.ActionBar6] = true,
	[M.ActionBar.ActionBar7] = true,
	[M.ActionBar.ActionBar8] = true,
}

M.ActionBarIsShown = {
	[M.ActionBar.ActionBar1] = function () return true end,
	[M.ActionBar.ActionBar2] = function () return Settings.GetValue("PROXY_SHOW_ACTIONBAR_2") end,
	[M.ActionBar.ActionBar3] = function () return Settings.GetValue("PROXY_SHOW_ACTIONBAR_3") end,
	[M.ActionBar.ActionBar4] = function () return Settings.GetValue("PROXY_SHOW_ACTIONBAR_4") end,
	[M.ActionBar.ActionBar5] = function () return Settings.GetValue("PROXY_SHOW_ACTIONBAR_5") end,
	[M.ActionBar.ActionBar6] = function () return Settings.GetValue("PROXY_SHOW_ACTIONBAR_6") end,
	[M.ActionBar.ActionBar7] = function () return Settings.GetValue("PROXY_SHOW_ACTIONBAR_7") end,
	[M.ActionBar.ActionBar8] = function () return Settings.GetValue("PROXY_SHOW_ACTIONBAR_8") end,
	[M.ActionBar.ExtraActionBar] = function () return HasExtraActionBar() end,
	[M.ActionBar.PetBar] = function () return PetHasActionBar() end,
	[M.ActionBar.PossessBar] = function () return IsPossessBarVisible() end,
	[M.ActionBar.StanceBar] = function () return GetNumShapeshiftForms() > 0 end,
}

M.ActionBarKey = {
	[M.ActionBar.ActionBar1] = "1",
	[M.ActionBar.ActionBar2] = "2",
	[M.ActionBar.ActionBar3] = "3",
	[M.ActionBar.ActionBar4] = "4",
	[M.ActionBar.ActionBar5] = "5",
	[M.ActionBar.ActionBar6] = "6",
	[M.ActionBar.ActionBar7] = "7",
	[M.ActionBar.ActionBar8] = "8",
	[M.ActionBar.ExtraActionBar] = "extra",
	[M.ActionBar.PetBar] = "pet",
	[M.ActionBar.PossessBar] = "possess",
	[M.ActionBar.StanceBar] = "stance",
}

M.ActionBarLabel = {
	[M.ActionBar.ActionBar1] = "Action Bar 1",
	[M.ActionBar.ActionBar2] = "Action Bar 2",
	[M.ActionBar.ActionBar3] = "Action Bar 3",
	[M.ActionBar.ActionBar4] = "Action Bar 4",
	[M.ActionBar.ActionBar5] = "Action Bar 5",
	[M.ActionBar.ActionBar6] = "Action Bar 6",
	[M.ActionBar.ActionBar7] = "Action Bar 7",
	[M.ActionBar.ActionBar8] = "Action Bar 8",
	[M.ActionBar.ExtraActionBar] = "Extra Action Bar",
	[M.ActionBar.PetBar] = "Pet Bar",
	[M.ActionBar.PossessBar] = "Possess Bar",
	[M.ActionBar.StanceBar] = "Stance Bar",
}

M.ActionBarOrder = {
	M.ActionBar.ActionBar1,
	M.ActionBar.ActionBar2,
	M.ActionBar.ActionBar3,
	M.ActionBar.ActionBar4,
	M.ActionBar.ActionBar5,
	M.ActionBar.ActionBar6,
	M.ActionBar.ActionBar7,
	M.ActionBar.ActionBar8,
	M.ActionBar.ExtraActionBar,
	M.ActionBar.PetBar,
	M.ActionBar.PossessBar,
	M.ActionBar.StanceBar,
}

M.AnimationEvent = {
	"PLAYER_ENTER_COMBAT",
	"PLAYER_LEAVE_COMBAT",
	"SPELL_ACTIVATION_OVERLAY_GLOW_HIDE",
	"SPELL_ACTIVATION_OVERLAY_GLOW_SHOW",
	"START_AUTOREPEAT_SPELL",
	"STOP_AUTOREPEAT_SPELL",
	"UNIT_SPELLCAST_CHANNEL_START",
	"UNIT_SPELLCAST_CHANNEL_STOP",
	"UNIT_SPELLCAST_EMPOWER_START",
	"UNIT_SPELLCAST_EMPOWER_STOP",
	"UNIT_SPELLCAST_FAILED",
	"UNIT_SPELLCAST_INTERRUPTED",
	"UNIT_SPELLCAST_RETICLE_CLEAR",
	"UNIT_SPELLCAST_RETICLE_TARGET",
	"UNIT_SPELLCAST_SENT",
	"UNIT_SPELLCAST_START",
	"UNIT_SPELLCAST_STOP",
	"UNIT_SPELLCAST_SUCCEEDED",
}

function M:OnEnable()
	self:CreateHook("SecureHook", EditModeManagerFrame, "EnterEditMode", "StyleAllActionBars")
	self:CreateHook("SecureHook", EditModeManagerFrame, "ExitEditMode", "StyleAllActionBars")
	self:CreateHook("SecureHook", "MultiActionBar_Update", "StyleAllActionBars")
	self:CreateHook("SecureHook", QuickKeybindFrame, "Show", "StyleAllActionBars")
	self:CreateHook("SecureHook", QuickKeybindFrame, "Hide", "StyleAllActionBars")
	self:RegisterEvent("SETTINGS_LOADED", "StyleAllActionBars")
	self:RegisterMessage("SIXUI_CLUTTER_KEY_CHANGED", "Update")
	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
	self:RegisterMessage("SIXUI_CONFIG_CHANGED")
	self:RegisterMessage("SIXUI_LAYOUT_CHANGED", "Update")
	self:RegisterMessage("SIXUI_PROFILE_CHANGED", "Update")
	self:Update()
end

function M:OnDisable()
	self:UpdateAnimations(true)
	self:ResetMainMenuBar()
	self:ResetAllActionBars()
	self:ResetAllActionButtons()
end

function M:Update()
	self:UpdateAnimations()
	self:ResetMainMenuBar()
	self:StyleMainMenuBar()
	self:ResetAllActionBars()
	self:StyleAllActionBars()
	self:ResetAllActionButtons()
	self:StyleAllActionButtons()
end

function M:UpdateAnimations(reset)
	if self.config.disableAnimations and not reset then
		A.table.foreach(self.AnimationEvent, function (_, e) ActionBarActionEventsFrame:UnregisterEvent(e) end)
		self:CreateHook("SecureHook", "ActionButton_ShowOverlayGlow", function (b) ActionButton_HideOverlayGlow(b) end)
	else
		A.table.foreach(self.AnimationEvent, function (_, e) ActionBarActionEventsFrame:RegisterEvent(e) end)
		self:Unhook("ActionButton_ShowOverlayGlow")
	end
end

function M:Button_SetChecked(button, checked)
	if button.GetBorder then
		self:ScheduleTimer(function ()
			button:GetBorder():SetColor(unpack(button:GetChecked() and {1, 1, 0} or {}))
		end, 0)
	end
end

function M:GetBarConfig(bar)
	local layout = A:GetActiveLayout()
	local barKey = self.ActionBarKey[bar]

	return self.config.barConfigs[layout][barKey]
end

function M:IterateBarConfigs()
	return coroutine.wrap(function ()
		for _, bar in next, M.ActionBarOrder do
			local barConfig = self:GetBarConfig(bar)

			if barConfig then
				coroutine.yield(bar, self:GetBarConfig(bar), self.ActionBarLabel[bar])
			end
		end
	end)
end

-------------------------------------------------------------------------------
-- Action Bar
-------------------------------------------------------------------------------

M.HIDE_STATE_TEXT = "hide"

M.SHOW_STATE_TEXT = "show"

M.gridLayoutBars = {}

M.visibilityBars = {}

function M:StyleMainMenuBar()
	A:GetFrameConfig(MainMenuBar.BorderArt):SetTexture(nil)
	A:GetFrameConfig(MainMenuBar.EndCaps):DisableDrawLayer("OVERLAY")
end

function M:ResetMainMenuBar()
	A:GetFrameConfig(MainMenuBar.BorderArt):ResetAll()
	A:GetFrameConfig(MainMenuBar.EndCaps):ResetAll()
end

function M:StyleAllActionBars()
	A:RunSecure(self.StyleAllActionBars, function ()
		for bar, barConfig in self:IterateBarConfigs() do
			self:StyleActionBar(bar, barConfig)
		end
	end)
end

function M:ResetAllActionBars()
	A:RunSecure(self.StyleAllActionBars, function ()
		for bar, barConfig in self:IterateBarConfigs() do
			self:ResetActionBar(bar, barConfig)
		end
	end)
end

function M:StyleActionBar(bar, barConfig)
	self:UpdateGridLayout(bar, barConfig)
	self:UpdateVisibility(bar, barConfig)
end

function M:ResetActionBar(bar, barConfig)
	self:UpdateGridLayout(bar, barConfig, true)
	self:UpdateVisibility(bar, barConfig, true)
end

function M:UpdateGridLayout(bar, barConfig, reset)
	if not bar.gridInitialized then
		return
	elseif bar.isHorizontal and barConfig.reverseRowsOnHorizontalBars and not reset then
		local stride = math.ceil(#bar.shownButtonContainers / bar.numRows)
		local buttonPadding = math.max(bar.buttonPadding, 4)
		local layout = GridLayoutUtil.CreateStandardGridLayout(stride, buttonPadding, buttonPadding, 1, -1)

		GridLayoutUtil.ApplyGridLayout(bar.shownButtonContainers, AnchorUtil.CreateAnchor("TOPLEFT", bar, "TOPLEFT"), layout)
		self.gridLayoutBars[bar] = true
	elseif self.gridLayoutBars[bar] then
		self.gridLayoutBars[bar] = nil
		bar.oldGridSettings = nil
		bar:UpdateGridLayout()
	end
end

function M:UpdateVisibility(bar, barConfig, reset)
	local isShown = M.ActionBarIsShown[bar]()

	if barConfig.stateText ~= "" and not reset and isShown and not EditModeManagerFrame:IsEditModeActive() and not KeybindFrames_InQuickKeybindMode() then
		local stateText = A.FlavourStateText(barConfig.stateText)
		RegisterAttributeDriver(bar, "state-visibility", stateText)
		self.visibilityBars[bar] = true
	elseif self.visibilityBars[bar] then
		self.visibilityBars[bar] = nil
		local stateText = isShown and self.SHOW_STATE_TEXT or self.HIDE_STATE_TEXT
		RegisterAttributeDriver(bar, "state-visibility", stateText)
	end
end

-------------------------------------------------------------------------------
-- Action Button
-------------------------------------------------------------------------------

M.skinnedActionButtons = {}

M.text = {}

function M:StyleAllActionButtons()
	for bar, barConfig in self:IterateBarConfigs() do
		for _, button in next, bar.actionButtons or {bar.button} do
			self:StyleActionButton(button, barConfig)
		end
	end
end

function M:ResetAllActionButtons()
	for button in next, self.skinnedActionButtons do
		self:ResetActionButton(button)
	end
end

function M:StyleActionButton(button, barConfig)
	if not self.skinnedActionButtons[button] and barConfig.skinActionButtons then
		self.skinnedActionButtons[button] = true

		A:GetFrameConfig(button):CreateBorder(C.BORDER_PADDING):DisableDrawLayer("BORDER")
		A:GetFrameConfig(button.Border):SetDrawLayer("BORDER")
		A:GetFrameConfig(button.FlyoutBorderShadow):SetDrawLayer("BORDER")
		A:GetFrameConfig(button.HotKey):SetAlpha((barConfig.showBindingText or (barConfig.showTextWithClutterKey and self.isClutterKeyDown)) and 1 or 0)
		A:GetFrameConfig(button.Name):SetAlpha((barConfig.showMacroText or (barConfig.showTextWithClutterKey and self.isClutterKeyDown)) and 1 or 0)
		A:GetFrameConfig(button.NormalTexture):SetDrawLayer("BORDER")
		A:GetFrameConfig(button.SlotArt):SetDrawLayer("BORDER")
		A:GetFrameConfig(button.SlotBackground):SetDrawLayer("BORDER")
		A:GetFrameConfig(button.style):SetDrawLayer("BORDER")
		A:GetFrameConfig(button.cooldown):SetAllPoints(button)
		A:GetFrameConfig(button.icon):SetTexCoord(unpack(C.ZOOMED_TEX_COORDS[0.3])):RemoveMaskTexture(button.IconMask)

		if barConfig.showTextWithClutterKey then
			if button.HotKey and not barConfig.showBindingText then
				self.text[button.HotKey] = true
			end

			if button.Name and not barConfig.showMacroText then
				self.text[button.Name] = true
			end
		end
	end
end

function M:ResetActionButton(button)
	if self.skinnedActionButtons[button] then
		self.skinnedActionButtons[button] = nil

		A:GetFrameConfig(button):ResetAll()
		A:GetFrameConfig(button.Border):ResetAll()
		A:GetFrameConfig(button.FlyoutBorderShadow):ResetAll()
		A:GetFrameConfig(button.HotKey):ResetAll()
		A:GetFrameConfig(button.Name):ResetAll()
		A:GetFrameConfig(button.NormalTexture):ResetAll()
		A:GetFrameConfig(button.SlotArt):ResetAll()
		A:GetFrameConfig(button.SlotBackground):ResetAll()
		A:GetFrameConfig(button.style):ResetAll()
		A:GetFrameConfig(button.cooldown):ResetAll()
		A:GetFrameConfig(button.icon):ResetAll()

		if self.text[button.HotKey] then
			self.text[button.HotKey] = nil
		end

		if self.text[button.Name] then
			self.text[button.Name] = nil
		end
	end
end

function M:UpdateText()
	local alpha = self.isClutterKeyDown and 1 or 0

	for text in next, self.text do
		text:SetAlpha(alpha)
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CLUTTER_STATE_CHANGED(event, state)
	self.isClutterKeyDown = state
	self:UpdateText()
end

function M:SIXUI_CONFIG_CHANGED(event, module, config, key, ...)
	if module == self:GetName() then
		self:Update()
	end
end
