
local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database

local M = A:NewModule("Blizzard EditMode")

Mixin(M, C.SetupMixin)

function M.GetEditModeLayout(layoutName)
	for layoutIndex, layout in next, EditModeManagerFrame:GetLayouts() do
		if layout.layoutName == layoutName then
			return layoutIndex, layout
		end
	end
end

function M:CanCreateLayout(layoutName)
	return self:HasProfile(layoutName) or not EditModeManagerFrame:AreLayoutsOfTypeMaxed(Enum.EditModeLayoutType.Account)
end

function M:CreateProfileFromString(layoutName, layoutInfo, successFn)
	if self:CanCreateLayout(layoutName) then
		self:DeleteProfile(layoutName)
		EditModeManagerFrame:MakeNewLayout(layoutInfo, Enum.EditModeLayoutType.Account, layoutName, true)
		successFn()
	else
		A.PrintF("SixUI cannot create the '%s' Edit Mode layout because you have reached the maximum number of account layouts.", A.Highlight(layoutName))
	end
end

function M:DeleteProfile(layoutName)
	local layoutIndex = self.GetEditModeLayout(layoutName)

	if layoutIndex then
		EditModeManagerFrame:DeleteLayout(layoutIndex)
		EditModeManagerFrame:UpdateDropdownOptions()
	end
end

function M:GetActiveProfile()
	return EditModeManagerFrame:GetActiveLayoutInfo().layoutName
end

function M:HasProfile(layoutName)
	return self.GetEditModeLayout(layoutName) ~= nil
end

function M:IsActiveProfile(layoutName)
	return self:GetActiveProfile() == layoutName
end

function M:SetActiveProfile(layoutName)
	local layoutIndex = self.GetEditModeLayout(layoutName)

	if layoutIndex then
		EditModeManagerFrame:SelectLayout(layoutIndex)
	end
end
