
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: BONUS_OBJECTIVE_TRACKER_MODULE, next, ObjectiveTrackerBlocksFrame, ObjectiveTrackerFrame, RegisterAttributeDriver, table, WORLD_QUEST_TRACKER_MODULE

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Blizzard ObjectiveTracker")

local OBJECTIVE_TRACKER_BUTTON = nil
local OBJECTIVE_TRACKER_HEADERS = {}
local SHOW_STATE_TEXT = "show"

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	OBJECTIVE_TRACKER_BUTTON = ObjectiveTrackerFrame and ObjectiveTrackerFrame.Header and ObjectiveTrackerFrame.Header.MinimizeButton

	for _, module in next, {
		ObjectiveTrackerFrame,
		ScenarioObjectiveTracker,
		UIWidgetObjectiveTracker,
		CampaignQuestObjectiveTracker,
		QuestObjectiveTracker,
		AdventureObjectiveTracker,
		AchievementObjectiveTracker,
		MonthlyActivitiesObjectiveTracker,
		ProfessionsRecipeTracker,
		BonusObjectiveTracker,
		WorldQuestObjectiveTracker,
	} do
		if module and module.Header then
			table.insert(OBJECTIVE_TRACKER_HEADERS, module.Header)
		end
	end
end

function M:OnEnable()
	self:ConfigureArtwork()
	self:ConfigureButton()
	self:ConfigureFrame()
	self:ConfigureVisibility()
	self:RegisterMessage("SIXUI_CLUTTER_KEY_CHANGED", "ConfigureVisibility")
	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
	self:RegisterMessage("SIXUI_CONFIG_CHANGED")
	self:RegisterMessage("SIXUI_PROFILE_CHANGED")
end

function M:ConfigureArtwork()
	for _, header in next, OBJECTIVE_TRACKER_HEADERS do
		if self.config.showArtwork then
			header:GetRegions():Show()
		else
			header:GetRegions():Hide()
		end
	end
end

function M:ConfigureButton()
	local alpha = (self.config.showButton or self.isClutterKeyDown) and 1 or 0

	if OBJECTIVE_TRACKER_BUTTON then
		OBJECTIVE_TRACKER_BUTTON:SetAlpha(alpha)
	end
end

function M:ConfigureFrame()
	ObjectiveTrackerFrame:SetFrameStrata("BACKGROUND")
end

function M:ConfigureVisibility()
	RegisterAttributeDriver(ObjectiveTrackerFrame, "state-visibility", self:GetStateText())
end

function M:GetStateText()
	return self.config.stateText and self.config.stateText ~= "" and A.FlavourStateText(self.config.stateText) or SHOW_STATE_TEXT
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CLUTTER_STATE_CHANGED(event, state)
	self.isClutterKeyDown = state
	self:ConfigureButton()
end

function M:SIXUI_CONFIG_CHANGED(event, module, config, key, ...)
	if module == self:GetName() then
		if key == "showArtwork" then
			self:ConfigureArtwork()
		elseif key == "showButton" then
			self:ConfigureButton()
		elseif key == "stateText" then
			self:ConfigureVisibility()
		end
	end
end

function M:SIXUI_PROFILE_CHANGED(event)
	self:ConfigureArtwork()
	self:ConfigureButton()
	self:ConfigureVisibility()
end
