
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: unpack, ZoneAbilityFrame

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Blizzard ZoneAbilityFrame")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M.IterateSpellButtons()
	return coroutine.wrap(function ()
		for spellButton in ZoneAbilityFrame.SpellButtonContainer:EnumerateActive() do
			if spellButton then
				coroutine.yield(spellButton)
			end
		end
	end)
end

function M:OnEnable()
	self:CreateHook("SecureHook", ZoneAbilityFrame, "UpdateDisplayedZoneAbilities", "StyleAllButtons")
	self:StyleFrame()
	self:StyleAllButtons()
end

function M:OnDisable()
	self:ResetFrame()
	self:ResetAllButtons()
end

function M:StyleFrame()
	A:GetFrameConfig(ZoneAbilityFrame):DisableDrawLayer("OVERLAY")
end

function M:ResetFrame()
	A:GetFrameConfig(ZoneAbilityFrame):ResetAll()
end

function M:StyleAllButtons()
	for spellButton in self.IterateSpellButtons() do
		self:StyleButton(spellButton)
	end
end

function M:ResetAllButtons()
	for spellButton in self.IterateSpellButtons() do
		self:ResetButton(spellButton)
	end
end

function M:StyleButton(spellButton)
	A:GetFrameConfig(spellButton):CreateBorder(2)
	A:GetFrameConfig(spellButton.Icon):SetTexCoord(unpack(C.ZOOMED_TEX_COORDS[0.3]))
	A:GetFrameConfig(spellButton.NormalTexture):SetTexture(nil)
end

function M:ResetButton(spellButton)
	A:GetFrameConfig(spellButton):ResetAll()
	A:GetFrameConfig(spellButton.Icon):ResetAll()
	A:GetFrameConfig(spellButton.NormalTexture):ResetAll()
end
