
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: unpack

local BuffFrame = BuffFrame
local IsInInstance = IsInInstance
local select = select

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Buff CollapseAndExpandButton")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	self.frameConfig = A:GetFrameConfig(BuffFrame.CollapseAndExpandButton)
end

function M:OnEnable()
	self:RegisterMessage("SIXUI_CONFIG_CHANGED")

	if self.config.autoCollapseBuffs ~= C.Enum.AutoCollapseBuffs.Never then
		self:RegisterEvent("PLAYER_ENTERING_WORLD")
		self:UpdateCollapsed()
	else
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end

	if not self.config.showCollapseAndExpandButton then
		self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
		self:UpdateShown()
	else
		self:UnregisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
	end
end

function M:OnDisable()
	self:ResetCollapsed()
	self:ResetShown()
end

function M:IsCollapsed()
	return not BuffFrame.CollapseAndExpandButton:GetChecked()
end

function M:SetCollapsed(collapsed)
	if self:IsCollapsed() ~= collapsed then
		BuffFrame.CollapseAndExpandButton:Click()
	end
end

function M:UpdateCollapsed()
	self:SetCollapsed((self.config.autoCollapseBuffs == C.Enum.AutoCollapseBuffs.Always) or (self.config.autoCollapseBuffs == C.Enum.AutoCollapseBuffs.WhileInAnInstance and IsInInstance()))
end

function M:ResetCollapsed()
	self:SetCollapsed(false)
end

function M:UpdateShown()
	self.frameConfig:SetAlpha(self.isClutterKeyDown and 1 or 0)
end

function M:ResetShown()
	self.frameConfig:ResetAlpha()
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:PLAYER_ENTERING_WORLD()
	self:UpdateCollapsed()
end

function M:SIXUI_CLUTTER_STATE_CHANGED(event, state)
	self.isClutterKeyDown = state
	self:UpdateShown()
end

function M:SIXUI_CONFIG_CHANGED(event, module, config, key, ...)
	if module == M:GetName() then
		if key == "autoCollapseBuffs" then
			self:ResetCollapsed()
		elseif key == "showCollapseAndExpandButton" then
			self:ResetShown()
		end

		self:OnEnable()
	end
end
