
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: C_GuildInfo, ChatEdit_UpdateHeader, coroutine, DEFAULT_CHAT_FRAME, format, GetChannelName, IsControlKeyDown, IsGuildLeader, IsInGroup, IsInRaid, IsShiftKeyDown, LE_PARTY_CATEGORY_HOME, LE_PARTY_CATEGORY_INSTANCE, next, type

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat Cycle")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.ChatType = {
	{chatType = "SAY", IsAvailable = function () return true end},
	{chatType = "YELL", IsAvailable = function () return true end},
	{chatType = "INSTANCE_CHAT", IsAvailable = function () return IsInGroup(LE_PARTY_CATEGORY_INSTANCE) or IsInRaid(LE_PARTY_CATEGORY_INSTANCE) end},
	{chatType = "PARTY", IsAvailable = function () return IsInGroup(LE_PARTY_CATEGORY_HOME) end},
	{chatType = "RAID", IsAvailable = function () return IsInRaid(LE_PARTY_CATEGORY_HOME) end},
	{chatType = "GUILD", IsAvailable = function () return C_GuildInfo.CanSpeakInGuildChat() end},
	{chatType = "OFFICER", IsAvailable = function () return C_GuildInfo.IsGuildOfficer() or IsGuildLeader() end},
	{chatType = "SMART_WHISPER", IsAvailable = function () return DEFAULT_CHAT_FRAME.editBox:GetAttribute("tellTarget") ~= nil end},
	{chatType = "CHANNEL", channelTarget = 1, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 2, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 3, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 4, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 5, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 6, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 7, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 8, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 9, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 10, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 11, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 12, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 13, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 14, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 15, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 16, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 17, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 18, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 19, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
	{chatType = "CHANNEL", channelTarget = 20, IsAvailable = function (self) return GetChannelName(self.channelTarget) ~= 0 end},
}

function M:OnInitialize()
	for index, entry in next, self.ChatType do
		if type(entry) == "table" then
			self.ChatType[format("%s%s", entry.chatType, entry.channelTarget or "")] = index
		end
	end

	for alias, chatType in next, {
		BN_WHISPER = "SMART_WHISPER",
		WHISPER	= "SMART_WHISPER",
	} do
		self.ChatType[alias] = self.ChatType[chatType]
	end
end

function M:OnEnable()
	self:CreateHook("RawHook", "ChatEdit_CustomTabPressed", true)
	self:CreateHook("RawHook", "ChatEdit_SecureTabPressed", true)
end

function M:GetChatTypeIndex(chatType, channelIndex)
	return self.ChatType[chatType == "CHANNEL" and (chatType .. channelIndex) or chatType] or 0
end

function M:IterateChatTypes(previousIndex, reverse)
	local step = reverse and -1 or 1

	return coroutine.wrap(function ()
		for count = 1, #self.ChatType do
			local index = (previousIndex + (count * step) - 1) % #self.ChatType + 1
			local entry = self.ChatType[index]

			if entry:IsAvailable() then
				coroutine.yield(entry)
			end
		end
	end)
end

function M:CycleChatType(editBox)
	local chatType = editBox:GetAttribute("chatType")
	local channelTarget = editBox:GetAttribute("channelTarget")
	local previousIndex = self:GetChatTypeIndex(chatType, channelTarget)
	local reverse = IsShiftKeyDown()
	local entry = self:IterateChatTypes(previousIndex, reverse)()
	if (not entry) then return end
	editBox:SetAttribute("chatType", entry.chatType)
	editBox:SetAttribute("stickyType", entry.chatType)
	editBox:SetAttribute("channelTarget", entry.channelTarget)
	ChatEdit_UpdateHeader(editBox)
end

function M:ShouldCycleChatType(editBox)
	return not IsControlKeyDown() and not A.IsSlashCommand(editBox:GetText())
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:ChatEdit_CustomTabPressed(editBox)
	if self:ShouldCycleChatType(editBox) then
		self:CycleChatType(editBox)
	end
end

function M:ChatEdit_SecureTabPressed(editBox)
	if IsControlKeyDown() then
		self.hooks.ChatEdit_SecureTabPressed(editBox)
	end
end
