
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: ChatFrame_AddMessageEventFilter, ChatFrame_RemoveMessageEventFilter, DEFAULT_CHAT_FRAME

local next = next
local type = type

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat Filters")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.AddOnPattern = {
	-- BadBoy
	"^|cFF33FF99BadBoy|r",

	-- Blizzard
	"^Dungeon Difficulty set to ",
	"^Loot Specialization set to: ",
	"^Raid Difficulty set to ",
	"^Soulbound with .-%.$",
	"^You have learned a new passive effect: ",
	"^You have learned a new spell: ",
	"^You have unlearned ",

	-- Clique
	"^|cFF33FF99Clique|r:",

	-- GTFO
	"^%[GTFO%]",

	-- Gnosis
	"^|cff33ff99Gnosis|r:",

	-- Skada
	"^|cFF33FF99Skada|r:",

	-- TomCats
	"^|TInterface/AddOns/TomCats/images/tomcat_chat_icon_lg%.blp:",

	-- VuhDo
	"^|cffffe566{VuhDo}|r",

	-- WeakAuras
	"^|cff9900ffWeakAuras:|r",
}

M.LatinEvent = {
	"CHAT_MSG_CHANNEL",
	"CHAT_MSG_SAY",
	"CHAT_MSG_WHISPER",
	"CHAT_MSG_YELL",
}

M.LatinLocale = {
	frFR = true, -- French
	deDE = true, -- German
	enGB = true, enUS = true, -- English
	itIT = true, -- Italian
	ruRU = true, -- Russian
	esES = true, esMX = true, -- Spanish
	ptBR = true, -- Portuguese
}

M.LatinPattern = "[\227-\237]"

M.LatinRegion = {
	[1] = true, -- Americas & Oceania
	[3] = true, -- Europe
}

function M.FilterAddOn(text)
	if type(text) == "string" then
		for _, pattern in next, M.AddOnPattern do
			if text:find(pattern) then
				return true
			end
		end
	end

	return false
end

function M.FilterLatin(chatFrame, event, text)
	if text:find(M.LatinPattern) then
		return true
	end
end

function M.ShouldEnableLatinFilter()
	return M.LatinLocale[C.PLAYER_LOCALE] and M.LatinRegion[C.PLAYER_REGION_ID]
end

function M:OnEnable()
	self:EnableAddOnFilter()
	self:EnableLatinFilter()
end

function M:OnDisable()
	self:DisableLatinFilter()
end

function M:EnableAddOnFilter()
	self:CreateHook("SecureHook", DEFAULT_CHAT_FRAME, "AddMessage", "ChatFrame_AddMessage")
	self:CreateHook("SecureHook", DEFAULT_CHAT_FRAME, "BackFillMessage", "ChatFrame_AddMessage")
	self:ScheduleTimer("DisableAddOnFilter", 15)
end

function M:DisableAddOnFilter()
	self:Unhook(DEFAULT_CHAT_FRAME, "AddMessage")
	self:Unhook(DEFAULT_CHAT_FRAME, "BackFillMessage")
end

function M:EnableLatinFilter()
	if self.ShouldEnableLatinFilter() then
		for _, event in next, self.LatinEvent do
			ChatFrame_AddMessageEventFilter(event, self.FilterLatin)
		end
	end
end

function M:DisableLatinFilter()
	for _, event in next, self.LatinEvent do
		ChatFrame_RemoveMessageEventFilter(event, self.FilterLatin)
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:ChatFrame_AddMessage(chatFrame, text, ...)
	if self.FilterAddOn(text) then
		chatFrame:RemoveMessagesByPredicate(function (t) return t == text end)
	end
end
