
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: _G, ChatFontNormal, coroutine, next, NUM_CHAT_WINDOWS, string

local CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA = CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA
local SixUI_DisplayGroup1 = SixUI_DisplayGroup1
local SixUI_EditBox = SixUI_EditBox

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat Frame")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.skinnedChatFrames = {}

function M.IterateChatFrames()
	return coroutine.wrap(function ()
		for index = 1, NUM_CHAT_WINDOWS do
			local chatFrameName = "ChatFrame" .. index
			local chatFrame = _G[chatFrameName]
			local chatTab = _G[chatFrameName .. "Tab"]

			if chatFrame then
				coroutine.yield(chatFrame, chatTab)
			end
		end
	end)
end

function M:OnEnable()
	self:CreateHook("SecureHook", "ChatEdit_OnEditFocusGained")
	self:CreateHook("SecureHook", "ChatEdit_OnEditFocusLost")
	self:CreateHook("SecureHook", "ChatEdit_UpdateHeader")
	self:CreateHook("SecureHook", "FCF_CopyChatSettings", "FCF_ResetChatWindow")
	self:CreateHook("SecureHook", "FCF_DockFrame")
	self:CreateHook("SecureHook", "FCF_ResetChatWindow")
	self:CreateHook("SecureHook", "FCF_SetChatWindowFontSize")
	self:CreateHook("SecureHook", "FCF_UnDockFrame")
	self:CreateHook("SecureHook", "ResetChatWindows", "Update")
	self:RegisterMessage("SIXUI_CONFIG_CHANGED")
	self:RegisterMessage("SIXUI_LAYOUT_CHANGED", "Update")
	self:RegisterMessage("SIXUI_PROFILE_CHANGED", "Update")
	self:Update()
end

function M:OnDisable()
	self:ResetAllChatFrames()
	self:ResetRelated()
end

function M:Update()
	self:ResetAllChatFrames()
	self:StyleAllChatFrames()
	self:ResetRelated()
	self:StyleRelated()
end

function M:UpdateFonts()
	for chatFrame in self.IterateChatFrames() do
		A:GetFrameConfig(chatFrame):SetFont(C.TEXT_MEDIUM:GetFont())
	end
end

function M:StyleRelated()
	A:GetFrameConfig("ChatFrameChannelButton"):SetParent("ChatFrame1Tab")
	A:GetFrameConfig("ChatFrameMenuButton"):SetShown(false)
	A:GetFrameConfig("ChatFrameToggleVoiceDeafenButton"):SetParent("ChatFrame1Tab")
	A:GetFrameConfig("ChatFrameToggleVoiceMuteButton"):SetParent("ChatFrame1Tab")
	A:GetFrameConfig("QuickJoinToastButton"):SetShown(false)
	A:GetFrameConfig("SixUI_ChatFrame"):SetShown(true)
end

function M:ResetRelated()
	A:GetFrameConfig("ChatFrameChannelButton"):ResetAll()
	A:GetFrameConfig("ChatFrameMenuButton"):ResetAll()
	A:GetFrameConfig("ChatFrameToggleVoiceDeafenButton"):ResetAll()
	A:GetFrameConfig("ChatFrameToggleVoiceMuteButton"):ResetAll()
	A:GetFrameConfig("QuickJoinToastButton"):ResetAll()
	A:GetFrameConfig("SixUI_ChatFrame"):ResetAll()
end

function M:StyleAllChatFrames()
	for chatFrame, chatTab in self.IterateChatFrames() do
		self:StyleChatFrame(chatFrame, chatTab)
	end
end

function M:ResetAllChatFrames()
	for chatFrame, chatTab in next, self.skinnedChatFrames do
		self:ResetChatFrame(chatFrame, chatTab)
	end
end

function M:StyleChatFrame(chatFrame, chatTab)
	if chatFrame and not self.skinnedChatFrames[chatFrame] then
		self.skinnedChatFrames[chatFrame] = chatTab
		self:FCF_UnDockFrame(chatFrame)
		self:StyleChatEditBox(chatFrame.editBox)
		self:StyleChatTab(chatTab)
		A:GetFrameConfig(chatFrame.buttonFrame):SetShown(false)
		A:GetFrameConfig(chatFrame.ResizeButton):ClearAllPoints():SetPoint("BOTTOMRIGHT", chatFrame, "BOTTOMRIGHT", C.CHATFRAME_INSET, -C.CHATFRAME_INSET)
		A:GetFrameConfig(chatFrame.ScrollBar):SetShown(false)
		A:GetFrameConfig(chatFrame.ScrollToBottomButton):SetShown(false)
		A:GetFrameConfig(chatFrame):SetFont(C.TEXT_MEDIUM:GetFont()):DisableDrawLayer("BACKGROUND"):DisableDrawLayer("BORDER")
		chatFrame:SetClampedToScreen(false)
		chatFrame:SetFading(false)
	end
end

function M:ResetChatFrame(chatFrame, chatTab)
	if chatFrame and self.skinnedChatFrames[chatFrame] then
		self.skinnedChatFrames[chatFrame] = nil
		self:ResetChatEditBox(chatFrame.editBox)
		self:ResetChatTab(chatTab)
		A:GetFrameConfig(chatFrame.buttonFrame):ResetAll()
		A:GetFrameConfig(chatFrame.ResizeButton):ResetAll()
		A:GetFrameConfig(chatFrame.ScrollBar):ResetAll()
		A:GetFrameConfig(chatFrame.ScrollToBottomButton):ResetAll()
		A:GetFrameConfig(chatFrame):ResetAll()
		chatFrame:SetClampedToScreen(true)
		chatFrame:SetFading(true)
	end
end

function M:StyleChatEditBox(editBox)
	if editBox then
		A:GetFrameConfig(editBox):SetParent(SixUI_EditBox):SetAllPoints(SixUI_EditBox):DisableDrawLayer("BACKGROUND"):DisableDrawLayer("BORDER")
		editBox:SetAltArrowKeyMode(false)
		editBox:SetFontObject(C.TEXT_MEDIUM)
		editBox.header:SetFontObject(C.TEXT_MEDIUM)
	end
end

function M:ResetChatEditBox(editBox)
	if editBox then
		A:GetFrameConfig(editBox):ResetAll()
		editBox:SetAltArrowKeyMode(true)
		editBox:SetFontObject(ChatFontNormal)
		editBox.header:SetFontObject(ChatFontNormal)
	end
end

function M:StyleChatTab(chatTab)
	if chatTab then
		self:CreateHook("SecureHook", chatTab, "SetAlpha", "ChatTab_SetAlpha")
		self:ChatTab_SetAlpha(chatTab, chatTab:GetAlpha())
	end
end

function M:ResetChatTab(chatTab)
	if chatTab then
		self:Unhook(chatTab, "SetAlpha")
		chatTab:SetAlpha(chatTab:GetParent().hasBeenFaded and chatTab.mouseOverAlpha or chatTab.noMouseAlpha)
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CONFIG_CHANGED(event, key, ...)
	local c, g, k = string.split(".", key)

	if key == self:GetName() then
		self:Update()
	elseif c == "font" then
		self:UpdateFonts()
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

-- Use a hook to show the edit box when it gains focus.
function M:ChatEdit_OnEditFocusGained()
	SixUI_EditBox:SetAlpha(1)
	SixUI_DisplayGroup1:Hide()
end

-- Use a hook to hide the edit box when it loses focus.
function M:ChatEdit_OnEditFocusLost()
	SixUI_EditBox:SetAlpha(0)
	SixUI_DisplayGroup1:Show()
end

-- Use a hook to synchronize the edit box's border color with SixUI.
function M:ChatEdit_UpdateHeader(editBox)
	SixUI_EditBox:GetBorder():SetColor(editBox:GetTextColor())
end

-- Hide a chat tab if the mouse is not over it and it's not alerting.
function M:ChatTab_SetAlpha(chatTab, alpha)
	if alpha ~= 0 and alpha ~= chatTab.mouseOverAlpha and alpha ~= CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA then
		chatTab:SetAlpha(0)
	end
end

-- Use a hook to remove borders from chat frames when they are docked.
function M:FCF_DockFrame(chatFrame)
	B:RemoveBorder(chatFrame)
end

function M:FCF_ResetChatWindow(chatFrame)
	self:ResetChatFrame(chatFrame)
	self:StyleChatFrame(chatFrame)
end

function M:FCF_SetChatWindowFontSize(_, chatFrame)
	if not chatFrame then
		chatFrame = FCF_GetCurrentChatFrame()
	end

	self:ResetChatFrame(chatFrame)
	self:StyleChatFrame(chatFrame)
end

-- Use a hook to create borders on chat frames when they are undocked.
function M:FCF_UnDockFrame(chatFrame)
	if not chatFrame.isDocked then
		A:GetFrameConfig(chatFrame):CreateBorder(C.CHATFRAME_INSET)
	end
end
