
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: _G, CHAT_FRAMES, CreateFrame, Mixin, table, UIFrameFadeIn, UIFrameFadeOut

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat ScrollIndicator")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.buttonPool = CreateFramePool("BUTTON", UIParent, "SixUI_BottomButtonTemplate")

M.buttons = {}

function M:OnInitialize()
	self.IterateChatFrames = A:GetModule("Chat Frame").IterateChatFrames
end

function M:OnEnable()
	self:ConfigureAllChatFrames()
end

function M:OnDisable()
	self.buttonPool:ReleaseAll()
end

function M:ConfigureAllChatFrames()
	for chatFrame in self.IterateChatFrames() do
		self:ConfigureChatFrame(chatFrame)
	end
end

function M:ConfigureChatFrame(chatFrame)
	self:CreateHook("SecureHook", chatFrame, "SetScrollOffset", "ChatFrame_SetScrollOffset")
	self:ChatFrame_SetScrollOffset(chatFrame, chatFrame.scrollOffset)
end

function M:ShowScrollIndicator(chatFrame)
	if not self.buttonPool:IsActive(chatFrame.scrollIndicator) then
		chatFrame.scrollIndicator = self.buttonPool:Acquire()
		chatFrame.scrollIndicator:SetParent(chatFrame)
		chatFrame.scrollIndicator:SetPoint("BOTTOMRIGHT")
		chatFrame.scrollIndicator:Show()
	end
end

function M:HideScrollIndicator(chatFrame)
	if self.buttonPool:IsActive(chatFrame.scrollIndicator) then
		self.buttonPool:Release(chatFrame.scrollIndicator)
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:ChatFrame_SetScrollOffset(chatFrame, scrollOffset)
	if scrollOffset > 0 then
		self:ShowScrollIndicator(chatFrame)
	else
		self:HideScrollIndicator(chatFrame)
	end
end
