
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

local coroutine = coroutine
local math = math
local Mixin = Mixin
local next = next

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Core RaidFrameConfig")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.GroupSize = {
	Five = 5,
	Ten = 10,
	Fifteen = 15,
	Twenty = 20,
	TwentyFive = 25,
	Thirty = 30,
	ThirtyFive = 35,
	Fourty = 40,
}

M.GroupSizeLabel = {
	[M.GroupSize.Five] = "5 Players",
	[M.GroupSize.Ten] = "10 Players",
	[M.GroupSize.Fifteen] = "15 Players",
	[M.GroupSize.Twenty] = "20 Players",
	[M.GroupSize.TwentyFive] = "25 Players",
	[M.GroupSize.Thirty] = "30 Players",
	[M.GroupSize.ThirtyFive] = "35 Players",
	[M.GroupSize.Fourty] = "40 Players",
}

M.RaidFrameConfigMixin = {
	GetPoint = function (self)
		return self.point, self.relativeTo, self.relativePoint, self.offsetX, self.offsetY
	end,
	IsEqualToBarSize = function (self, width, height)
		return self.width == width and self.height == height
	end,
	IsEqualToPoint = function (self, point, relativeTo, relativePoint, offsetX, offsetY)
		return self.point == point and self.relativeTo == relativeTo and self.relativePoint == relativePoint and self.offsetX == offsetX and self.offsetY == offsetY
	end,
}

function M:GetRaidFrameConfig(groupSize)
	return Mixin(self.config.raidFrameConfigs[A:GetActiveLayout()][math.min(math.ceil(groupSize / 5) * 5, 40)], self.RaidFrameConfigMixin)
end

function A:GetRaidFrameConfig(groupSize)
	return M:GetRaidFrameConfig(groupSize)
end

function M:IterateRaidFrameConfigs()
	return coroutine.wrap(function ()
		for _, groupSize in next, {
			self.GroupSize.Five,
			self.GroupSize.Ten,
			self.GroupSize.Fifteen,
			self.GroupSize.Twenty,
			self.GroupSize.TwentyFive,
			self.GroupSize.Thirty,
			self.GroupSize.ThirtyFive,
			self.GroupSize.Fourty,
		} do
			coroutine.yield(groupSize, self:GetRaidFrameConfig(groupSize), self.GroupSizeLabel[groupSize])
		end
	end)
end

function A:IterateRaidFrameConfigs()
	return M:IterateRaidFrameConfigs()
end
