
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: coroutine, LibStub, next, unpack

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Integration AdiBags"):WithDependency("AdiBags")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.ADDON_NAME = format("%s_%s", A:GetName(), M:GetName())

M.MESSAGES_TO_REGISTER = {
	"AdiBags_AcquireButton",
	"AdiBags_BagFrameCreated",
	"AdiBags_ReleaseButton",
	"AdiBags_UpdateButton",
	"AdiBags_UpdateBorder",
}

M.frames = {}

M.items = {}

function M.IterateFrames()
	return coroutine.wrap(function ()
		for _, bag in M.AdiBags:IterateBags() do
			if bag.frame then
				coroutine.yield(bag.frame)
			end
		end
	end)
end

function M.IterateItems()
	return coroutine.wrap(function ()
		for _, pool in next, {M.AdiBags:GetPool("BankItemButton") or {}, M.AdiBags:GetPool("ItemButton") or {}} do
			if pool.IterateActiveObjects then
				for item in pool:IterateActiveObjects() do
					coroutine.yield(item)
				end
			end
		end
	end)
end

function M:OnEnable()
	self.ABEvent = LibStub('ABEvent-1.0')
	if (not self.ABEvent) then return end
	self.AdiBags = LibStub("AceAddon-3.0"):GetAddon("AdiBags")
	if (not self.AdiBags) then return end

	for _, message in next, self.MESSAGES_TO_REGISTER do
		self.ABEvent.RegisterMessage(self.ADDON_NAME, message, function (...) self[message](self, ...) end)
	end

	self:StyleAllFrames()
	self:StyleAllItems()
end

function M:OnDisable()
	if self.ABEvent then
		self.ABEvent.UnregisterAllMessages(self.ADDON_NAME)
	end

	self:ResetAllFrames()
	self:ResetAllItems()
end

function M:StyleAllFrames()
	for frame in self.IterateFrames() do
		self:StyleFrame(frame)
	end
end

function M:ResetAllFrames()
	for frame in next, self.frames do
		self:ResetFrame(frame)
	end
end

function M:StyleFrame(frame)
	self.frames[frame] = true
	self:CreateBorder(frame)
	frame:DisableDrawLayer("BACKGROUND")
	frame:DisableDrawLayer("BORDER")
end

function M:ResetFrame(frame)
	self.frames[frame] = nil
	self:RemoveBorder(frame)
	frame:EnableDrawLayer("BACKGROUND")
	frame:EnableDrawLayer("BORDER")
end

function M:StyleAllItems()
	for item in self.IterateItems() do
		self:StyleItem(item)
	end
end

function M:ResetAllItems()
	for item in next, self.items do
		self:ResetItem(item)
	end
end

function M:StyleItem(item)
	self.items[item] = true
	self:CreateBorder(item, C.BORDER_PADDING):SetBackground(false)
	self:StyleIcon(item.IconQuestTexture)
	self:StyleIcon(item.IconTexture)
	self:StyleAllRegions(item)
end

function M:ResetItem(item)
	self.items[item] = nil
	self:RemoveBorder(item)
	self:ResetIcon(item.IconQuestTexture)
	self:ResetIcon(item.IconTexture)
	self:ResetAllRegions(item)
end

function M:StyleIcon(icon)
	icon:SetTexCoord(unpack(C.ZOOMED_TEX_COORDS[0.3]))
end

function M:ResetIcon(icon)
	icon:SetTexCoord(0, 1, 0, 1)
end

function M:StyleAllRegions(item)
	item.IconBorder:Hide()
	item.NormalTexture:Hide()

	if item.hasItem then
		item:EnableDrawLayer("BORDER")
		item:GetBorder():SetRGB(item.IconBorder:GetVertexColor())
	else
		item:DisableDrawLayer("BORDER")
		item:GetBorder():SetRGB()
	end
end

function M:ResetAllRegions(item)
	item.IconBorder:Show()
	item.IconQuestTexture:Show()
	item.NormalTexture:Show()
	item:EnableDrawLayer("BORDER")
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:AdiBags_AcquireButton(event, item)
	self:StyleItem(item)
end

function M:AdiBags_BagFrameCreated(event, bag)
	self:StyleFrame(bag.frame)
end

function M:AdiBags_ReleaseButton(event, item)
	self:ResetItem(item)
end

function M:AdiBags_UpdateButton(event, item)
	self:StyleItem(item)
end

function M:AdiBags_UpdateBorder(event, item)
	if self.items[item] then
		self:StyleAllRegions(item)
	end
end
