local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database

local M = A:NewModule("Integration BigWigs"):WithDependency("BigWigs")

function M.IsApiAvailable()
	return not not BigWigsAPI
end

function M.IsSavedVariableAvailable()
	return not not (BigWigs3DB and BigWigs3DB.profileKeys and BigWigs3DB.profiles)
end

function M.IsDatabaseAvailable()
	return not not (BigWigs and BigWigs.db)
end

function M.IsStyleRegistered(styleName)
	return BigWigsAPI:GetBarStyle(styleName) ~= nil
end

function M:OnInitialize()
	self:RegisterStyle()
end

function M:OnEnable()
	self:RegisterStyle()
end

function M:RegisterStyle()
	if self.IsApiAvailable() and not self.IsStyleRegistered(self.STYLE_NAME) then
		BigWigsAPI:RegisterBarStyle(self.STYLE_NAME, self.STYLE)
	end
end

-------------------------------------------------------------------------------
-- Skin
-------------------------------------------------------------------------------

M.BAR_SPACING = 0

M.BAR_PADDING = C.BORDER_PADDING

M.BAR_HEIGHT = 25

M.ICON_SIZE = M.BAR_HEIGHT

M.STYLE_NAME = "SixUI"

M.STYLE = {
	apiVersion = 1,
	fontSizeEmphasized = 12,
	fontSizeNormal = 12,
	version = 1,
}

function M.STYLE.ApplyStyle(bar)
	local texture = bar:GetIcon()

	B:CreateBorder(bar, M.BAR_PADDING, M.BAR_PADDING, M.BAR_PADDING, M.ICON_SIZE + M.BAR_PADDING)

	bar:SetHeight(M.BAR_HEIGHT)
	bar:SetTexture(C.LIGHT_TEXTURE_PATH)
	bar.candyBarBackground:SetAlpha(0)
	bar.candyBarLabel:ClearAllPoints()
	bar.candyBarLabel:SetPoint("LEFT", bar, "LEFT", 6, 0)
	bar.candyBarDuration:ClearAllPoints()
	bar.candyBarDuration:SetPoint("RIGHT", bar, "RIGHT", -6, 0)

	if texture then
		local icon = bar.candyBarIconFrame

		bar:SetIcon(nil)
		icon:SetTexture(texture)
		icon:ClearAllPoints()
		icon:SetPoint("BOTTOMRIGHT", bar, "BOTTOMLEFT")
		icon:SetSize(M.ICON_SIZE, M.ICON_SIZE)
		icon:Show()
		bar:Set("bigwigs:restoreicon", texture)
	end
end

function M.STYLE.BarStopped(bar)
	local texture = bar:Get("bigwigs:restoreicon")

	B:RemoveBorder(bar)

	bar.candyBarBackground:SetAlpha(1)
	bar.candyBarLabel:ClearAllPoints()
	bar.candyBarLabel:SetPoint("TOPLEFT", bar.candyBarBar, "TOPLEFT", 2, 0)
	bar.candyBarLabel:SetPoint("BOTTOMRIGHT", bar.candyBarBar, "BOTTOMRIGHT", -2, 0)
	bar.candyBarDuration:ClearAllPoints()
	bar.candyBarDuration:SetPoint("TOPLEFT", bar.candyBarBar, "TOPLEFT", 2, 0)
	bar.candyBarDuration:SetPoint("BOTTOMRIGHT", bar.candyBarBar, "BOTTOMRIGHT", -2, 0)

	if texture then
		bar:SetIcon(texture)
		bar:Set("bigwigs:restoreicon", nil)
	end
end

function M.STYLE.GetSpacing(bar)
	return M.BAR_SPACING + (M.BAR_PADDING * 2)
end

function M.STYLE.GetStyleName()
	return M.STYLE_NAME
end

M:RegisterStyle()

-------------------------------------------------------------------------------
-- Setup
-------------------------------------------------------------------------------

Mixin(M, C.SetupMixin)

function M.GetProfileKey()
	return C.PLAYER_NAME .. " - " .. GetRealmName()
end

function M:CreateProfileFromString(profileName, profileString, successFn)
	BigWigsAPI:ImportProfileString("SixUI", profileString, profileName, successFn)
end

function M:DeleteProfile(profileName)
	if self.IsDatabaseAvailable() then
		BigWigs.db:DeleteProfile(profileName, true)
	elseif self.IsSavedVariableAvailable() then
		BigWigs3DB.profiles[profileName] = nil
	end
end

function M:GetActiveProfile(profileName)
	if self.IsDatabaseAvailable() then
		return BigWigs.db:GetCurrentProfile()
	elseif self.IsSavedVariableAvailable() then
		return BigWigs3DB.profileKeys[self.GetProfileKey()]
	end
end

function M:HasProfile(profileName)
	if self.IsDatabaseAvailable() then
		return A.table.any(BigWigs.db:GetProfiles(), function (p) return p == profileName end)
	elseif self.IsSavedVariableAvailable() then
		return BigWigs3DB.profiles[profileName] ~= nil
	else
		return false
	end
end

function M:IsProfileActive(profileName)
	return self:GetActiveProfile() == profileName
end

function M:SetActiveProfile(profileName)
	if self.IsDatabaseAvailable() then
		BigWigs.db:SetProfile(profileName)
	elseif self.IsSavedVariableAvailable() then
		BigWigs3DB.profileKeys[M.GetProfileKey()] = profileName
	end
end
