
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: Dominos, next

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Integration Dominos"):WithDependency("Dominos")
local Masque = LibStub('Masque', true)

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	local ButtonThemer = Dominos:GetModule("ButtonThemer")
	self:CreateHook("SecureHook", ButtonThemer, "Register", "StyleAllButtons")
	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
	self:StyleAllButtons()
end

function M:OnDisable()
	self:SIXUI_CLUTTER_STATE_CHANGED(nil, false)
	self:UnstyleAllButtons()
end

-- Apply SixUI's style to all buttons.
function M:StyleAllButtons()
	if Masque then
		for _, group in next, Masque:Group("Dominos").SubList do
			for button in next, group.Buttons do
				self:StyleButton(button)
			end
		end
	end
end

-- Apply SixUI's style to a button.
function M:StyleButton(button)
	if button then
		B:CreateBorder(button, C.BORDER_PADDING)
	end
end

-- Remove SixUI's style from all buttons.
function M:UnstyleAllButtons()
	if Masque then
		for _, group in next, Masque:Group("Dominos").SubList do
			for button in next, group.Buttons do
				B:RemoveBorder(button)
			end
		end
	end
end

-- Show binding text if it hasn't been set by the player.
function M:ShowBindingText(state)
	if (state and not Dominos:ShowBindingText()) or (not state and self.showBindingText) then
		self.showBindingText = state
		Dominos:SetShowBindingText(state)
	end
end

-- Show macro text if it hasn't been set by the player.
function M:ShowMacroText(state)
	if (state and not Dominos:ShowMacroText()) or (not state and self.showMacroText) then
		self.showMacroText = state
		Dominos:SetShowMacroText(state)
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CLUTTER_STATE_CHANGED(event, state)
	if self.config.showBindingText then
		self:ShowBindingText(state)
	end

	if self.config.showMacroText then
		self:ShowMacroText(state)
	end
end
