local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database

local M = A:NewModule("Integration Grid2"):WithDependency("Grid2")

M.Override = {
	frameBorder = 1,
	frameBorderColor = {r = 0, g = 0, b = 0, a = 1},
	frameBorderDistance = -1,
	frameColor = {r = 0, g = 0, b = 0, a = 0},
	frameContentColor = {r = 0, g = 0, b = 0, a = 0},
	frameHeight = 1,
	frameWidth = 1,
	Padding = 4,
	Spacing = 0,
}

M.Status = {
	["dungeon-role"] = {
		{configKey = "useHighQualityIcons", hookType = "RawHook", hookArgs = {"GetIcon", "DungeonRole_GetIcon"}},
		{configKey = "useHighQualityIcons", hookType = "RawHook", hookArgs = {"GetTexCoord", "DungeonRole_GetTexCoord"}},
		{configKey = "useClutterKey", hookType = "RawHook", hookArgs = {"IsActive", "DungeonRole_IsActive"}},
	},
	["raid-icon-player"] = {
		{configKey = "useAlternativeRaidIcons", hookType = "RawHook", hookArgs = {"GetIcon", "Status_GetIcon"}},
	},
	["raid-icon-target"] = {
		{configKey = "useAlternativeRaidIcons", hookType = "RawHook", hookArgs = {"GetIcon", "Status_GetIcon"}},
	},
	["ready-check"] = {
		{configKey = "useHighQualityIcons", hookType = "RawHook", hookArgs = {"GetIcon", "Status_GetIcon"}},
		{configKey = "useHighQualityIcons", hookType = "RawHook", hookArgs = {"GetTexCoord", "Status_GetTexCoord"}},
	},
	["summon"] = {
		{configKey = "useHighQualityIcons", hookType = "RawHook", hookArgs = {"GetIcon", "Summon_GetIcon"}},
		{configKey = "useHighQualityIcons", hookType = "RawHook", hookArgs = {"GetTexCoord", "Summon_GetTexCoord"}},
	},
}

M.databaseKeyEnabled = {}

function M.IterateButtons()
	return coroutine.wrap(function ()
		for _, header in next, Grid2LayoutFrame and {Grid2LayoutFrame:GetChildren()} or {} do
			if (header.GetName and header:GetName() or ""):match("Grid2LayoutHeader%d+") then
				for _, button in next, {header:GetChildren()} do
					if (button.GetName and button:GetName() or ""):match("Grid2LayoutHeader%d+UnitButton%d+") then
						coroutine.yield(button)
					end
				end
			end
		end
	end)
end

function M.RefreshGrid2Theme()
	A:RunSecure(Grid2.RefreshTheme, Grid2.RefreshTheme, Grid2)
end

function M.RefreshGrid2ThemeWithDelay(...)
	M:CancelTimer(M.refreshGrid2ThemeTimer)
	M.refreshGrid2ThemeTimer = M:ScheduleTimer("RefreshGrid2Theme", 0.01, ...)
end

function M.UpdateGrid2Indicators()
	Grid2Frame:UpdateIndicators()
end

function M.UpdateGrid2IndicatorsWithDelay(...)
	M:CancelTimer(M.updateGrid2IndicatorsTimer)
	M.updateGrid2IndicatorsTimer = M:ScheduleTimer("UpdateGrid2Indicators", 0.01, ...)
end

function M:OnInitialize()
	self.DungeonRoleIcon = A.AtlasToFile("groupfinder-icon-role-large-dps")

	self.DungeonRoleTexCoord = setmetatable({
		DAMAGER = {A.AtlasToTexCoord("groupfinder-icon-role-large-dps")},
		HEALER = {A.AtlasToTexCoord("groupfinder-icon-role-large-heal")},
		TANK = {A.AtlasToTexCoord("groupfinder-icon-role-large-tank")},
	}, {__index = function () return {} end})

	self.StatusIcon = setmetatable({
		["Interface\\RaidFrame\\ReadyCheck-NotReady"] = A.AtlasToFile("ui-lfg-declinemark-raid"),
		["Interface\\RaidFrame\\ReadyCheck-Ready"] = A.AtlasToFile("ui-lfg-readymark-raid"),
		["Interface\\RaidFrame\\ReadyCheck-Waiting"] = A.AtlasToFile("ui-lfg-pendingmark-raid"),
		["Interface\\TargetingFrame\\UI-RaidTargetingIcon_1"] = [[Interface/AddOns/SixUI/Resources/Textures/RaidTargetIcon/1]],
		["Interface\\TargetingFrame\\UI-RaidTargetingIcon_2"] = [[Interface/AddOns/SixUI/Resources/Textures/RaidTargetIcon/2]],
		["Interface\\TargetingFrame\\UI-RaidTargetingIcon_3"] = [[Interface/AddOns/SixUI/Resources/Textures/RaidTargetIcon/3]],
		["Interface\\TargetingFrame\\UI-RaidTargetingIcon_4"] = [[Interface/AddOns/SixUI/Resources/Textures/RaidTargetIcon/4]],
		["Interface\\TargetingFrame\\UI-RaidTargetingIcon_5"] = [[Interface/AddOns/SixUI/Resources/Textures/RaidTargetIcon/5]],
		["Interface\\TargetingFrame\\UI-RaidTargetingIcon_6"] = [[Interface/AddOns/SixUI/Resources/Textures/RaidTargetIcon/6]],
		["Interface\\TargetingFrame\\UI-RaidTargetingIcon_7"] = [[Interface/AddOns/SixUI/Resources/Textures/RaidTargetIcon/7]],
		["Interface\\TargetingFrame\\UI-RaidTargetingIcon_8"] = [[Interface/AddOns/SixUI/Resources/Textures/RaidTargetIcon/8]],
	}, {__index = function (_, k) return k end})

	self.StatusTexCoord = setmetatable({
		["Interface\\RaidFrame\\ReadyCheck-NotReady"] = {A.AtlasToTexCoord("ui-lfg-declinemark-raid")},
		["Interface\\RaidFrame\\ReadyCheck-Ready"] = {A.AtlasToTexCoord("ui-lfg-readymark-raid")},
		["Interface\\RaidFrame\\ReadyCheck-Waiting"] = {A.AtlasToTexCoord("ui-lfg-pendingmark-raid")},
	}, {__index = function () return {} end})

	self.SummonIcon = A.AtlasToFile("Raid-Icon-SummonPending")

	self.SummonTexCoord = setmetatable({
		[1] = {A.AtlasToTexCoord("Raid-Icon-SummonPending")},
		[2] = {A.AtlasToTexCoord("Raid-Icon-SummonAccepted")},
		[3] = {A.AtlasToTexCoord("Raid-Icon-SummonDeclined")},
	}, {__index = function () return {} end})
end

function M:OnEnable()
	self:CreateHook("SecureHook", "CreateFrame")
	self:CreateHook("SecureHook", Grid2, "LoadConfig", "Update")
	self:CreateHook("RawHook", Grid2, "MediaFetch", "Grid2_MediaFetch")
	self:CreateHook("SecureHook", Grid2, "ReloadTheme", "Update")
	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED", "OnClutterStateChanged")
	self:RegisterMessage("SIXUI_CONFIG_CHANGED", "OnConfigChanged")
	self:RegisterMessage("SIXUI_LAYOUT_CHANGED", "Update")
	self:RegisterMessage("SIXUI_PROFILE_CHANGED", "Update")
	self:Update()
end

function M:OnDisable()
	self:OverrideAllDatabases(true)
	self:OverrideAllStatuses(true)
	self:ResetAllFrames()
	self:ResetAllButtons()
	self:UpdatePanelPosition()
	self:RefreshGrid2Theme()
end

function M:Update()
	self:OverrideAllDatabases()
	self:OverrideAllStatuses()
	self:ResetAllFrames()
	self:SkinAllFrames()
	self:ResetAllButtons()
	self:SkinAllButtons()
	self:UpdatePanelPosition()
	self:UpdateButtonSize()
end

function M:OnClutterStateChanged(event, state)
	self.isClutterKeyDown = state

	if self.config.showRoleIconsWithClutterKey then
		self.UpdateGrid2Indicators()
	end
end

function M:OnConfigChanged(event, module, config, key, ...)
	if module == "Core RaidFrameConfig" then
		if key == "width" or key == "height" then
			self:UpdateButtonSize()
		else
			self:UpdatePanelPosition()
		end
	elseif module == self:GetName() then
		self:Update()
	end
end

-------------------------------------------------------------------------------
-- Skin
-------------------------------------------------------------------------------

M.skinnedButtons = {}

M.skinnedFrames = {}

function M:OverrideAllDatabases(disable)
	self:OverrideDatabase(Grid2Frame.db, disable)
	self:OverrideDatabase(Grid2Layout.db, disable)
end

function M:OverrideDatabase(db, disable)
	local profile = db.profile.original or db.profile

	db.profile = disable and profile or A.CreateProxyTable(profile, {
		frameBorder = self.config.skinRaidFrames and self.Override.frameBorder or nil,
		frameBorderColor = self.config.skinRaidFrames and self.Override.frameBorderColor or nil,
		frameBorderDistance = self.config.skinRaidFrames and self.Override.frameBorderDistance or nil,
		frameColor = self.config.skinRaidFrames and self.Override.frameColor or nil,
		frameContentColor = self.config.skinRaidFrames and self.Override.frameContentColor or nil,
		frameHeight = self.config.useRaidFrameConfig and self.config.controlButtonSize and self.Override.frameHeight or nil,
		frameWidth = self.config.useRaidFrameConfig and self.config.controlButtonSize and self.Override.frameWidth or nil,
		Padding = self.config.skinRaidFrames and self.Override.Padding or nil,
		Spacing = self.config.skinRaidFrames and self.Override.Spacing or nil,
	}, true)

	self:RefreshGrid2ThemeWithDelay()
end

function M:OverrideAllStatuses(unhook)
	for statusName in next, self.Status do
		self:OverrideStatus(Grid2:GetStatusByName(statusName), unhook)
	end
end

function M:OverrideStatus(status, unhook)
	if status then
		for _, data in next, self.Status[status.name] do
			if not unhook and self.config[data.configKey] then
				if self:CreateHook(data.hookType, status, unpack(data.hookArgs)) then
					self.UpdateGrid2IndicatorsWithDelay()
				end
			else
				if self:Unhook(status, unpack(data.hookArgs)) then
					self.UpdateGrid2IndicatorsWithDelay()
				end
			end
		end
	end
end

function M:SkinAllFrames()
	self:SkinFrame(Grid2LayoutFrame)
end

function M:ResetAllFrames()
	self:ResetFrame(Grid2LayoutFrame)
end

function M:SkinFrame(frame)
	if frame and not self.skinnedFrames[frame] and self.config.skinRaidFrames then
		self.skinnedFrames[frame] = true
		self:SkinFrameBack(frame.frameBack)
	end
end

function M:ResetFrame(frame)
	if frame and self.skinnedFrames[frame] then
		self.skinnedFrames[frame] = nil
		self:ResetFrameBack(frame.frameBack)
	end
end

function M:SkinFrameBack(frameBack)
	A:GetFrameConfig(frameBack):SetShown(false)
end

function M:ResetFrameBack(frameBack)
	A:GetFrameConfig(frameBack):ResetAll()
end

function M:SkinAllButtons()
	for button in self.IterateButtons() do
		self:SkinButton(button)
	end
end

function M:ResetAllButtons()
	for button in next, self.skinnedButtons do
		self:ResetButton(button)
	end
end

function M:SkinButton(button)
	if not self.skinnedButtons[button] and self.config.skinRaidFrames then
		self.skinnedButtons[button] = true
		self:CreateBorder(button, C.BORDER_PADDING):SetIgnoreParentAlpha(true)
		self:CreateHook("SecureHook", button, "SetBackdropBorderColor", "UpdateButton", true)
		self:UpdateButton(button, button:GetBackdropBorderColor())
		self:UpdateButtonSizeWithDelay()
	end
end

function M:UpdateButton(button, r, g, b, a)
	if self.skinnedButtons[button] then
		button:GetBorder():SetColor(unpack((r ~= 0 or g ~= 0 or b ~= 0) and a ~= 0 and {r, g, b} or {}))
	end
end

function M:ResetButton(button)
	if self.skinnedButtons[button] then
		self.skinnedButtons[button] = nil
		self:RemoveBorder(button)
		self:UpdateButtonSizeWithDelay()
	end
end

-------------------------------------------------------------------------------
-- Raid Frame Config
-------------------------------------------------------------------------------

function M.GetGroupSize()
	return #A.FilterIterator(M.IterateButtons, function (b) return b.unit ~= nil end)
end

function M:UpdatePanelPosition()
	if self.config.useRaidFrameConfig and self.config.controlPanelPosition then
		local raidFrameConfig = A:GetRaidFrameConfig(self.GetGroupSize())

		if not raidFrameConfig:IsEqualToPoint(Grid2LayoutFrame:GetPoint()) then
			A:GetFrameConfig(Grid2LayoutFrame):ClearAllPoints():SetPoint(raidFrameConfig:GetPoint())
		end
	else
		A:GetFrameConfig(Grid2LayoutFrame):ResetAll()
	end
end

function M:UpdateButtonSize()
	if self.config.useRaidFrameConfig and self.config.controlButtonSize then
		local raidFrameConfig = A:GetRaidFrameConfig(self.GetGroupSize())

		if not raidFrameConfig:IsEqualToBarSize(Grid2Frame.db.profile.frameWidth, Grid2Frame.db.profile.frameHeight) then
			Mixin(self.Override, {
				frameWidth = raidFrameConfig.width,
				frameHeight = raidFrameConfig.height,
			})

			self:OverrideAllDatabases()
			A:RunSecure(self.UpdateButtonSize, Grid2Layout.UpdateDisplay, Grid2Layout)
		end
	end
end

function M:UpdateButtonSizeWithDelay(...)
	self:CancelTimer(self.updateButtonSizeTimer)
	self.updateButtonSizeTimer = self:ScheduleTimer("UpdateButtonSize", 0.01, ...)
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:CreateFrame(frameType, frameName, parentFrame, inheritsFrame)
	if frameName then
		if frameName == "Grid2LayoutFrame" then
			self:UpdatePanelPosition()
		elseif frameName:match("Grid2LayoutHeader%d+UnitButton%d+") then
			self:SkinButton(_G[frameName])
		end
	end
end

function M:DungeonRole_GetIcon()
	return self.DungeonRoleIcon
end

function M:DungeonRole_GetTexCoord(_, unit)
	return unpack(self.DungeonRoleTexCoord[UnitGroupRolesAssigned(unit)])
end

function M:DungeonRole_IsActive(DungeonRole, ...)
	return self.isClutterKeyDown and self.hooks[DungeonRole].IsActive(DungeonRole, ...)
end

function M:Grid2_MediaFetch(Grid2, mediaType, key, ...)
	return self.hooks[Grid2].MediaFetch(Grid2, mediaType, mediaType == "statusbar" and self.config.skinRaidFrames and self.config.overrideTextures and C.LIGHT_TEXTURE_NAME or key, ...)
end

function M:Status_GetIcon(Status, ...)
	return self.StatusIcon[self.hooks[Status].GetIcon(Status, ...)]
end

function M:Status_GetTexCoord(Status, ...)
	return unpack(self.StatusTexCoord[self.hooks[Status].GetIcon(Status, ...)])
end

function M:Summon_GetIcon()
	return self.SummonIcon
end

function M:Summon_GetTexCoord(_, unit)
	return unpack(self.SummonTexCoord[C_IncomingSummon.IncomingSummonStatus(unit)])
end

-------------------------------------------------------------------------------
-- Setup
-------------------------------------------------------------------------------

Mixin(M, C.AceSetupMixin)

function M.LoadGrid2Options()
	Grid2:OpenGrid2Options()

	if Grid2Options.optionsFrame then
		Grid2Options.optionsFrame:Hide()
	end
end

function M:CreateProfileFromString(profileName, profileString, successFn)
	local activeProfile = self:GetActiveProfile()
	self:DeleteProfile(profileName)
	self.LoadGrid2Options()
	Grid2Options:ExportText("", nil)
	MultiLineEditBox1ScrollFrame.obj:SetText(profileString)
	MultiLineEditBox1ScrollFrame.obj.button:Enable()
	MultiLineEditBox1ScrollFrame.obj.button:Click()
	self:SetActiveProfile(activeProfile)
	successFn()
end

function M:GetDatabase()
	return Grid2.db
end
