
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: coroutine, GameTimeFrame, Minimap, MinimapBackdrop, MinimapCluster, MiniMapMailIcon, next, UIParent

local MouseIsOver = MouseIsOver
local SixUI_Minimap = SixUI_Minimap
local SixUI_MinimapButtons = SixUI_MinimapButtons
local UIFrameFadeIn = UIFrameFadeIn
local UIFrameFadeOut = UIFrameFadeOut
local ZoneText = SixUI_MinimapButtons.ZoneText

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Minimap Frame")

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

M.MINIMAP_SCALE = SixUI_Minimap:GetHeight() / Minimap:GetHeight()

M.styledMinimapRelated = {}

function M.IterateMinimapRelated()
	return coroutine.wrap(function ()
		for _, frame in next, {
			GameTimeFrame,
			Minimap.ZoomIn,
			Minimap.ZoomOut,
			MinimapBackdrop,
			MinimapCluster.BorderTop,
			MinimapCluster.IndicatorFrame,
			MinimapCluster.InstanceDifficulty,
			MinimapCluster.Tracking,
			MinimapCluster.ZoneTextButton,
			MiniMapMailIcon,
		} do
			if frame then
				coroutine.yield(frame)
			end
		end
	end)
end

function M:OnInitialize()
	MinimapCluster.minimumWidth, MinimapCluster.minimumHeight = MinimapCluster.MinimapContainer:GetSize()
	MinimapCluster.maximumWidth, MinimapCluster.maximumHeight = MinimapCluster.MinimapContainer:GetSize()
	MinimapCluster:SetClampedToScreen(false)
end

function M:OnEnable()
	self:RegisterMessage("SIXUI_CONFIG_CHANGED")
	self:RegisterMessage("SIXUI_PROFILE_CHANGED")
	self:RegisterEvent("UI_SCALE_CHANGED")
	self:StyleMinimap()
end

function M:OnDisable()
	self:ResetMinimap()
end

function M:StyleMinimap()
	if self.config.skinMinimap then
		Minimap:SetMaskTexture([[Interface\ChatFrame\ChatFrameBackground]])
		self:CreateBorder(SixUI_Minimap, 2)
		self:CreateHook("HookScript", Minimap, "OnEnter", self.Minimap_OnEnter)
		self:StyleMinimapCluster(MinimapCluster)
		self:StyleMinimapRelated()
		SixUI_Minimap:SetScale(UIParent:GetScale())
		SixUI_Minimap:Show()
	end
end

function M:ResetMinimap()
	Minimap:SetMaskTexture([[Interface\Masks\CircleMaskScalable]])
	self:RemoveBorder(SixUI_Minimap)
	self:Unhook(Minimap, "OnEnter")
	self:ResetMinimapCluster()
	self:ResetMinimapRelated()
	SixUI_Minimap:SetScale(1)
	SixUI_Minimap:Hide()
end

function M:StyleMinimapCluster()
	self:CreateHook("SecureHook", MinimapCluster, "SetHeaderUnderneath", "MinimapCluster_SetHeaderUnderneath")
	MinimapCluster:SetHeaderUnderneath(true)
	MinimapCluster:SetScale(self.MINIMAP_SCALE)
end

function M:ResetMinimapCluster()
	self:Unhook(MinimapCluster, "SetHeaderUnderneath")
	MinimapCluster:UpdateSystemSettingHeaderUnderneath()
	MinimapCluster:SetScale(1)
end

function M:StyleMinimapRelated()
	for frame in self.IterateMinimapRelated() do
		A:GetFrameConfig(frame):SetShown(false)
	end
end

function M:ResetMinimapRelated()
	for frame in self.IterateMinimapRelated() do
		A:GetFrameConfig(frame):ResetAll()
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CONFIG_CHANGED(event, module, config, key, ...)
	if module == self:GetName() then
		self:ResetMinimap()
		self:StyleMinimap()
	end
end

function M:SIXUI_PROFILE_CHANGED()
	self:ResetMinimap()
	self:StyleMinimap()
end

function M:UI_SCALE_CHANGED()
	SixUI_Minimap:SetScale(UIParent:GetScale())
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M.Minimap_OnEnter()
	UIFrameFadeIn(SixUI_MinimapButtons, 0.25, SixUI_MinimapButtons:GetAlpha(), 1)
	UIFrameFadeOut(ZoneText, 0.25, ZoneText:GetAlpha(), 0)
	SixUI_Minimap:SetScript("OnUpdate", M.Minimap_OnUpdate)
end

function M.Minimap_OnUpdate()
	if not MouseIsOver(SixUI_Minimap) then
		SixUI_Minimap:SetScript("OnUpdate", nil)
		UIFrameFadeOut(SixUI_MinimapButtons, 0.25, SixUI_MinimapButtons:GetAlpha(), 0)
		UIFrameFadeIn(ZoneText, 0.25, ZoneText:GetAlpha(), 1)
	end
end

function M:MinimapCluster_SetHeaderUnderneath(frame, headerUnderneath)
	if not headerUnderneath then
		frame:SetHeaderUnderneath(true)
	end
end
