local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database

local M = A:GetModule("Core Config")

function M.GetAddonIconText(addonName)
	local iconTexture = C_AddOns.GetAddOnMetadata(addonName, "IconTexture")
	local iconAtlas = C_AddOns.GetAddOnMetadata(addonName, "IconAtlas")

	if not iconTexture and not iconAtlas then
		iconTexture = [[Interface\ICONS\INV_Misc_QuestionMark]]
	end

	if iconTexture then
		return CreateSimpleTextureMarkup(iconTexture, 20, 20)
	elseif iconAtlas then
		return CreateAtlasMarkup(iconAtlas, 20, 20)
	end
end

function M.GenerateAddOnsOptionsTable()
	local IntegrationWeakAuras = A:GetModule("Integration WeakAuras")
	local optionsTable = {
		name = "AddOns",
		type = "group",
		get = M.GetConfigAttribute,
		set = M.SetConfigAttribute,
		args = M.GenerateOrderedArgs(),
	}

	optionsTable.args.description = {
		order = 1,
		name = "Options for other AddOns.",
		type = "description",
		fontSize = "medium",
	}

	optionsTable.args["Integration AdiBags"] = {
		type = "group",
		name = M.FormatText("AdiBags", not A:IsModuleEnabledByConfig("Integration AdiBags"), true),
		hidden = not C_AddOns.IsAddOnLoaded("AdiBags"),
		args = {
			description = {
				order = 0,
				name = "Skins inventory slots.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration AdiBags"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration Bagnon"] = {
		type = "group",
		name = M.FormatText("Bagnon", not A:IsModuleEnabledByConfig("Integration Bagnon"), true),
		hidden = not C_AddOns.IsAddOnLoaded("Bagnon"),
		args = {
			description = {
				order = 0,
				name = "Skins inventory slots.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration Bagnon"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration Dominos"] = {
		type = "group",
		name = M.FormatText("Dominos", not A:IsModuleEnabledByConfig("Integration Dominos"), true),
		hidden = not C_AddOns.IsAddOnLoaded("Dominos"),
		args = {
			description = {
				order = 0,
				name = "Skins action buttons.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				order = 1,
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration Dominos"},
					},
				},
			},
			clutterKey = {
				name = "Clutter Key",
				type = "group",
				inline = true,
				args = {
					showBindingText = {
						name = "Show Binding Text",
						type = "toggle",
						width = "full",
						arg = {name = "Integration Dominos", key = "showBindingText"},
					},
					showMacroText = {
						name = "Show Macro Text",
						type = "toggle",
						width = "full",
						arg = {name = "Integration Dominos", key = "showMacroText"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration Gnosis"] = {
		type = "group",
		name = M.FormatText("Gnosis", not A:IsModuleEnabledByConfig("Integration Gnosis"), true),
		hidden = not C_AddOns.IsAddOnLoaded("Gnosis"),
		args = {
			description = {
				order = 0,
				name = "Skins cast bars. Colors cast bars by class and reaction.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				order = 1,
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration Gnosis"},
					},
				},
			},
			appearance = {
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					skinBars = {
						order = 1,
						name = "Skin Bars",
						desc = "Skin Gnosis's bars.",
						type = "toggle",
						arg = {name = "Integration Gnosis", key = "skinBars"},
					},
					useClassColors = {
						name = "Use Class Colors",
						desc = "Use class colors for Gnosis' player bars.",
						type = "toggle",
						arg = {name = "Integration Gnosis", key = "useClassColors"},
					},
					useReactionColors = {
						name = "Use Reaction Colors",
						desc = "Use the reaction colors for Gnosis' NPC bars",
						type = "toggle",
						arg = {name = "Integration Gnosis", key = "useReactionColors"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration Grid2"] = {
		type = "group",
		name = M.FormatText("Grid2", not A:IsModuleEnabledByConfig("Integration Grid2"), true),
		hidden = not C_AddOns.IsAddOnLoaded("Grid2"),
		args = {
			description = {
				order = 0,
				name = "Skins raid frames. Adjusts raid frames using SixUI's Raid Frames options.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				order = 1,
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration Grid2"},
					},
				},
			},
			appearance = {
				order = 2,
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					skinRaidFrames = {
						order = 1,
						name = "Skin Raid Frames",
						desc = "Skin Grid2's unit frames.",
						type = "toggle",
						arg = {name = "Integration Grid2", key = "skinRaidFrames"},
					},
					overrideTextures = {
						order = 2,
						name = "Override Textures",
						desc = "Override Grid2's status bar textures.",
						type = "toggle",
						disabled = function () return A:GetModuleConfig("Integration Grid2").skinRaidFrames ~= true end,
						arg = {name = "Integration Grid2", key = "overrideTextures"},
					},
					useAlternativeRaidIcons = {
						name = "Use Alt. Raid Icons",
						desc = "Use alternative raid icons for some of Grid2's statuses (raid-icon-player and raid-icon-target).",
						type = "toggle",
						arg = {name = "Integration Grid2", key = "useAlternativeRaidIcons"},
					},
					useClutterKey = {
						name = "Use Clutter Key",
						desc = "Use the clutter key to show some of Grid2's statuses (dungeon-role).",
						type = "toggle",
						arg = {name = "Integration Grid2", key = "useClutterKey"},
					},
					useHighQualityIcons = {
						name = "Use High-Quality Icons",
						desc = "Use high-quality icons for some of Grid2's statuses (dungeon-role, ready-check, and summon).",
						type = "toggle",
						arg = {name = "Integration Grid2", key = "useHighQualityIcons"},
					},
				},
			},
			layout = {
				name = "Layout",
				type = "group",
				inline = true,
				disabled = function () return A:GetModuleConfig("Integration Grid2").useRaidFrameConfig ~= true end,
				args = {
					useRaidFrameConfig = {
						order = 1,
						name = "Use Raid Frame Config",
						desc = "Use SixUI's raid frame options to adjust Grid2 as your group size changes.",
						type = "toggle",
						disabled = false,
						arg = {name = "Integration Grid2", key = "useRaidFrameConfig"},
					},
					controlButtonSize = {
						name = "Control Button Size",
						desc = "Control Grid2's button size.",
						type = "toggle",
						arg = {name = "Integration Grid2", key = "controlButtonSize"},
					},
					controlPanelPosition = {
						name = "Control Panel Position",
						desc = "Control Grid2's panel position.",
						type = "toggle",
						arg = {name = "Integration Grid2", key = "controlPanelPosition"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration ShadowedUnitFrames"] = {
		type = "group",
		name = M.FormatText("Shadowed Unit Frames", not A:IsModuleEnabledByConfig("Integration ShadowedUnitFrames"), true),
		hidden = not C_AddOns.IsAddOnLoaded("ShadowedUnitFrames"),
		args = {
			description = {
				order = 0,
				name = "Skins unit frames and auras.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				order = 1,
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration ShadowedUnitFrames"},
					},
				},
			},
			appearance = {
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					skinUnitFrames = {
						order = 1,
						name = "Skin Unit Frames",
						desc = "Skin SUF's unit frames.",
						type = "toggle",
						disabled = false,
						arg = {name = "Integration ShadowedUnitFrames", key = "skinUnitFrames"},
					},
					skinAuras = {
						order = 2,
						name = "Skin Auras",
						desc = "Skin SUF's auras.",
						type = "toggle",
						disabled = function () return A:GetModuleConfig("Integration ShadowedUnitFrames").skinUnitFrames ~= true end,
						arg = {name = "Integration ShadowedUnitFrames", key = "skinAuras"},
					},
					useClutterKey = {
						name = "Use Clutter Key",
						desc = "Use the clutter key to reload SUF's tags, ensuring any dynamic tags are updated.",
						type = "toggle",
						arg = {name = "Integration ShadowedUnitFrames", key = "useClutterKey"},
					},
					useInstantFading = {
						name = "Use Instant Fading",
						desc = "Use instant fading when SUF shows or hides elements, such as the player unit frame.",
						type = "toggle",
						arg = {name = "Integration ShadowedUnitFrames", key = "useInstantFading"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration Skada"] = {
		type = "group",
		name = M.FormatText("Skada", not A:IsModuleEnabledByConfig("Integration Skada"), true),
		hidden = not C_AddOns.IsAddOnLoaded("Skada"),
		args = {
			description = {
				order = 0,
				name = "Skins windows and controls columns.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				order = 1,
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration Skada"},
					},
				},
			},
			appearance = {
				order = 2,
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					skinWindows = {
						order = 1,
						name = "Skin Windows",
						desc = "Skin Skada's windows.",
						type = "toggle",
						arg = {name = "Integration Skada", key = "skinWindows"},
					},
					overrideTextures = {
						order = 2,
						name = "Override Textures",
						desc = "Override Skada's status bar textures.",
						type = "toggle",
						disabled = function () return A:GetModuleConfig("Integration Skada").skinWindows ~= true end,
						arg = {name = "Integration Skada", key = "overrideTextures"},
					},
					optimizeBarHeight = {
						name = "Optimize Bar Height",
						desc = "Optimize Skada's bar height to fill the window.",
						type = "toggle",
						arg = {name = "Integration Skada", key = "optimizeBarHeight"},
					},
					optimizeBarSpacing = {
						name = "Optimize Bar Spacing",
						desc = "Optimize Skada's bar spacing to be pixel perfect.",
						type = "toggle",
						arg = {name = "Integration Skada", key = "optimizeBarSpacing"},
					},
				},
			},
			columns = {
				name = "Columns",
				type = "group",
				inline = true,
				disabled = function () return A:GetModuleConfig("Integration Skada").useClutterKey ~= true end,
				args = {
					useClutterKey = {
						order = 1,
						name = "Use Clutter Key",
						desc = "Use the clutter key to toggle columns for Skada's damage and healing modules.",
						type = "toggle",
						disabled = false,
						arg = {name = "Integration Skada", key = "useClutterKey"},
					},
					percent = {
						name = "Percent",
						desc = "Control when Skada's percent columns are shown.",
						type = "select",
						style = "dropdown",
						values = C.Enum.SkadaColumnTypeLabel,
						arg = {name = "Integration Skada", key = "columnPercent"},
					},
					throughput = {
						name = "Throughput",
						desc = "Control when Skada's DPS and HPS columns are shown.",
						type = "select",
						style = "dropdown",
						values = C.Enum.SkadaColumnTypeLabel,
						arg = {name = "Integration Skada", key = "columnThroughput"},
					},
					total = {
						name = "Total",
						desc = "Control when Skada's damage and healing columns are shown.",
						type = "select",
						style = "dropdown",
						values = C.Enum.SkadaColumnTypeLabel,
						arg = {name = "Integration Skada", key = "columnTotal"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration TinyTooltip"] = {
		type = "group",
		name = M.FormatText("TinyTooltip", not A:IsModuleEnabledByConfig("Integration TinyTooltip"), true),
		hidden = not C_AddOns.IsAddOnLoaded("TinyTooltip"),
		args = {
			description = {
				order = 0,
				name = "Skins tooltips.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration TinyTooltip"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration VuhDo"] = {
		type = "group",
		name = M.FormatText("VuhDo", not A:IsModuleEnabledByConfig("Integration VuhDo"), true),
		hidden = not C_AddOns.IsAddOnLoaded("VuhDo"),
		args = {
			description = {
				order = 0,
				name = "Skins raid frames. Adjusts raid frames using SixUI's Raid Frames options.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				order = 1,
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration VuhDo"},
					},
				},
			},
			appearance = {
				name = "Appearance",
				type = "group",
				inline = true,
				disabled = function () return A:GetModuleConfig("Integration VuhDo").skinRaidFrames ~= true end,
				args = {
					skinRaidFrames = {
						order = 1,
						name = "Skin Raid Frames",
						desc = "Skin VuhDo's unit frames.",
						type = "toggle",
						disabled = false,
						arg = {name = "Integration VuhDo", key = "skinRaidFrames"},
					},
					overrideTextures = {
						name = "Override Textures",
						desc = "Override VuhDo's status bar textures.",
						type = "toggle",
						arg = {name = "Integration VuhDo", key = "overrideTextures"},
					},
					resizeDebuffs = {
						name = "Resize Debuffs",
						desc = "Resize VuhDo's debuff icons to match the size of HoT icons.",
						type = "toggle",
						arg = {name = "Integration VuhDo", key = "resizeDebuffs"},
					},
					zoomIcons = {
						name = "Zoom Icons",
						desc = "Add a zoom to VuhDo's HoT and debuff icons.",
						type = "range",
						min = 0,
						max = 1,
						step = 0.01,
						isPercent = true,
						arg = {name = "Integration VuhDo", key = "zoomIcons"},
					},
				},
			},
			layout = {
				name = "Layout",
				type = "group",
				inline = true,
				disabled = function () return A:GetModuleConfig("Integration VuhDo").useRaidFrameConfig ~= true end,
				args = {
					useRaidFrameConfig = {
						order = 1,
						name = "Use Raid Frame Config",
						desc = "Use SixUI's raid frame options to adjust VuhDo as your group size changes.",
						type = "toggle",
						disabled = false,
						arg = {name = "Integration VuhDo", key = "useRaidFrameConfig"},
					},
					controlButtonSize = {
						name = "Control Button Size",
						desc = "Control VuhDo's button size.",
						type = "toggle",
						arg = {name = "Integration VuhDo", key = "controlButtonSize"},
					},
					controlPanelPosition = {
						name = "Control Panel Position",
						desc = "Control VuhDo's panel position.",
						type = "toggle",
						arg = {name = "Integration VuhDo", key = "controlPanelPosition"},
					},
				},
			},
			roles = {
				name = "Roles",
				type = "group",
				inline = true,
				args = {
					overrideHunterRole = {
						name = "Survival Hunter",
						desc = "Override how VuhDo sorts Survival Hunters.",
						type = "select",
						style = "dropdown",
						values = {[true] = "Ranged", [false] = "Melee"},
						arg = {name = "Integration VuhDo", key = "overrideHunterRole"},
					},
					overrideHolyPaladinRole = {
						name = "Holy Paladin",
						desc = "Override how VuhDo sorts Holy Paladins.",
						type = "select",
						style = "dropdown",
						values = {[true] = "Melee", [false] = "Healer"},
						arg = {name = "Integration VuhDo", key = "overrideHolyPaladinRole"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration WeakAuras"] = {
		type = "group",
		name = M.FormatText("WeakAuras", not A:IsModuleEnabledByConfig("Integration WeakAuras"), true),
		hidden = not C_AddOns.IsAddOnLoaded("WeakAuras"),
		args = {
			description = {
				order = 0,
				name = "Skins icons and progress bars.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				order = 1,
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration WeakAuras"},
					},
				},
			},
			appearance = {
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					skinIcons = {
						name = "Skin Icons",
						desc = "Skin WeakAuras' icons.",
						type = "toggle",
						arg = {name = "Integration WeakAuras", key = "skinIcons"},
					},
					skinProgressBars = {
						name = "Skin Progress Bars",
						desc = "Skin WeakAuras' progress bars.",
						type = "toggle",
						arg = {name = "Integration WeakAuras", key = "skinProgressBars"},
					},
				},
			},
			blacklist = {
				name = "Ignore List",
				type = "group",
				inline = true,
				args = {
					new = {
						order = 1,
						name = "New",
						desc = "Add an aura to the ignore list to disable skinning for it.",
						type = "input",
						get = function () end,
						set = function (_, target) IntegrationWeakAuras:Ignore(target) end,
					},
					delete = {
						order = 2,
						name = "Delete",
						desc = "Delete an entry from the ignore list.",
						type = "select",
						style = "dropdown",
						values = function () return IntegrationWeakAuras:GetIgnoreList() end,
						get = function () end,
						set = function (_, target) IntegrationWeakAuras:Unignore(target) end,
					},
				},
			},
			importButton = {
				name = "Import SixUI's WeakAuras Pack",
				type = "execute",
				func = function () IntegrationWeakAuras:Import() end,
				width = "full",
			},
		},
	}

	return optionsTable
end
