local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database

local M = A:GetModule("Core Config")

function M.GenerateAutomationsOptionsTable()
	local AutomationInvite = A:GetModule("Automation Invite")
	local AutomationSell = A:GetModule("Automation Sell")
	local optionsTable = {
		name = "Automations",
		type = "group",
		get = M.GetConfigAttribute,
		set = M.SetConfigAttribute,
		args = M.GenerateOrderedArgs(),
	}

	optionsTable.args.description = {
		order = 1,
		name = "Options for SixUI's automations.",
		type = "description",
		fontSize = "medium",
	}

	optionsTable.args["Automation Emote"] = {
		type = "group",
		name = M.FormatText("Emote", not A:IsModuleEnabledByConfig("Automation Emote")),
		args = {
			description = {
				order = 0,
				name = "Auto-thanks the player who resurrected you.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Emote"},
					},
				},
			},
		},
	}

	optionsTable.args["Automation Interact"] = {
		type = "group",
		name = M.FormatText("Interact", not A:IsModuleEnabledByConfig("Automation Interact")),
		args = {
			description = {
				order = 0,
				name = "Auto-interacts with frames while you're busy or holding a modifier key.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Interact"},
					},
				},
			},
			interactions = {
				name = "Interactions",
				type = "group",
				inline = true,
				args = {
					acceptQuests = {
						name = "Accept Quests",
						desc = "Accept quests when they are offered.",
						type = "toggle",
						arg = {name = "Automation Interact", key = "acceptQuests"},
					},
					acceptReadyChecks = {
						name = "Accept Ready Checks",
						desc = "Accept ready checks when they are initiated.",
						type = "toggle",
						arg = {name = "Automation Interact", key = "acceptReadyChecks"},
					},
					chooseGossipOptions = {
						name = "Choose Gossip Options",
						desc = "Choose gossip (conversation) options when there is only one option or a default option.",
						type = "toggle",
						arg = {name = "Automation Interact", key = "chooseGossipOptions"},
					},
					completeQuests = {
						name = "Complete Quests",
						desc = "Complete quests when there is only one reward.",
						type = "toggle",
						arg = {name = "Automation Interact", key = "completeQuests"},
					},
				},
			},
			modifierKey = {
				name = "Modifier Key",
				type = "select",
				style = "radio",
				values = C.Enum.ModifierKeyLabel,
				get = M.GetConfigAttribute,
				set = M.SetConfigAttribute,
				arg = {name = "Automation Interact", key = "modifierKey"},
			},
		},
	}

	optionsTable.args["Automation Invite"] = {
		type = "group",
		name = M.FormatText("Invite", not A:IsModuleEnabledByConfig("Automation Invite")),
		args = {
			description = {
				order = 0,
				name = "Auto-accepts group invitations from players you know.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				order = 0,
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Invite"},
					},
				},
			},
			allow = {
				name = "Allow",
				type = "group",
				inline = true,
				args = {
					allowFriends = {
						name = "Friends",
						desc = "Auto-accept group invitations from friends.",
						type = "toggle",
						arg = {name = "Automation Invite", key = "allowFriends"},
					},
					allowClubMembers = {
						name = "Guild Members",
						desc = "Auto-accept group invitations from guild members.",
						type = "toggle",
						arg = {name = "Automation Invite", key = "allowClubMembers"},
					},
					allowLastCharacterWhispered = {
						name = "Last Character Whispered",
						desc = "Auto-accept group invitations from the last character you whispered.",
						type = "toggle",
						arg = {name = "Automation Invite", key = "allowLastCharacterWhispered"},
					},
				},
			},
			disable = {
				name = "Disable",
				type = "group",
				inline = true,
				args = {
					disableInQueue = {
						name = "In a Queue",
						desc = "Disable auto-accept while in a queue.",
						type = "toggle",
						arg = {name = "Automation Invite", key = "disableInQueue"},
					},
					disableInInstance = {
						name = "In an Instance",
						desc = "Disable auto-accept while in an instance.",
						type = "toggle",
						arg = {name = "Automation Invite", key = "disableInInstance"},
					},
				},
			},
			blacklist = {
				name = "Ignore List",
				type = "group",
				inline = true,
				args = {
					new = {
						order = 1,
						name = "New",
						desc = "Add a BattleTag, character name, or guild rank to the ignore list to disable auto-accept for them.",
						type = "input",
						get = function () end,
						set = function (_, target) AutomationInvite:Ignore(target) end,
					},
					delete = {
						order = 2,
						name = "Delete",
						desc = "Delete an entry from the ignore list.",
						type = "select",
						style = "dropdown",
						values = function () return AutomationInvite:GetIgnoreList() end,
						get = function () end,
						set = function (_, target) AutomationInvite:Unignore(target) end,
					},
				},
			},
		},
	}

	optionsTable.args["Automation Repair"] = {
		type = "group",
		name = M.FormatText("Repair", not A:IsModuleEnabledByConfig("Automation Repair")),
		args = {
			description = {
				order = 0,
				name = "Auto-repairs items when you visit a vendor.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Repair"},
					},
				},
			},
			fundSource = {
				name = "Fund Source",
				type = "group",
				inline = true,
				args = {
					useCharacterGold = {
						name = "Character Gold",
						desc = "Fund auto-repair using character gold.",
						type = "toggle",
						arg = {name = "Automation Repair", key = "useCharacterGold"},
					},
					useGuildGold = {
						name = "Guild Gold",
						desc = "Fund auto-repair using guild gold.",
						type = "toggle",
						arg = {name = "Automation Repair", key = "useGuildGold"},
					},
				},
			},
		},
	}

	optionsTable.args["Automation Sell"] = {
		type = "group",
		name = M.FormatText("Sell", not A:IsModuleEnabledByConfig("Automation Sell")),
		args = {
			description = {
				order = 0,
				name = "Auto-sells junk when you visit a vendor.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Sell"},
					},
				},
			},
			blacklist = {
				name = "Ignore List",
				type = "group",
				inline = true,
				args = {
					new = {
						order = 1,
						name = "New",
						desc = "Add an item link to the ignore list to disable auto-sell for it.",
						type = "input",
						get = function () end,
						set = function (_, target) AutomationSell:Ignore(target) end,
					},
					delete = {
						order = 2,
						name = "Delete",
						desc = "Delete an entry from the ignore list.",
						type = "select",
						style = "dropdown",
						values = function () return AutomationSell:GetIgnoreList() end,
						get = function () end,
						set = function (_, target) AutomationSell:Unignore(target) end,
					},
				},
			},
		},
	}

	optionsTable.args["Automation Watch"] = {
		type = "group",
		name = M.FormatText("Watch", not A:IsModuleEnabledByConfig("Automation Watch")),
		args = {
			description = {
				order = 0,
				name = "Auto-watches a faction when you gain or lose reputation.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Watch"},
					},
				},
			},
		},
	}

	return optionsTable
end
