local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database

local M = A:GetModule("Core Config")

function M.GenerateBlizzardOptionsTable()
	local BlizzardActionBar = A:GetModule("Blizzard ActionBar")
	local optionsTable = {
		name = "Blizzard",
		type = "group",
		get = M.GetConfigAttribute,
		set = M.SetConfigAttribute,
		args = M.GenerateOrderedArgs(),
	}

	optionsTable.args.description = {
		order = 1,
		name = "Options for Blizzard's frames.",
		type = "description",
		fontSize = "medium",
	}

	optionsTable.args.actionBars = {
		name = "Action Bars",
		type = "group",
		childGroups = "select",
		args = M.GenerateOrderedArgs(),
	}

	for bar, barConfig, label in BlizzardActionBar:IterateBarConfigs() do
		local bindingText = BlizzardActionBar.ActionBarWithBindingText[bar]
		local grid = BlizzardActionBar.ActionBarWithGrid[bar]
		local macroText = BlizzardActionBar.ActionBarWithMacroText[bar]

		optionsTable.args.actionBars.args[label] = {
			name = label,
			type = "group",
			get = M.GetConfigAttribute,
			set = M.SetConfigAttribute,
			args = {
				appearance = {
					name = "Appearance",
					type = "group",
					inline = true,
					args = {
						skinActionButtons = {
							order = 0,
							name = "Skin Action Buttons",
							type = "toggle",
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "skinActionButtons"},
						},
						disableAnimations = {
							name = "Disable Animations",
							desc = "Disable animations on your action buttons.",
							type = "toggle",
							hidden = not grid,
							arg = {name = "Blizzard ActionBar", key = "disableAnimations"},
						},
						showBindingText = {
							name = "Show Binding Text",
							desc = "Show binding text on your action buttons.",
							type = "toggle",
							disabled = not barConfig.skinActionButtons,
							hidden = not bindingText,
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "showBindingText"},
						},
						showMacroText = {
							name = "Show Macro Text",
							desc = "Show macro text on your action buttons.",
							type = "toggle",
							disabled = not barConfig.skinActionButtons,
							hidden = not macroText,
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "showMacroText"},
						},
						showTextWithClutterKey = {
							name = "Use Clutter Key",
							desc = "Use the clutter key to toggle binding and macro text.",
							type = "toggle",
							disabled = not barConfig.skinActionButtons,
							hidden = not bindingText and not macroText,
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "showTextWithClutterKey"},
						},
					},
				},
				layout = {
					name = "Layout",
					type = "group",
					inline = true,
					hidden = not grid,
					args = {
						grow = {
							name = "Grow",
							desc = "Sets the growth direction for multi-row horizontal action bars.",
							type = "select",
							style = "dropdown",
							values = {[true] = "Down", [false] = "Up"},
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "reverseRowsOnHorizontalBars"},
						},
					},
				},
				visbility = {
					name = "Visbility",
					type = "group",
					inline = true,
					args = {
						stateText = {
							name = "State Text",
							desc = "Control the action bar's visibility using macro conditionals.",
							type = "input",
							multiline = true,
							width = "full",
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "stateText"},
						},
					},
				},
			},
		}
	end

	optionsTable.args["Buff Frame"] = {
		type = "group",
		name = "Buff Frame",
		get = M.GetConfigAttribute,
		set = M.SetConfigAttribute,
		args = {
			appearance = {
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					skinBuffs = {
						order = 1,
						name = "Skin Buffs",
						type = "toggle",
						arg = {name = "Buff Frame", key = "skinBuffs"},
					},
					skinDebuffs = {
						order = 2,
						name = "Skin Debuffs",
						type = "toggle",
						arg = {name = "Buff Frame", key = "skinDebuffs"},
					},
					showBuffBars = {
						name = "Show Progress Bars",
						desc = "Show progress bars instead of text. When text is hidden, hold the clutter key to show it.",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Buff Bars"},
					},
					showCollapseAndExpandButton = {
						name = "Show Collapse Button",
						desc = "Show the collapse buffs button. When the button is hidden, hold the clutter key to show it.",
						type = "toggle",
						arg = {name = "Buff CollapseAndExpandButton", key = "showCollapseAndExpandButton"},
					},
				},
			},
			layout = {
				name = "Layout",
				type = "group",
				inline = true,
				args = {
					autoCollapseBuffs = {
						name = "Auto-Collapse Buffs",
						desc = "Auto-collapse irrelevant buffs after a loading screen.",
						type = "select",
						style = "dropdown",
						values = {
							[C.Enum.AutoCollapseBuffs.Always] = "Always",
							[C.Enum.AutoCollapseBuffs.WhileInAnInstance] = "While in an Instance",
							[C.Enum.AutoCollapseBuffs.Never] = "Never",
						},
						arg = {name = "Buff CollapseAndExpandButton", key = "autoCollapseBuffs"},
					},
				}
			}
		},
	}

	optionsTable.args["Chat Frame"] = {
		name = "Chat Frame",
		type = "group",
		get = M.GetModuleEnabled,
		set = M.SetModuleEnabled,
		args = {
			appearance = {
				order = 1,
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					skinChatFrame = {
						order = 1,
						name = "Skin Chat Frame",
						type = "toggle",
						arg = {name = "Chat Frame"},
					},
					showScrollIndicator = {
						name = "Show Scroll Indicator",
						desc = "Show a scroll indicator when the chat frame isn't scrolled to the bottom.",
						type = "toggle",
						arg = {name = "Chat ScrollIndicator"},
					},
				},
			},
			messages = {
				name = "Messages",
				type = "group",
				inline = true,
				args = {
					abbreviateMessages = {
						name = "Abbreviate Messages",
						desc = "Abbreviate messages (e.g., [2. Trade] becomes [2]).",
						type = "toggle",
						arg = {name = "Chat Abbreviations"},
					},
					filterMessages = {
						name = "Filter Messages",
						desc = "Filter messages containing AddOn spam and non-Latin characters.",
						type = "toggle",
						arg = {name = "Chat Filters"},
					},
					logMessages = {
						name = "Log Messages",
						desc = "Log messages to WoWChatLog.txt.",
						type = "toggle",
						arg = {name = "Chat Logging"},
					},
					messageSounds = {
						name = "Message Sounds",
						desc = "Play guild message and whisper sounds.",
						type = "toggle",
						arg = {name = "Chat Sounds"},
					},
					restoreMessages = {
						name = "Restore Messages",
						desc = "Restore messages when you log in or reload.",
						type = "toggle",
						arg = {name = "Chat History"},
					},
				},
			},
			shortcuts = {
				name = "Shortcuts",
				type = "group",
				inline = true,
				args = {
					altClickToInvitePlayer = {
						name = "Alt-Click to Invite Player",
						desc = "Alt-Click on a player's name to send them a group invitation.",
						type = "toggle",
						arg = {name = "Chat Invite"},
					},
					clickToCopyAddress = {
						name = "Click to Copy Address",
						desc = "Click on a BattleTag, IP address, or web address to open the copy box.",
						type = "toggle",
						arg = {name = "Chat URL"},
					},
					clickToWatchFaction = {
						name = "Click to Watch Faction",
						desc = "Click on a faction's name in a reputation message to enable its 'Show as Experience Bar' option.",
						type = "toggle",
						arg = {name = "Chat Faction"},
					},
					rightClickToCopyChat = {
						name = "Right-Click to Copy Chat",
						desc = "Right-Click on the chat frame to open the copy box.",
						type = "toggle",
						arg = {name = "Chat Copy"},
					},
					tabToCycleChannel = {
						name = "Tab to Cycle Channel",
						desc = "Press tab while typing a message to cycle between channels.",
						type = "toggle",
						arg = {name = "Chat Cycle"},
					},
				},
			},
		},
	}

	optionsTable.args["Minimap Frame"] = {
		type = "group",
		name = "Minimap",
		args = {
			appearance = {
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					skinMinimap = {
						name = "Skin Minimap",
						type = "toggle",
						arg = {name = "Minimap Frame", key = "skinMinimap"},
					},
				},
			},
		},
	}

	optionsTable.args["Blizzard ObjectiveTracker"] = {
		type = "group",
		name = "Objective Tracker",
		args = {
			appearance = {
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					showArtwork = {
						name = "Show Artwork",
						desc = "Show the Objective Tracker's artwork.",
						type = "toggle",
						arg = {name = "Blizzard ObjectiveTracker", key = "showArtwork"},
					},
					showButton = {
						name = "Show Button",
						desc = "Show the Objective Tracker button. When the button is hidden, hold the clutter key to show it.",
						type = "toggle",
						arg = {name = "Blizzard ObjectiveTracker", key = "showButton"},
					},
				},
			},
			visibility = {
				name = "Visbility",
				type = "group",
				inline = true,
				args = {
					stateText = {
						name = "State Text",
						desc = "Control the Objective Tracker's visibility using macro conditionals.",
						type = "input",
						multiline = true,
						width = "full",
						arg = {name = "Blizzard ObjectiveTracker", key = "stateText"},
					},
				},
			},
		},
	}

	optionsTable.args["Blizzard ZoneAbilityFrame"] = {
		type = "group",
		name = "Zone Ability Frame",
		args = {
			appearance = {
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					skinFrame = {
						name = "Skin Frame",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Blizzard ZoneAbilityFrame"},
					},
				},
			},
		},
	}

	return optionsTable
end
