local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database

local M = A:GetModule("Core Config")

function M.GetDisplayConfigColor(info)
	local displayConfig = info.arg.displayConfig
	local attribute = info.arg.attribute
	local r, g, b = displayConfig[attribute .. "R"], displayConfig[attribute .. "G"], displayConfig[attribute .. "B"]

	if r == 0 and g == 0 and b == 0 then
		return C.PLAYER_COLOR:GetRGB()
	else
		return r, g, b
	end
end

function M.SetDisplayConfigColor(info, r, g, b)
	local displayConfig = info.arg.displayConfig
	local attribute = info.arg.attribute
	local callback = info.arg.callback
	local keyR, keyG, keyB = attribute .. "R", attribute .. "G", attribute .. "B"

	if r == C.PLAYER_COLOR.r and g == C.PLAYER_COLOR.g and b == C.PLAYER_COLOR.b then
		displayConfig[keyR], displayConfig[keyG], displayConfig[keyB] = 0, 0, 0
	else
		displayConfig[keyR], displayConfig[keyG], displayConfig[keyB] = r, g, b
	end

	callback()

	if r == 0 and g == 0 and b == 0 then
		M:RefreshOptionsTableWithDelay("SixUI Displays")
	end
end

function M.GenerateDisplaysOptionsTable()
	local DisplayGroup = A:GetModule("Display Group")
	local groupOptions = DisplayGroup:GetGroupOptions()
	local groupSorting = DisplayGroup:GetGroupSorting()
	local optionsTable = {
		name = "Displays",
		type = "group",
		args = M.GenerateOrderedArgs(),
	}

	optionsTable.args.description = {
		order = 1,
		name = "Options for SixUI's displays.",
		type = "description",
		fontSize = "medium",
	}

	for _, groupName in next, groupSorting do
		optionsTable.args[groupName] = {
			name = groupOptions[groupName],
			type = "group",
			childGroups = "select",
			args = {},
		}

		for displayName, displayConfig, dataObject in DisplayGroup:IterateDisplayConfigs(groupName) do
			optionsTable.args[groupName].args[displayName] = {
				name = M.FormatText(displayName, not displayConfig.enabled),
				type = "group",
				args = {
					description = {
						order = 0,
						name = format("%s\n\n", dataObject.description or ""),
						type = "description",
						hidden = not dataObject.description,
						fontSize = "medium",
					},
					display = {
						order = 1,
						name = "Display",
						type = "group",
						inline = true,
						args = {
							enabled = {
								order = 1,
								name = "Enabled",
								type = "toggle",
								get = function () return displayConfig.enabled end,
								set = function (_, enabled)
									displayConfig.enabled = enabled
									DisplayGroup[enabled and "CreateDisplay" or "DeleteDisplay"](DisplayGroup, displayName, displayConfig)
									DisplayGroup:UpdateDisplayGroup(displayConfig.group)
								end,
							},
							group = {
								order = 2,
								name = "Group",
								type = "select",
								style = "dropdown",
								values = function () return groupOptions end,
								sorting = function () return groupSorting end,
								get = function () return displayConfig.group end,
								set = function (_, group)
									displayConfig.group = group
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
									DisplayGroup:UpdateDisplayGroups()
								end,
							},
							order = {
								order = 3,
								name = "Order",
								type = "range",
								min = 0,
								max = 10,
								step = 1,
								get = function () return displayConfig.order end,
								set = function (_, order)
									displayConfig.order = order
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
									DisplayGroup:UpdateDisplayGroup(displayConfig.group)
								end,
							},
							width = {
								order = 4,
								name = "Width",
								type = "range",
								min = 0,
								max = 1000,
								step = 1,
								get = function () return displayConfig.width end,
								set = function (_, width)
									displayConfig.width = width
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
									DisplayGroup:UpdateDisplayGroup(displayConfig.group)
								end,
							},
						},
					},
					attributes = {
						order = 2,
						name = "Attributes",
						type = "multiselect",
						hidden = dataObject.type == "progress bar",
						values = function () return DisplayGroup:GetAttributeOptions() end,
						get = function (_, key) return displayConfig[key] end,
						set = function (_, key, state)
							displayConfig[key] = state
							DisplayGroup:UpdateDisplay(displayName, displayConfig)
						end,
					},
					color = {
						order = 3,
						name = "Colors",
						type = "group",
						inline = true,
						hidden = not (dataObject.type == "progress bar" or displayConfig.glyph or displayConfig.label or displayConfig.text or displayConfig.value or displayConfig.suffix),
						args = {
							useCustomBarColor = {
								order = 1,
								name = "Use Custom Bar Color",
								type = "toggle",
								hidden = dataObject.type ~= "progress bar",
								get = function () return displayConfig.useCustomBarColor end,
								set = function (_, state)
									displayConfig.useCustomBarColor = state
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
							barColor = {
								order = 2,
								name = "Bar Color",
								type = "color",
								hidden = dataObject.type ~= "progress bar",
								disabled = not displayConfig.useCustomBarColor,
								get = M.GetDisplayConfigColor,
								set = M.SetDisplayConfigColor,
								arg = {displayConfig = displayConfig, attribute = "bar", callback = function () DisplayGroup:UpdateDisplay(displayName, displayConfig) end}
							},
							glyphColor = {
								order = 4,
								name = "Glyph Color",
								type = "color",
								hidden = not displayConfig.glyph,
								get = M.GetDisplayConfigColor,
								set = M.SetDisplayConfigColor,
								arg = {displayConfig = displayConfig, attribute = "glyph", callback = function () DisplayGroup:UpdateDisplay(displayName, displayConfig) end}
							},
							labelColor = {
								order = 6,
								name = "Label Color",
								type = "color",
								hidden = not displayConfig.label,
								get = M.GetDisplayConfigColor,
								set = M.SetDisplayConfigColor,
								arg = {displayConfig = displayConfig, attribute = "label", callback = function () DisplayGroup:UpdateDisplay(displayName, displayConfig) end}
							},
							textColor = {
								order = 6,
								name = "Text Color",
								type = "color",
								hidden = not displayConfig.text,
								get = M.GetDisplayConfigColor,
								set = M.SetDisplayConfigColor,
								arg = {displayConfig = displayConfig, attribute = "text", callback = function () DisplayGroup:UpdateDisplay(displayName, displayConfig) end}
							},
							valueColor = {
								order = 8,
								name = "Value Color",
								type = "color",
								hidden = not displayConfig.value,
								get = M.GetDisplayConfigColor,
								set = M.SetDisplayConfigColor,
								arg = {displayConfig = displayConfig, attribute = "value", callback = function () DisplayGroup:UpdateDisplay(displayName, displayConfig) end}
							},
							suffixColor = {
								order = 10,
								name = "Suffix Color",
								type = "color",
								hidden = not displayConfig.suffix,
								get = M.GetDisplayConfigColor,
								set = M.SetDisplayConfigColor,
								arg = {displayConfig = displayConfig, attribute = "suffix", callback = function () DisplayGroup:UpdateDisplay(displayName, displayConfig) end}
							},
						},
					},
					icon = {
						order = 4,
						name = "Icon",
						type = "group",
						inline = true,
						hidden = function () return not displayConfig.icon end,
						args = {
							iconSize = {
								order = 1,
								name = "Size",
								type = "range",
								min = 1,
								max = 30,
								step = 1,
								get = function () return displayConfig.iconSize end,
								set = function (_, iconSize)
									displayConfig.iconSize = iconSize
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
							iconZoom = {
								order = 2,
								name = "Zoom",
								type = "range",
								min = 0,
								max = 100,
								step = 1,
								get = function () return displayConfig.iconZoom end,
								set = function (_, iconZoom)
									displayConfig.iconZoom = iconZoom
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
						},
					},
					glyph = {
						order = 5,
						name = "Glyph",
						type = "group",
						inline = true,
						hidden = function () return not displayConfig.glyph end,
						args = {
							glyphOverride = {
								order = 3,
								name = "Override",
								type = "select",
								style = "dropdown",
								values = function () return DisplayGroup:GetGlyphOptions() end,
								get = function () return displayConfig.glyphOverride end,
								set = function (_, glyphOverride)
									displayConfig.glyphOverride = glyphOverride
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
						},
					},
				},
			}
		end
	end

	return optionsTable
end
