local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database

local M = A:GetModule("Core Config")

local CoreLayout = A:GetModule("Core Layout")
local CoreSetup = A:GetModule("Core Setup")
local LibSharedMedia = LibStub:GetLibrary("LibSharedMedia-3.0")

function M.GetGeneralAttribute(info, ...)
	local name = info[#info]
	local c, g, k = string.split(".", name)

	if info.type == "color" then
		local c = D.profile[c][g][k]
		return c.r, c.g, c.b, c.a
	elseif info.type == "multiselect" then
		return D.profile[c][g][k][...]
	else
		return D.profile[c][g][k]
	end
end

function M.SetGeneralAttribute(info, ...)
	local name = info[#info]
	local c, g, k = string.split(".", name)

	if info.type == "color" then
		local c = D.profile[c][g][k]
		c.r, c.g, c.b, c.a = ...
	elseif info.type == "multiselect" then
		D.profile[c][g][k][...] = select(2, ...)
	else
		D.profile[c][g][k] = ...
	end

	M:SendMessage("SIXUI_CONFIG_CHANGED", name, ...)
end

function M.GenerateGeneralOptionsTable()
	return {
		name = "General",
		type = "group",
		get = M.GetGeneralAttribute,
		set = M.SetGeneralAttribute,
		args = {
			description = {
				order = 1,
				name = format("Welcome to SixUI version %s.", C.VERSION),
				type = "description",
				fontSize = "medium",
			},
			border = {
				name = "Border",
				type = "group",
				args = {
					background = {
						order = 1,
						name = "Background",
						type = "group",
						inline = true,
						args = {
							["border.background.color"] = {
								name = "Background Color",
								type = "color",
								hasAlpha = true,
							},
							["border.background.inset"] = {
								name = "Inset",
								type = "range",
								min = 0,
								max = 10,
								step = 1,
							},
						},
					},
					stroke = {
						order = 2,
						name = "Stroke",
						type = "group",
						inline = true,
						args = {
							["border.stroke.color"] = {
								order = 1,
								name = "Stroke Color",
								type = "color",
								hasAlpha = true,
							},
							["border.stroke.anchorOffset"] = {
								order = 2,
								name = "Inset",
								type = "range",
								min = 0,
								max = 10,
								step = 1,
							},
							["border.stroke.width"] = {
								order = 3,
								name = "Width",
								type = "range",
								min = 1,
								max = 10,
								step = 1,
							},
						},
					},
					accent = {
						order = 3,
						name = "Accent",
						type = "group",
						inline = true,
						args = {
							["border.accent.color"] = {
								order = 1,
								name = "Accent Color",
								type = "color",
								hasAlpha = true,
							},
							["border.accent.anchorOffset"] = {
								order = 2,
								name = "Inset",
								type = "range",
								min = 0,
								max = 10,
								step = 1,
							},
							["border.accent.width"] = {
								order = 3,
								name = "Width",
								type = "range",
								min = 1,
								max = 10,
								step = 1,
							},
						},
					},
				},
			},
			clutter = {
				name = "Clutter",
				type = "group",
				args = {
					modifierKey = {
						name = "Modifier Key",
						type = "select",
						style = "radio",
						values = C.Enum.ModifierKeyLabel,
						get = M.GetConfigAttribute,
						set = M.SetConfigAttribute,
						arg = {name = "Miscellaneous Clutter", key = "modifierKey"},
					},
				},
			},
			color = {
				name = "Colors",
				type = "group",
				args = {
					debuff = {
						name = "Debuffs",
						type = "group",
						inline = true,
						args = {
							["color.debuff.curse"] = {
								name = "Curse",
								type = "color",
								hasAlpha = true,
							},
							["color.debuff.disease"] = {
								name = "Disease",
								type = "color",
								hasAlpha = true,
							},
							["color.debuff.magic"] = {
								name = "Magic",
								type = "color",
								hasAlpha = true,
							},
							["color.debuff.other"] = {
								name = "Other",
								type = "color",
								hasAlpha = true,
							},
							["color.debuff.poison"] = {
								name = "Poison",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					faction = {
						name = "Factions",
						type = "group",
						inline = true,
						args = {
							["color.faction.alliance"] = {
								name = "Alliance",
								type = "color",
								hasAlpha = true,
							},
							["color.faction.horde"] = {
								name = "Horde",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					rating = {
						name = "Ratings",
						type = "group",
						inline = true,
						args = {
							["color.rating.gold"] = {
								order = 1,
								name = "Gold",
								type = "color",
								hasAlpha = true,
							},
							["color.rating.silver"] = {
								order = 2,
								name = "Silver",
								type = "color",
								hasAlpha = true,
							},
							["color.rating.bronze"] = {
								order = 3,
								name = "Bronze",
								type = "color",
								hasAlpha = true,
							},
							["color.rating.untimed"] = {
								order = 4,
								name = "Untimed",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					reaction = {
						name = "Reactions",
						type = "group",
						inline = true,
						args = {
							["color.reaction.hostile"] = {
								name = "Hostile",
								type = "color",
								hasAlpha = true,
							},
							["color.reaction.neutral"] = {
								name = "Neutral",
								type = "color",
								hasAlpha = true,
							},
							["color.reaction.friendly"] = {
								name = "Friendly",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					reputation = {
						name = "Reputations",
						type = "group",
						inline = true,
						args = {
							["color.reputation.exceptionallyHostile"] = {
								order = 1,
								name = "Exceptionally Hostile",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.veryHostile"] = {
								order = 2,
								name = "Very Hostile",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.hostile"] = {
								order = 3,
								name = "Hostile",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.neutral"] = {
								order = 4,
								name = "Neutral",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.friendly"] = {
								order = 5,
								name = "Friendly",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.veryFriendly"] = {
								order = 6,
								name = "Very Friendly",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.exceptionallyFriendly"] = {
								order = 7,
								name = "Exceptionally Friendly",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.exalted"] = {
								order = 8,
								name = "Exalted",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					resource = {
						name = "Resources",
						type = "group",
						inline = true,
						args = {
							["color.resource.honor"] = {
								name = "Honor",
								type = "color",
								hasAlpha = true,
							},
							["color.resource.renown"] = {
								name = "Renown",
								type = "color",
								hasAlpha = true,
							},
							["color.resource.restedXP"] = {
								name = "Rested XP",
								type = "color",
								hasAlpha = true,
							},
							["color.resource.valor"] = {
								name = "Valor",
								type = "color",
								hasAlpha = true,
							},
							["color.resource.xp"] = {
								name = "XP",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					utility = {
						name = "Utility",
						type = "group",
						inline = true,
						args = {
							["color.utility.fallback"] = {
								name = "Fallback",
								type = "color",
								hasAlpha = true,
							},
							["color.utility.guild"] = {
								name = "Guild",
								type = "color",
								hasAlpha = true,
							},
							["color.utility.highlight"] = {
								name = "Highlight",
								type = "color",
								hasAlpha = true,
							},
						},
					},
				},
			},
			font = {
				name = "Font",
				type = "group",
				args = {
					appearance = {
						order = 1,
						name = "Appearance",
						type = "group",
						inline = true,
						args = {
							["font.appearance.name"] = {
								name = "Font",
								type = "select",
								style = "dropdown",
								dialogControl = "LSM30_Font",
								values = LibSharedMedia:HashTable(LibSharedMedia.MediaType.FONT),
							},
						},
					},
					override = {
						order = 2,
						name = "Override",
						type = "group",
						inline = true,
						args = {
							["font.appearance.overrideGameFonts"] = {
								name = "Game Fonts",
								desc = "Override game fonts with SixUI's font.",
								type = "toggle",
								width = "full",
							},
							["font.appearance.overrideSharedMediaFonts"] = {
								name = "Shared Media Fonts (Requires Reload)",
								desc = "Override shared media fonts with SixUI's font. Font changes require a reload.",
								type = "toggle",
								width = "full",
							},
						},
					},
					textFontSize = {
						order = 3,
						name = "Text Font Size",
						type = "group",
						inline = true,
						args = {
							["font.textFontSize.medium"] = {
								name = "Chat",
								desc = "Chat text font size. Default: 13.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
							["font.textFontSize.large"] = {
								name = "Heading",
								desc = "Heading text font size. Default: 14.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
							["font.textFontSize.regular"] = {
								name = "Regular",
								desc = "Regular text font size. Default: 12.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
						},
					},
					glyphFontSize = {
						order = 4,
						name = "Glyph Font Size",
						type = "group",
						inline = true,
						args = {
							["font.glyphFontSize.regular"] = {
								name = "Display",
								desc = "Display glyph font size. Default: 12.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
							["font.glyphFontSize.large"] = {
								name = "Minimap",
								desc = "Minimap glyph font size. Default: 16.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
							["font.glyphFontSize.small"] = {
								name = "Tooltip",
								desc = "Tooltip glyph font size. Default: 7.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
						},
					},
				},
			},
			layout = {
				name = "Layout",
				type = "group",
				args = {
					description = {
						order = 0,
						name = "Changing the layout in SixUI sets the active profile of other AddOns to 'SixUI [layout name]' if it exists, or 'SixUI' if not. SixUI's Action Bars and Raid Frames options are specific to each layout.",
						type = "description",
						fontSize = "medium",
					},
					activeLayout = {
						name = "Active Layout",
						type = "select",
						style = "radio",
						values = C.Enum.Layout,
						get = function () return CoreLayout:GetActiveLayout() end,
						set = function (_, layout) CoreLayout:SetActiveLayout(layout) end,
					},
				},
			},
			reset = {
				type = "group",
				name =  "Reset",
				args = {
					changeAllProfiles = {
						name = "Change All Profiles",
						type = "group",
						inline = true,
						args = {
							changeProfileDescription = {
								order = 1,
								name = "Change the active profile in all supported AddOns to SixUI's profile.",
								type = "description",
							},
							changeAllProfilesButton = {
								name = "Change All Profiles",
								type = "execute",
								confirm = function () return format("The active profile will be changed to '%s' in all supported AddOns.", CoreLayout:GetProfileName()) end,
								func = function () CoreLayout:UpdateActiveProfile(true) end,
								width = "full",
							},
						},
					},
					reapplyAllChanges = {
						name = "Reapply All Changes",
						type = "group",
						inline = true,
						args = {
							reapplyAllChangesDescription = {
								order = 1,
								name = "Reapply all changes using the setup frame.",
								type = "description",
							},
							reapplyAllChangesButton = {
								name = "Reapply All Changes",
								type = "execute",
								confirm = function () return format("The applied and ignored statuses of all changes will be reset.", CoreLayout:GetProfileName()) end,
								func = function ()
									for migration in CoreSetup.IterateMigrations() do
										migration:SetApplied(false)
										migration:SetIgnored(false)
									end
								end,
								width = "full",
							},
						},
					},
					reapplyChange = {
						name = "Reapply Change",
						type = "group",
						inline = true,
						args = {
							reapplyChangeDescription = {
								order = 1,
								name = "Reapply a specific change using the setup frame.",
								type = "description",
							},
							reapplyChangeButton = {
								name = "Reapply Change",
								type = "select",
								style = "dropdown",
								values = A.table.reduce(CoreSetup.Migrations, function (ms, m) return Mixin(ms, {[m] = M.GetMigrationLabel(m)}) end, {}),
								sorting = CoreSetup.Migrations,
								confirm = function () return format("The applied and ignored statuses of this change will be reset.", CoreLayout:GetProfileName()) end,
								get = function () end,
								set = function (_, migration)
									migration:SetApplied(false)
									migration:SetIgnored(false)
								end,
								width = "full",
							},
						},
					},
				},
			},
		},
	}
end
