local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database

local M = A:NewModule("Core Config")

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")

function M.FormatText(text, isDisabled, isAddon)
	local color = isDisabled and DISABLED_FONT_COLOR or HIGHLIGHT_FONT_COLOR
	local prefix = isAddon and (M.GetAddonIconText(text) .. " ") or ""
	return color:WrapTextInColorCode(prefix .. text)
end

function M.GenerateOptionsTable()
	local optionsTable = {
		type = "group",
		name = "SixUI",
		args = M.GenerateOrderedArgs(),
	}

	optionsTable.args.general = M.GenerateGeneralOptionsTable()
	optionsTable.args.addons = M.GenerateAddOnsOptionsTable()
	optionsTable.args.automations = M.GenerateAutomationsOptionsTable()
	optionsTable.args.blizzard = M.GenerateBlizzardOptionsTable()
	optionsTable.args.displays = M.GenerateDisplaysOptionsTable()
	optionsTable.args.raidFrames = M.GenerateRaidFramesOptionsTable()
	optionsTable.args.profiles = AceDBOptions:GetOptionsTable(D)

	return optionsTable
end

function M.GenerateOrderedArgs(order)
	return setmetatable({}, {
		__newindex = function (t, k, v)
			order = (order or 0) + 1
			v.order = order
			rawset(t, k, v)
		end,
	})
end

function M.GetConfigAttribute(info, ...)
	local name = info.arg.name
	local config = info.arg.config or A:GetModuleConfig(name)
	local key, keyR, keyG, keyB, keyA = info.arg.key, info.arg.keyR or "r", info.arg.keyG or "g", info.arg.keyB or "b", info.arg.keyA or "a"

	if info.type == "color" then
		return config[keyR], config[keyG], config[keyB], config[keyA]
	elseif info.type == "multiselect" then
		return config[key][...]
	else
		return config[key]
	end
end

function M.GetModuleEnabled(info)
	return A:IsModuleEnabledByConfig(info.arg.name)
end

function M.SetConfigAttribute(info, ...)
	local name = info.arg.name
	local config = info.arg.config or A:GetModuleConfig(name)
	local key, keyR, keyG, keyB, keyA = info.arg.key, info.arg.keyR or "r", info.arg.keyG or "g", info.arg.keyB or "b", info.arg.keyA or "a"

	if info.type == "color" then
		config[keyR], config[keyG], config[keyB], config[keyA] = ...
	elseif info.type == "multiselect" then
		config[key][...] = select(2, ...)
	else
		config[key] = ...
	end

	M:SendMessage("SIXUI_CONFIG_CHANGED", name, config, key, ...)
end

function M.SetModuleEnabled(info, state)
	A:SetModuleState(info.arg.name, state)
end

function M.ToggleOptionsFrame(forceState)
	if forceState == false then
		AceConfigDialog:Close("SixUI")
	elseif forceState or not AceConfigDialog:Close("SixUI") then
		AceConfigDialog:Open("SixUI")
	end
end

function M:OnInitialize()
	self:RegisterOptionsTables()
end

function M:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateMigrations")
	self:RegisterMessage("SIXUI_MIGRATION_UPDATE", "UpdateMigrations")
end

function M:RegisterOptionsTables()
	-- SixUI
	AceConfig:RegisterOptionsTable("SixUI", M.GenerateOptionsTable, nil)
	AceConfigDialog:SetDefaultSize("SixUI", 800, 600)
	AceConfigDialog:AddToBlizOptions("SixUI", "SixUI")

	-- SixUI Setup
	AceConfig:RegisterOptionsTable("SixUI Setup", M.GenerateSetupOptionsTable, nil)
	AceConfigDialog:SetDefaultSize("SixUI Setup", 415, 415)
end

function M:RefreshOptionsTable(appName)
	AceConfigDialog:ConfigTableChanged(nil, appName)
end

function M:RefreshOptionsTableWithDelay(...)
	self:CancelTimer(self.refreshOptionsTableTimer)
	self.refreshOptionsTableTimer = self:ScheduleTimer("RefreshOptionsTable", 0.01, ...)
end
