local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database

local M = A:GetModule("Core Config")

M.AnchorPointValues = {
	TOPLEFT = "Top Left",
	TOP = "Top",
	TOPRIGHT = "Top Right",
	LEFT = "Left",
	CENTER = "Center",
	RIGHT = "Right",
	BOTTOMLEFT = "Bottom Left",
	BOTTOM = "Bottom",
	BOTTOMRIGHT = "Bottom Right",
}

function M.GenerateRaidFramesOptionsTable()
	local optionsTable = {
		name = "Raid Frames",
		type = "group",
		get = M.GetConfigAttribute,
		set = M.SetConfigAttribute,
		args = M.GenerateOrderedArgs(),
	}

	optionsTable.args.description = {
		order = 1,
		name = "Options for SixUI's raid frames.",
		type = "description",
		fontSize = "medium",
	}

	for groupSize, raidFrameConfig, label in A:IterateRaidFrameConfigs() do
		optionsTable.args[label] = {
			name = label,
			type = "group",
			order = groupSize,
			args = {
				buttonSize = {
					order = 1,
					name = "Button Size",
					type = "group",
					inline = true,
					args = {
						width = {
							order = 1,
							name = "Width",
							type = "range",
							min = 1,
							max = 250,
							step = 1,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "width"},
						},
						height = {
							order = 2,
							name = "Height",
							type = "range",
							min = 1,
							max = 150,
							step = 1,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "height"},
						},
					},
				},
				panelPosition = {
					order = 2,
					name = "Panel Position",
					type = "group",
					inline = true,
					args = {
						point = {
							order = 1,
							name = "Panel Anchor Point",
							type = "select",
							values = M.AnchorPointValues,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "point"},
						},
						relativeTo = {
							order = 2,
							name = "Anchor Frame",
							type = "input",
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "relativeTo"},
						},
						relativePoint = {
							order = 3,
							name = "Anchor Frame Anchor Point",
							type = "select",
							values = M.AnchorPointValues,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "relativePoint"},
						},
						offsetX = {
							order = 4,
							name = "X Offset",
							type = "range",
							min = -2880,
							max = 2880,
							step = 0.1,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "offsetX"},
						},
						offsetY = {
							order = 5,
							name = "Y Offset",
							type = "range",
							min = -2880,
							max = 2880,
							step = 0.1,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "offsetY"},
						},
					},
				},
			},
		}
	end

	return optionsTable
end
