
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: HIGHLIGHT_FONT_COLOR, ToggleCharacter, XP

local BreakUpLargeNumbers = BreakUpLargeNumbers
local floor = floor
local GetXPExhaustion = GetXPExhaustion
local IsXPUserDisabled = IsXPUserDisabled
local LEVEL_GAINED = LEVEL_GAINED
local UnitXP = UnitXP
local UnitXPMax = UnitXPMax

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database
local M = A:NewModule("Display XP")
local O = LibStub("LibDataBroker-1.1"):NewDataObject("SixUI XP", {
	description = "Shows while you're below max level.",
	type = "progress bar",
	label = XP,
	level = UNKNOWN,
	text = UNKNOWN,
	value = "0",
	projectedValue = "0",
	suffix = "%",
	visible = false,
})

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M.FormatText(currentXp, xpToLevel, currentXpPercent, restedXp, restedXpPercent)
	return table.concat({
		M.FormatXpText(currentXp, xpToLevel, currentXpPercent),
		M.FormatRestedXpText(restedXp, restedXpPercent),
	}, "\n")
end

function M.FormatXpText(currentXp, xpToLevel, currentXpPercent)
	return C.XP_RESOURCE_COLOR:WrapTextInColorCode(C.XP_TEXT:format(
		BreakUpLargeNumbers(currentXp),
		BreakUpLargeNumbers(xpToLevel),
		A.FormatDecimal(currentXpPercent, 1, true)
	))
end

function M.FormatRestedXpText(restedXp, restedXpPercent)
	return restedXp and C.RESTED_XP_RESOURCE_COLOR:WrapTextInColorCode(C.RESTED_XP_TEXT:format(
		BreakUpLargeNumbers(restedXp),
		A.FormatDecimal(restedXpPercent, 1, true)
	)) or nil
end

function M:OnEnable()
	self:RegisterEvent("DISABLE_XP_GAIN", "Update")
	self:RegisterEvent("ENABLE_XP_GAIN", "Update")
	self:RegisterEvent("PLAYER_LEVEL_CHANGED", "Update")
	self:RegisterEvent("PLAYER_UPDATE_RESTING", "Update")
	self:RegisterEvent("PLAYER_XP_UPDATE", "Update")
	self:RegisterEvent("UPDATE_EXHAUSTION", "Update")
	self:RegisterEvent("UPDATE_EXPANSION_LEVEL", "Update")
	self:RegisterMessage("SIXUI_CONFIG_CHANGED", "OnConfigChanged")
	self:Update()
end

function M:OnConfigChanged(event, key, ...)
	if key == "color.resource.xp" or key == "color.resource.restedXP" then
		self:Update()
	end
end

function M:ShouldBeVisible()
	return C.PLAYER_LEVEL < C.PLAYER_MAX_LEVEL and (not IsXPUserDisabled or not IsXPUserDisabled())
end

function M:Update()
	local currentXp = UnitXP("player")
	local xpToLevel = UnitXPMax("player")
	local currentXpPercent = currentXp / xpToLevel * 100
	local restedXp = GetXPExhaustion()
	local restedXpPercent = restedXp and restedXp / xpToLevel * 100 or 0
	local color = restedXp and C.RESTED_XP_RESOURCE_COLOR or C.XP_RESOURCE_COLOR

	O.barR, O.barG, O.barB = color.r, color.g, color.b
	O.level = LEVEL_GAINED:format(C.PLAYER_LEVEL)
	O.text = self.FormatText(currentXp, xpToLevel, currentXpPercent, restedXp, restedXpPercent)
	O.value = currentXpPercent
	O.projectedValue = restedXp and min(100, currentXpPercent + restedXpPercent) or 0
	O.visible = self:ShouldBeVisible()
end

-------------------------------------------------------------------------------
-- DataObject
-------------------------------------------------------------------------------

function O.OnClick(frame, button)
	ToggleCharacter("PaperDollFrame")
end

function O.OnTooltipShow(tooltip)
	tooltip:SetText(O.label)
	tooltip:AddLine(O.text, HIGHLIGHT_FONT_COLOR:GetRGB())
	tooltip:AddLine(O.level)
end
